#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "Riostream.h"
#define PSECNtupleAna_cxx
#include "PSECNtupleAna.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <string>
#include <math.h>

void PSECNtupleAna::Loop(const char* OutHistos, const char* OutSummary, const char* OutImage)
{
//   In a ROOT session, you can do:
//      Root > .L PSECNtupleAna.C
//      Root > PSECNtupleAna t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
	
	
	//Structure of this (really) big function:
	//- declaration of means, rms
	//- declaration of fixed scale histos
	//- declaration of custom scale histos, and automatic creation by TTree::Draw
	//- Loop over events
	// * Fill histos
	// * modify means and rms
	//- final calculation of means and rms
	//- write the summary
	//- write images of histos

	//Declaration of means and RMSs
	double         mean_RunNum=0.;
	double         mean_RunTyp=0.;
	double         mean_EventNum=0.;
	double         mean_BufferSequenceN=0.;
	double         mean_RealTimeLR=0.;
	double         mean_RTimeHR=0.;
	double         mean_LrRamp=0.;
	double         mean_HrRamp=0.;
	double         mean_TAC1Tim=0.;
	double         mean_TAC2Tim=0.;
	double         mean_TAC3Tim=0.;
	double         mean_Dev1Ph=0.;
	double         mean_Dev2Ph=0.;
	double         mean_Scaler1=0.;
	double         mean_Scaler2=0.;
	double         mean_Scaler3=0.;
	double         rms_RunNum=0.;
	double         rms_RunTyp=0.;
	double         rms_EventNum=0.;
	double         rms_BufferSequenceN=0.;
	double         rms_RealTimeLR=0.;
	double         rms_RTimeHR=0.;
	double         rms_LrRamp=0.;
	double         rms_HrRamp=0.;
	double         rms_TAC1Tim=0.;
	double         rms_TAC2Tim=0.;
	double         rms_TAC3Tim=0.;
	double         rms_Dev1Ph=0.;
	double         rms_Dev2Ph=0.;
	double         rms_Scaler1=0.;
	double         rms_Scaler2=0.;
	double         rms_Scaler3=0.;

	//Idem for computed data
	double	       mean_LrRampDiff;
	double	       mean_HrRampDiff;
	double	       mean_TAC23Tim;
	double	       n_TAC23Tim;
	double	       mean_IsTAC1Fired;
	double	       mean_IsTAC2Fired;
	double	       mean_IsTAC3Fired;
	double	       mean_NTACFired;
	double	       rms_LrRampDiff;
	double	       rms_HrRampDiff;
	double	       rms_TAC23Tim;
	double	       rms_IsTAC1Fired;
	double	       rms_IsTAC2Fired;
	double	       rms_IsTAC3Fired;
	double	       rms_NTACFired;

	TFile *FileOut = new TFile(OutHistos,"RECREATE");
	
	TLegend *legend=new TLegend(0.6,0.65,0.88,0.85);
	legend->SetTextFont(72);
	legend->SetTextSize(0.03);
	
	ofstream summary (OutSummary, ios::out);

	//TCanvas *canvas = new TCanvas("canvas","nearline analysis",200,10,600,400);
	//canvas->SetGrid();
	
   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntriesFast();

   //Fixed scale histograms
   //1D histograms of all branches
   TDirectory *cdFixedHistos = FileOut->mkdir("HistosFixedScale");
   cdFixedHistos->cd();
   TDirectory *cdFixedData = cdFixedHistos->mkdir("dataFixedHistos");
   cdFixedData->cd();
   TH1F *RunNumHisto = new TH1F("RunNum","RunNum",100,0.,100.);
   TH1F *RunTypHisto = new TH1F("RunTyp","RunTyp",4,0.,3.);
   TH1F *EventNumHisto = new TH1F("EventNum","EventNum",1000,0.,100000.);
   TH1F *BufferSequenceNHisto = new TH1F("BufferSequenceN","BufferSequenceN",100,0.,1024.);
   TH1F *RealTimeLRHisto = new TH1F("RealTimeLR","RealTimeLR",100,1213e6,1215e6);
   TH1F *RTimeHRHisto = new TH1F("RTimeHR","RTimeHR",100,0.,1e6);
   TH1F *LrRampHisto = new TH1F("LrRamp","LrRamp",100,0.,1023.);
   TH1F *HrRampHisto = new TH1F("HrRamp","HrRamp",100,0.,1023.);
   TH1F *TAC1TimHisto = new TH1F("TAC1Tim","TAC1Tim",100,0.,16383.);
   TH1F *TAC2TimHisto = new TH1F("TAC2Tim","TAC2Tim",100,0.,16383.);
   TH1F *TAC3TimHisto = new TH1F("TAC3Tim","TAC3Tim",100,0.,16383.);
   TH1F *Dev1PhHisto = new TH1F("Dev1Ph","Dev1Ph",100,0.,1023.);
   TH1F *Dev2PhHisto = new TH1F("Dev2Ph","Dev2Ph",100,0.,1023.);
   TH1F *Scaler1Histo = new TH1F("Scaler1","Scaler1",100,0.,16383.);
   TH1F *Scaler2Histo = new TH1F("Scaler2","Scaler2",100,0.,16383.);
   TH1F *Scaler3Histo = new TH1F("Scaler3","Scaler3",100,0.,16383.);
   
;

   //1D histograms of computed variables
   cdFixedHistos->cd();
   TDirectory *cdFixedComputed = cdFixedHistos->mkdir("ComputedFixedHistos");
   cdFixedComputed->cd();
   TH1F *LrRampDiffHisto = new TH1F("LrRampDiff","LrRampDiff",100,0.,1024.);
   TH1F *HrRampDiffHisto = new TH1F("HrRampDiff","HrRampDiff",100,0.,1024.);
   TH1F *TAC23TimHisto = new TH1F("TAC23Tim","TAC23Tim",100,-1024.,1024.);
   TH1F *NTACFiredHisto = new TH1F("NTACFired","NTACFired",4,0.,3.);

   //2D histograms of charge vs time of flight by adc
   cdFixedHistos->cd();
   TDirectory *cdFixedOthers = cdFixedHistos->mkdir("OtherFixedHistos");
   cdFixedOthers->cd();
   TH2F *TAC1vsPh1Histo = new TH2F("TAC1vsPh1","TAC1vsPh1",100,0.,1023.,100,0.,16383.);
   TH2F *TAC1vsPh2Histo = new TH2F("TAC1vsPh2","TAC1vsPh2",100,0.,1023.,100,0.,16383.);
   TH2F *TAC2vsPh1Histo = new TH2F("TAC2vsPh1","TAC2vsPh1",100,0.,1023.,100,0.,16383.);
   TH2F *TAC3vsPh2Histo = new TH2F("TAC3vsPh2","TAC3vsPh2",100,0.,1023.,100,0.,16383.);
   
   cout << "Now root is creating some histograms setting automatically the scales" << endl;

   //Automatic scale histograms
   //1D Histograms of all branches
   TDirectory *cdCustomHistos = FileOut->mkdir("HistosCustomScale");
   cdCustomHistos->cd();
   TDirectory *cdCustomData = cdCustomHistos->mkdir("dataCustomHistos");
   cdCustomData->cd();
   fChain->Draw("RunNum>>RunNum_C");
   TH1F *RunNum_CHisto = (TH1F*)gDirectory->Get("RunNum_C");
   fChain->Draw("RunTyp>>RunTyp_C");
   TH1F *RunTyp_CHisto = (TH1F*)gDirectory->Get("RunTyp_C");
   fChain->Draw("EventNum>>EventNum_C");
   TH1F *EventNum_CHisto = (TH1F*)gDirectory->Get("EventNum_C");
   fChain->Draw("BufferSequenceN>>BufferSequenceN_C");
   TH1F *BufferSequenceN_CHisto = (TH1F*)gDirectory->Get("BufferSequenceN_C");
   fChain->Draw("RealTimeLR>>RealTimeLR_C");
   TH1F *RealTimeLR_CHisto = (TH1F*)gDirectory->Get("RealTimeLR_C");
   fChain->Draw("RTimeHR>>RTimeHR_C");
   TH1F *RTimeHR_CHisto = (TH1F*)gDirectory->Get("RTimeHR_C");
   fChain->Draw("LrRamp>>LrRamp_C");
   TH1F *LrRamp_CHisto = (TH1F*)gDirectory->Get("LrRamp_C");
   fChain->Draw("HrRamp>>HrRamp_C");
   TH1F *HrRamp_CHisto = (TH1F*)gDirectory->Get("HrRamp_C");
   fChain->Draw("TAC1Tim>>TAC1Tim_C");
   TH1F *TAC1Tim_CHisto = (TH1F*)gDirectory->Get("TAC1Tim_C");
   fChain->Draw("TAC2Tim>>TAC2Tim_C");
   TH1F *TAC2Tim_CHisto = (TH1F*)gDirectory->Get("TAC2Tim_C");
   fChain->Draw("TAC3Tim>>TAC3Tim_C");
   TH1F *TAC3Tim_CHisto = (TH1F*)gDirectory->Get("TAC3Tim_C");
   fChain->Draw("Dev1Ph>>Dev1Ph_C");
   TH1F *Dev1Ph_CHisto = (TH1F*)gDirectory->Get("Dev1Ph_C");
   fChain->Draw("Dev2Ph>>Dev2Ph_C");
   TH1F *Dev2Ph_CHisto = (TH1F*)gDirectory->Get("Dev2Ph_C");
   fChain->Draw("Scaler1>>Scaler1_C");
   TH1F *Scaler1_CHisto = (TH1F*)gDirectory->Get("Scaler1_C");
   fChain->Draw("Scaler2>>Scaler2_C");
   TH1F *Scaler2_CHisto = (TH1F*)gDirectory->Get("Scaler2_C");
   fChain->Draw("Scaler3>>Scaler3_C");
   TH1F *Scaler3_CHisto = (TH1F*)gDirectory->Get("Scaler3_C");
   
   ;

   //1D histograms of computed variables
   cdCustomHistos->cd();
   TDirectory *cdCustomComputed = cdCustomHistos->mkdir("ComputedCustomHistos");
   cdCustomComputed->cd();
   fChain->Draw("LrRampDiff>>LrRampDiff_C");
   TH1F *LrRampDiff_CHisto = (TH1F*)gDirectory->Get("LrRampDiff_C");
   fChain->Draw("HrRampDiff>>HrRampDiff_C");
   TH1F *HrRampDiff_CHisto = (TH1F*)gDirectory->Get("HrRampDiff_C");
   fChain->Draw("TAC23Tim>>TAC23Tim_C");
   TH1F *TAC23Tim_CHisto = (TH1F*)gDirectory->Get("TAC23Tim_C");
   fChain->Draw("NTACFired>>NTACFired_C");
   TH1F *NTACFired_CHisto = (TH1F*)gDirectory->Get("NTACFired_C");

   //2D histograms of charge vs time of flight by adc
   cdCustomHistos->cd();
   TDirectory *cdCustomOthers = cdCustomHistos->mkdir("OtherCustomHistos");
   cdCustomOthers->cd();
   fChain->Draw("TAC1Tim:Dev1Ph>>TAC1vsPh1_C");
   TH2F *TAC1vsPh1_CHisto = (TH2F*)gDirectory->Get("TAC1vsPh1_C");
   fChain->Draw("TAC1Tim:Dev2Ph>>TAC1vsPh2_C");
   TH2F *TAC1vsPh2_CHisto = (TH2F*)gDirectory->Get("TAC1vsPh2_C");
   fChain->Draw("TAC2Tim:Dev1Ph>>TAC2vsPh1_C");
   TH2F *TAC2vsPh1_CHisto = (TH2F*)gDirectory->Get("TAC2vsPh1_C");
   fChain->Draw("TAC3Tim:Dev2Ph>>TAC3vsPh2_C");
   TH2F *TAC3vsPh2_CHisto = (TH2F*)gDirectory->Get("TAC3vsPh2_C");
   
   //2D histogram of TAC1Tim vs time
   fChain->Draw("LrRamp:HrRamp>>RampVsTime_C(1000)");
   TH2F *RampVsTime_CHisto = (TH2F*)gDirectory->Get("RampVsTime_C");

   //2D histograms of all variables vs unique event number
   TDirectory *cdEventsHistos = FileOut->mkdir("HistosVsEvents");
   cdEventsHistos->cd();
   TDirectory *cdEventsData = cdEventsHistos->mkdir("dataVsEventsHistos");
   cdEventsData->cd();
   fChain->Draw("RunTyp:UniqueEventNum>>RunTyp_VE");
   TH2F *RunTyp_VEHisto = (TH2F*)gDirectory->Get("RunTyp_VE");
   fChain->Draw("RunNum:UniqueEventNum>>RunNum_VE");
   TH2F *RunNum_VEHisto = (TH2F*)gDirectory->Get("RunNum_VE");
   fChain->Draw("EventNum:UniqueEventNum>>EventNum_VE");
   TH2F *EventNum_VEHisto = (TH2F*)gDirectory->Get("EventNum_VE");
   fChain->Draw("BufferSequenceN:UniqueEventNum>>BufferSequenceN_VE");
   TH2F *BufferSequenceN_VEHisto = (TH2F*)gDirectory->Get("BufferSequenceN_VE");
   fChain->Draw("RealTimeLR:UniqueEventNum>>RealTimeLR_VE");
   TH2F *RealTimeLR_VEHisto = (TH2F*)gDirectory->Get("RealTimeLR_VE");
   fChain->Draw("RTimeHR:UniqueEventNum>>RTimeHR_VE");
   TH2F *RTimeHR_VEHisto = (TH2F*)gDirectory->Get("RTimeHR_VE");
   fChain->Draw("LrRamp:UniqueEventNum>>LrRamp_VE");
   TH2F *LrRamp_VEHisto = (TH2F*)gDirectory->Get("LrRamp_VE");
   fChain->Draw("HrRamp:UniqueEventNum>>HrRamp_VE");
   TH2F *HrRamp_VEHisto = (TH2F*)gDirectory->Get("HrRamp_VE");
   fChain->Draw("TAC1Tim:UniqueEventNum>>TAC1Tim_VE");
   TH2F *TAC1Tim_VEHisto = (TH2F*)gDirectory->Get("TAC1Tim_VE");
   fChain->Draw("TAC2Tim:UniqueEventNum>>TAC2Tim_VE");
   TH2F *TAC2Tim_VEHisto = (TH2F*)gDirectory->Get("TAC2Tim_VE");
   fChain->Draw("TAC3Tim:UniqueEventNum>>TAC3Tim_VE");
   TH2F *TAC3Tim_VEHisto = (TH2F*)gDirectory->Get("TAC3Tim_VE");
   fChain->Draw("Dev1Ph:UniqueEventNum>>Dev1Ph_VE");
   TH2F *Dev1Ph_VEHisto = (TH2F*)gDirectory->Get("Dev1Ph_VE");
   fChain->Draw("Dev2Ph:UniqueEventNum>>Dev2Ph_VE");
   TH2F *Dev2Ph_VEHisto = (TH2F*)gDirectory->Get("Dev2Ph_VE");
   fChain->Draw("Scaler1:UniqueEventNum>>Scaler1_VE");
   TH2F *Scaler1_VEHisto = (TH2F*)gDirectory->Get("Scaler1_VE");
   fChain->Draw("Scaler2:UniqueEventNum>>Scaler2_VE");
   TH2F *Scaler2_VEHisto = (TH2F*)gDirectory->Get("Scaler2_VE");
   fChain->Draw("Scaler3:UniqueEventNum>>Scaler3_VE");
   TH2F *Scaler3_VEHisto = (TH2F*)gDirectory->Get("Scaler3_VE");
   
   cdEventsHistos->cd();
   TDirectory *cdEventsComputed = cdEventsHistos->mkdir("ComputedVsEventsHistos");
   cdEventsComputed->cd();
   fChain->Draw("LrRampDiff:UniqueEventNum>>LrRampDiff_VE");
   TH2F *LrRampDiff_VEHisto = (TH2F*)gDirectory->Get("LrRampDiff_VE");
   fChain->Draw("HrRampDiff:UniqueEventNum>>HrRampDiff_VE");
   TH2F *HrRampDiff_VEHisto = (TH2F*)gDirectory->Get("HrRampDiff_VE");
   fChain->Draw("TAC23Tim:UniqueEventNum>>TAC23Tim_VE");
   TH2F *TAC23Tim_VEHisto = (TH2F*)gDirectory->Get("TAC23Tim_VE");
   fChain->Draw("NTACFired:UniqueEventNum>>NTACFired_VE");
   TH2F *NTACFired_VEHisto = (TH2F*)gDirectory->Get("NTACFired_VE");
   
   cdFixedHistos->cd();

   Long64_t nbytes = 0, nb = 0;
   
   float RunNum0 = -999.;

   //***************************************************************************//
   //					LOOP					//
   //***************************************************************************//
   cout << "Looping over events" << endl;
   for (Long64_t jentry=0; jentry<nentries;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
	fChain->Fill();
      
      if (RunNum0 == -999.) RunNum0 = data_RunNum;
      



      //Calculate mean and rms of each leaf
      mean_RunNum += data_RunNum;
      mean_RunTyp += data_RunTyp;
      mean_EventNum += data_EventNum;
      mean_BufferSequenceN += data_BufferSequenceN;
      mean_RealTimeLR += data_RealTimeLR;
      mean_RTimeHR += data_RTimeHR;
      mean_LrRamp += data_LrRamp;
      mean_HrRamp += data_HrRamp;
      mean_TAC1Tim += data_TAC1Tim;
      mean_TAC2Tim += data_TAC2Tim;
      mean_TAC3Tim += data_TAC3Tim;
      mean_Dev1Ph += data_Dev1Ph;
      mean_Dev2Ph += data_Dev2Ph;
      mean_Scaler1 += data_Scaler1;
      mean_Scaler2 += data_Scaler2;
      mean_Scaler3 += data_Scaler3;
       rms_RunNum += data_RunNum*data_RunNum;
       rms_RunTyp += data_RunTyp*data_RunTyp;
       rms_EventNum += data_EventNum*data_EventNum;
       rms_BufferSequenceN += data_BufferSequenceN*data_BufferSequenceN;
       rms_RealTimeLR += data_RealTimeLR*data_RealTimeLR;
       rms_RTimeHR += data_RTimeHR*data_RTimeHR;
       rms_LrRamp += data_LrRamp*data_LrRamp;
       rms_HrRamp += data_HrRamp*data_HrRamp;
       rms_TAC1Tim += data_TAC1Tim*data_TAC1Tim;
       rms_TAC2Tim += data_TAC2Tim*data_TAC2Tim;
       rms_TAC3Tim += data_TAC3Tim*data_TAC3Tim;
       rms_Dev1Ph += data_Dev1Ph*data_Dev1Ph;
       rms_Dev2Ph += data_Dev2Ph*data_Dev2Ph;
       rms_Scaler1 += data_Scaler1*data_Scaler1;
       rms_Scaler2 += data_Scaler2*data_Scaler2;
       rms_Scaler3 += data_Scaler3*data_Scaler3;

	//Calculate mean and RMS for computed data
	mean_LrRampDiff += computed_LrRampDiff;
	mean_HrRampDiff += computed_HrRampDiff;
	mean_IsTAC1Fired += computed_IsTAC1Fired;
	mean_IsTAC2Fired += computed_IsTAC2Fired;
	mean_IsTAC3Fired += computed_IsTAC3Fired;
	mean_NTACFired += computed_NTACFired;
	rms_LrRampDiff += computed_LrRampDiff*computed_LrRampDiff;
	rms_HrRampDiff += computed_HrRampDiff*computed_HrRampDiff;
	rms_IsTAC1Fired += computed_IsTAC1Fired*computed_IsTAC1Fired;
	rms_IsTAC2Fired += computed_IsTAC2Fired*computed_IsTAC2Fired;
	rms_IsTAC3Fired += computed_IsTAC3Fired*computed_IsTAC3Fired;
	rms_NTACFired += computed_NTACFired*computed_NTACFired;
	if((int) computed_IsTAC2Fired && (int) computed_IsTAC3Fired){
		n_TAC23Tim += 1.;
		mean_TAC23Tim += computed_TAC23Tim;
		rms_TAC23Tim += computed_TAC23Tim*computed_TAC23Tim;
	}
      
      //Fill Histos
      RunNumHisto->Fill(data_RunNum);
      RunTypHisto->Fill(data_RunTyp);
      EventNumHisto->Fill(data_EventNum);
      BufferSequenceNHisto->Fill(data_BufferSequenceN);
      RealTimeLRHisto->Fill(data_RealTimeLR);
      RTimeHRHisto->Fill(data_RTimeHR);
      LrRampHisto->Fill(data_LrRamp);
      HrRampHisto->Fill(data_HrRamp);
      TAC1TimHisto->Fill(data_TAC1Tim);
      TAC2TimHisto->Fill(data_TAC2Tim);
      TAC3TimHisto->Fill(data_TAC3Tim);
      Dev1PhHisto->Fill(data_Dev1Ph);
      Dev2PhHisto->Fill(data_Dev2Ph);
      Scaler1Histo->Fill(data_Scaler1);
      Scaler2Histo->Fill(data_Scaler2);
      Scaler3Histo->Fill(data_Scaler3);

      LrRampDiffHisto->Fill(computed_LrRampDiff);
      HrRampDiffHisto->Fill(computed_HrRampDiff);
      TAC23TimHisto->Fill(computed_TAC23Tim);
      NTACFiredHisto->Fill(computed_NTACFired);

   }
   cout << "End of loop" << endl;
   //***************************************************************************//
   //				END OF LOOP					//
   //***************************************************************************//
   
   //Means and RMSs
   mean_RunNum=mean_RunNum/nentries;
   mean_RunTyp=   mean_RunTyp/nentries;
   mean_EventNum=   mean_EventNum/nentries;
   mean_BufferSequenceN=   mean_BufferSequenceN/nentries;
   mean_RealTimeLR=   mean_RealTimeLR/nentries;
   mean_RTimeHR=   mean_RTimeHR/nentries;
   mean_LrRamp=   mean_LrRamp/nentries;
   mean_HrRamp=   mean_HrRamp/nentries;
   mean_TAC1Tim=   mean_TAC1Tim/nentries;
   mean_TAC2Tim=   mean_TAC2Tim/nentries;
   mean_TAC3Tim=   mean_TAC3Tim/nentries;
   mean_Dev1Ph=   mean_Dev1Ph/nentries;
   mean_Dev2Ph=   mean_Dev2Ph/nentries;
   mean_Scaler1=   mean_Scaler1/nentries;
   mean_Scaler2=   mean_Scaler2/nentries;
   mean_Scaler3=   mean_Scaler3/nentries;

   rms_RunNum=sqrt( (double)rms_RunNum/nentries -mean_RunNum*mean_RunNum);
   rms_RunTyp=sqrt( (double)rms_RunTyp/nentries -mean_RunTyp*mean_RunTyp);
   rms_EventNum=sqrt( (double)rms_EventNum/nentries -mean_EventNum*mean_EventNum);
   rms_BufferSequenceN=sqrt( (double)rms_BufferSequenceN/nentries -mean_BufferSequenceN*mean_BufferSequenceN);
   rms_RealTimeLR=sqrt( (double)rms_RealTimeLR/nentries -mean_RealTimeLR*mean_RealTimeLR);
   rms_RTimeHR=sqrt( (double)rms_RTimeHR/nentries -mean_RTimeHR*mean_RTimeHR);
   rms_LrRamp=sqrt( (double)rms_LrRamp/nentries -mean_LrRamp*mean_LrRamp);
   rms_HrRamp=sqrt( (double)rms_HrRamp/nentries -mean_HrRamp*mean_HrRamp);
   rms_TAC1Tim=sqrt( (double)rms_TAC1Tim/nentries -mean_TAC1Tim*mean_TAC1Tim);
   rms_TAC2Tim=sqrt( (double)rms_TAC2Tim/nentries -mean_TAC2Tim*mean_TAC2Tim);
   rms_TAC3Tim=sqrt( (double)rms_TAC3Tim/nentries -mean_TAC3Tim*mean_TAC3Tim);
   rms_Dev1Ph=sqrt( (double)rms_Dev1Ph/nentries -mean_Dev1Ph*mean_Dev1Ph);
   rms_Dev2Ph=sqrt( (double)rms_Dev2Ph/nentries -mean_Dev2Ph*mean_Dev2Ph);
   rms_Scaler1=sqrt( (double)rms_Scaler1/nentries -mean_Scaler1*mean_Scaler1);
   rms_Scaler2=sqrt( (double)rms_Scaler2/nentries -mean_Scaler2*mean_Scaler2);
   rms_Scaler3=sqrt( (double)rms_Scaler3/nentries -mean_Scaler3*mean_Scaler3);
   


   //idem for computed data
   mean_LrRampDiff= mean_LrRampDiff/nentries;
   mean_HrRampDiff= mean_HrRampDiff/nentries;
   mean_TAC23Tim= mean_TAC23Tim/n_TAC23Tim;
   mean_IsTAC1Fired= mean_IsTAC1Fired/nentries;
   mean_IsTAC2Fired= mean_IsTAC2Fired/nentries;
   mean_IsTAC3Fired= mean_IsTAC3Fired/nentries;
   mean_NTACFired= mean_NTACFired/nentries;

   rms_LrRampDiff=sqrt( (double)rms_LrRampDiff/nentries -mean_LrRampDiff*mean_LrRampDiff);
   rms_HrRampDiff=sqrt( (double)rms_HrRampDiff/nentries -mean_HrRampDiff*mean_HrRampDiff);
   rms_TAC23Tim=sqrt( (double)rms_TAC23Tim/n_TAC23Tim -mean_TAC23Tim*mean_TAC23Tim);
   rms_IsTAC1Fired=sqrt( (double)rms_IsTAC1Fired/nentries -mean_IsTAC1Fired*mean_IsTAC1Fired);
   rms_IsTAC2Fired=sqrt( (double)rms_IsTAC2Fired/nentries -mean_IsTAC2Fired*mean_IsTAC2Fired);
   rms_IsTAC3Fired=sqrt( (double)rms_IsTAC3Fired/nentries -mean_IsTAC3Fired*mean_IsTAC3Fired);
   rms_NTACFired=sqrt( (double)rms_NTACFired/nentries -mean_NTACFired*mean_NTACFired);

   summary << "Summary of the run " << data_RunNum << endl;
   if (data_RunNum != RunNum0) summary << "Warning, more than one run was analysed (first was " << RunNum0 << "). Following statistics concern all data analyzed." << endl;
   summary << "Number of entries: " << nentries << endl;
   summary << endl << endl;
   summary << "Means and RMSs of data from input ntuple" << endl;
   summary << "mean_RunNum = " << mean_RunNum /*<< " +/- " << mean_RunNum/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RunTyp = " << mean_RunTyp /*<< " +/- " << mean_RunTyp/sqrt( (double)nentries)*/ << endl;
   summary << "mean_EventNum = " << mean_EventNum /*<< " +/- " << mean_EventNum/sqrt( (double)nentries)*/ << endl;
   summary << "mean_BufferSequenceN = " << mean_BufferSequenceN /*<< " +/- " << mean_BufferSequenceN/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RealTimeLR = " << mean_RealTimeLR /*<< " +/- " << mean_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RTimeHR = " << mean_RTimeHR /*<< " +/- " << mean_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "mean_LrRamp = " << mean_LrRamp /*<< " +/- " << mean_LrRamp/sqrt( (double)nentries)*/ << endl;
   summary << "mean_HrRamp = " << mean_HrRamp /*<< " +/- " << mean_HrRamp/sqrt( (double)nentries)*/ << endl;
   summary << "mean_TAC1Tim = " << mean_TAC1Tim << " +/- " << rms_TAC1Tim/sqrt( (double)nentries) << endl;
   summary << "mean_TAC2Tim = " << mean_TAC2Tim << " +/- " << rms_TAC2Tim/sqrt( (double)nentries) << endl;
   summary << "mean_TAC3Tim = " << mean_TAC3Tim << " +/- " << rms_TAC3Tim/sqrt( (double)nentries) << endl;
   summary << "mean_Dev1Ph = " << mean_Dev1Ph << " +/- " << rms_Dev1Ph/sqrt( (double)nentries) << endl;
   summary << "mean_Dev2Ph = " << mean_Dev2Ph << " +/- " << rms_Dev2Ph/sqrt( (double)nentries) << endl;
   summary << "mean_Scaler1 = " << mean_Scaler1 << " +/- " << rms_Scaler1/sqrt( (double)nentries) << endl;
   summary << "mean_Scaler2 = " << mean_Scaler2 << " +/- " << rms_Scaler2/sqrt( (double)nentries) << endl;
   summary << "mean_Scaler3 = " << mean_Scaler3 << " +/- " << rms_Scaler3/sqrt( (double)nentries) << endl;
   summary << "**************************************" << endl;
   summary << "rms_RunNum = " << rms_RunNum << endl;
   summary << "rms_RunTyp = " << rms_RunTyp << endl;
   summary << "rms_EventNum = " << rms_EventNum << endl;
   summary << "rms_BufferSequenceN = " << rms_BufferSequenceN << endl;
   summary << "rms_RealTimeLR = " << rms_RealTimeLR << endl;
   summary << "rms_RTimeHR = " << rms_RTimeHR << endl;
   summary << "rms_LrRamp = " << rms_LrRamp << endl;
   summary << "rms_HrRamp = " << rms_HrRamp << endl;
   summary << "rms_TAC1Tim = " << rms_TAC1Tim << endl;
   summary << "rms_TAC2Tim = " << rms_TAC2Tim << endl;
   summary << "rms_TAC3Tim = " << rms_TAC3Tim << endl;
   summary << "rms_Dev1Ph = " << rms_Dev1Ph << endl;
   summary << "rms_Dev2Ph = " << rms_Dev2Ph << endl;
   summary << "rms_Scaler1 = " << rms_Scaler1 << endl;
   summary << "rms_Scaler2 = " << rms_Scaler2 << endl;
   summary << "rms_Scaler3 = " << rms_Scaler3 << endl;
   summary << endl << "**************************************" << endl << endl;
   summary << "Means and RMSs from computed data" << endl;
   summary << "mean_LrRampDiff = " << mean_LrRampDiff << " +/- " << rms_LrRampDiff/sqrt( (double)nentries) << endl;
   summary << "mean_HrRampDiff = " << mean_HrRampDiff << " +/- " << rms_HrRampDiff/sqrt( (double)nentries) << endl;
   summary << "mean_TAC23Tim = " << mean_TAC23Tim << " +/- " << rms_TAC23Tim/sqrt( (double)n_TAC23Tim) << " counting " << n_TAC23Tim << " entries with both TACss 2 and 3 fired" << endl;
   summary << "mean_IsTAC1Fired = " << mean_IsTAC1Fired << " +/- " << rms_IsTAC1Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsTAC2Fired = " << mean_IsTAC2Fired << " +/- " << rms_IsTAC2Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsTAC3Fired = " << mean_IsTAC3Fired << " +/- " << rms_IsTAC3Fired/sqrt( (double)nentries) << endl;
   summary << "mean_NTACFired = " << mean_NTACFired << " +/- " << rms_NTACFired/sqrt( (double)nentries) << endl;
   summary << "**************************************" << endl;
   summary << "rms_LrRampDiff = " << rms_LrRampDiff << endl;
   summary << "rms_HrRampDiff = " << rms_HrRampDiff << endl;
   summary << "rms_TAC23Tim = " << rms_TAC23Tim << endl;
   summary << "rms_IsTAC1Fired = " << rms_IsTAC1Fired << endl;
   summary << "rms_IsTAC2Fired = " << rms_IsTAC2Fired << endl;
   summary << "rms_IsTAC3Fired = " << rms_IsTAC3Fired << endl;
   summary << "rms_NTACFired = " << rms_NTACFired << endl;

/*   //Write histos
   cdFixedData->cd();
   RunNumHisto->Write();
   RunTypHisto->Write();
   EventNumHisto->Write();
   BufferSequenceNHisto->Write();
   RealTimeLRHisto->Write();
   RTimeHRHisto->Write();
   LrRampHisto->Write();
   HrRampHisto->Write();
   TAC1TimHisto->Write();
   TAC2TimHisto->Write();
   TAC3TimHisto->Write();
   Dev1PhHisto->Write();
   Dev2PhHisto->Write();
   Scaler1Histo->Write();
   Scaler2Histo->Write();
   Scaler3Histo->Write();
   nScaler1Histo->Write();
   nScaler2Histo->Write();
   nScaler3Histo->Write();
   CherenkovLatchADCHisto->Write();

   cdFixedComputed->cd();
   Mcp12TimeAdcHisto->Write();
   Mcp34TimeAdcHisto->Write();
   McpTofHisto->Write();
   
   cdFixedOthers->cd();
   PhVsTimeAdc1Histo->Write();
   PhVsTimeAdc2Histo->Write();
   PhVsTimeAdc3Histo->Write();
   PhVsTimeAdc4Histo->Write();
   BufferSeqHisto->Write();
   BufferStartHRHisto->Write();
   BufferEndHRHisto->Write();
   BufferDelayHisto->Write();
   EventNumVsTimeHisto->Write();
*/
   
   //Write histos into image files
   
   //create directories
   string OutImageStr = OutImage;
   string ImageName;
   string cmd = "mkdir -p " + OutImageStr + "FixedHistos/DataFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/ComputedFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/OtherFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/DataCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/ComputedCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/OtherCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/DataEventsHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/ComputedEventsHistos";
   system(cmd.c_str());
   
   TCanvas *c1 = new TCanvas("c1","c1",1);
   
/*   //Fixed scale histos
   BufferSeqHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferSeqHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferStartHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferEndHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferDelayHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferDelayHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumVsTimeHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/EventNumVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
  */ 

   cout << "Now creating .gif image files for histograms" << endl;

   RunNumHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunNumHisto.gif";
   c1->Print(ImageName.c_str());
   RunTypHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunTypHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/EventNumHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferSequenceNHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RTimeHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRampHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/LrRampHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/HrRampHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   TAC1TimHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/TAC1TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC2TimHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/TAC2TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC3TimHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/TAC3TimHisto.gif";
   c1->Print(ImageName.c_str());
   Dev1PhHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Dev1PhHisto.gif";
   c1->Print(ImageName.c_str());
   Dev2PhHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Dev2PhHisto.gif";
   c1->Print(ImageName.c_str());
   Scaler1Histo->Draw();
   c1->SetLogy(0);
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Scaler1Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Scaler2Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler3Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Scaler3Histo.gif";
   c1->Print(ImageName.c_str());
   
   LrRampDiffHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/LrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampDiffHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/HrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   TAC23TimHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/TAC23TimHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   NTACFiredHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/NTACFiredHisto.gif";
   c1->Print(ImageName.c_str());
   
   TAC1vsPh1Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/TAC1vsPh1Histo.gif";
   c1->Print(ImageName.c_str());
   TAC1vsPh2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/TAC1vsPh2Histo.gif";
   c1->Print(ImageName.c_str());
   TAC2vsPh1Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/TAC2vsPh1Histo.gif";
   c1->Print(ImageName.c_str());
   TAC3vsPh2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/TAC3vsPh2Histo.gif";
   c1->Print(ImageName.c_str());
   
      //Custom scale histos
   
   RunNum_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunNumHisto.gif";
   c1->Print(ImageName.c_str());
   RunTyp_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunTypHisto.gif";
   c1->Print(ImageName.c_str());
   EventNum_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/EventNumHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceN_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferSequenceNHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RTimeHR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRamp_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/LrRampHisto.gif";
   c1->Print(ImageName.c_str());
   HrRamp_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/HrRampHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   TAC1Tim_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/TAC1TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC2Tim_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/TAC2TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC3Tim_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/TAC3TimHisto.gif";
   c1->Print(ImageName.c_str());
   Dev1Ph_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Dev1PhHisto.gif";
   c1->Print(ImageName.c_str());
   Dev2Ph_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Dev2PhHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   Scaler1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Scaler1Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Scaler2Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler3_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Scaler3Histo.gif";
   c1->Print(ImageName.c_str());
   
   LrRampDiff_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/LrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampDiff_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/HrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   TAC23Tim_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/TAC23TimHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   NTACFired_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/NTACFiredHisto.gif";
   c1->Print(ImageName.c_str());
   
   TAC1vsPh1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/TAC1vsPh1Histo.gif";
   c1->Print(ImageName.c_str());
   TAC1vsPh2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/TAC1vsPh2Histo.gif";
   c1->Print(ImageName.c_str());
   TAC2vsPh1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/TAC2vsPh1Histo.gif";
   c1->Print(ImageName.c_str());
   TAC3vsPh2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/TAC3vsPh2Histo.gif";
   c1->Print(ImageName.c_str());
   RampVsTime_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/RampVsTimeHisto.gif";
   c1->Print(ImageName.c_str());


      //Vs EventNum histos
   
   RunTyp_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunTypHisto.gif";
   c1->Print(ImageName.c_str());
   RunNum_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunNumHisto.gif";
   c1->Print(ImageName.c_str());
   EventNum_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/EventNumHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceN_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferSequenceNHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RTimeHR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRamp_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/LrRampHisto.gif";
   c1->Print(ImageName.c_str());
   HrRamp_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/HrRampHisto.gif";
   c1->Print(ImageName.c_str());
   TAC1Tim_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/TAC1TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC2Tim_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/TAC2TimHisto.gif";
   c1->Print(ImageName.c_str());
   TAC3Tim_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/TAC3TimHisto.gif";
   c1->Print(ImageName.c_str());
   Dev1Ph_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Dev1PhHisto.gif";
   c1->Print(ImageName.c_str());
   Dev2Ph_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Dev2PhHisto.gif";
   c1->Print(ImageName.c_str());
   Scaler1_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Scaler1Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler2_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Scaler2Histo.gif";
   c1->Print(ImageName.c_str());
   Scaler3_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Scaler3Histo.gif";
   c1->Print(ImageName.c_str());

   LrRampDiff_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/LrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampDiff_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/HrRampDiffHisto.gif";
   c1->Print(ImageName.c_str());
   TAC23Tim_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/TAC23TimHisto.gif";
   c1->Print(ImageName.c_str());
   NTACFired_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/NTACFiredHisto.gif";
   c1->Print(ImageName.c_str());


   
   FileOut->Write();
   FileOut->Close();
}
