#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "Riostream.h"
#define PSECNtupleAna_cxx
#include "PSECNtupleAna.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <string>
#include <math.h>

void PSECNtupleAna::Loop(const char* OutHistos, const char* OutSummary, const char* OutImage)
{
//   In a ROOT session, you can do:
//      Root > .L PSECNtupleAna.C
//      Root > PSECNtupleAna t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
	
	
	//Structure of this (really) big function:
	//- declaration of means, rms
	//- declaration of fixed scale histos
	//- declaration of custom scale histos, and automatic creation by TTree::Draw
	//- Loop over events
	// * Fill histos
	// * modify means and rms
	//- final calculation of means and rms
	//- write the summary
	//- write images of histos

	//Declaration of means and RMSs
	double         mean_RunNumber=0.;
	double         mean_RunType=0.;
	double         mean_EventNumber=0.;
	double         mean_BufferSequenceNumber=0.;
	double         mean_RealTimeLR=0.;
	double         mean_RealTimeHR=0.;
	double         mean_LrRampTimeAdc=0.;
	double         mean_HrRampTimeAdc=0.;
	double         mean_Mcp1TimeAdc=0.;
	double         mean_Mcp1PhAdc=0.;
	double         mean_Mcp2TimeAdc=0.;
	double         mean_Mcp2PhAdc=0.;
	double         mean_Mcp3TimeAdc=0.;
	double         mean_Mcp3PhAdc=0.;
	double         mean_Mcp4TimeAdc=0.;
	double         mean_Mcp4PhAdc=0.;
	double         mean_nScaler1=0.;
	double         mean_nScaler2=0.;
	double         mean_nScaler3=0.;
	double         mean_CherenkovLatchADC=0.;
	double         rms_RunNumber=0.;
	double         rms_RunType=0.;
	double         rms_EventNumber=0.;
	double         rms_BufferSequenceNumber=0.;
	double         rms_RealTimeLR=0.;
	double         rms_RealTimeHR=0.;
	double         rms_LrRampTimeAdc=0.;
	double         rms_HrRampTimeAdc=0.;
	double         rms_Mcp1TimeAdc=0.;
	double         rms_Mcp1PhAdc=0.;
	double         rms_Mcp2TimeAdc=0.;
	double         rms_Mcp2PhAdc=0.;
	double         rms_Mcp3TimeAdc=0.;
	double         rms_Mcp3PhAdc=0.;
	double         rms_Mcp4TimeAdc=0.;
	double         rms_Mcp4PhAdc=0.;
	double         rms_nScaler1=0.;
	double         rms_nScaler2=0.;
	double         rms_nScaler3=0.;
	double         rms_CherenkovLatchADC=0.;

	//Idem for computed data
	double	       mean_Mcp12TimeAdc=0.;
	double	       n_Mcp12TimeAdc=0.;	//Number of entries with both MCPs 1 and 2 fired
	double	       mean_Mcp34TimeAdc=0.;
	double	       n_Mcp34TimeAdc=0.;	//Number of entries with both MCPs 3 and 4 fired
	double	       mean_McpTof=0.;		//This is the mean of time of flight above all entries
	double	       mean_McpTof_true=0.;	//This is the mean of time of flight considering only entries where 4 MCPs were fired
	double	       n_McpTof_true=0.;	//Number of entries to consider for the previous mean
	double	       mean_IsMcp1Fired=0.;
	double	       mean_IsMcp2Fired=0.;
	double	       mean_IsMcp3Fired=0.;
	double	       mean_IsMcp4Fired=0.;
	double	       mean_NMcpFired=0.;
	double	       mean_LrRampTimeAdcDiff;
	double	       mean_HrRampTimeAdcDiff;
	double	       rms_Mcp12TimeAdc=0.;
	double	       rms_Mcp34TimeAdc=0.;
	double	       rms_McpTof=0.;
	double	       rms_McpTof_true=0.;
	double	       rms_IsMcp1Fired=0.;
	double	       rms_IsMcp2Fired=0.;
	double	       rms_IsMcp3Fired=0.;
	double	       rms_IsMcp4Fired=0.;
	double	       rms_NMcpFired=0.;
	double	       rms_LrRampTimeAdcDiff;
	double	       rms_HrRampTimeAdcDiff;

	TFile *FileOut = new TFile(OutHistos,"RECREATE");
	
	TLegend *legend=new TLegend(0.6,0.65,0.88,0.85);
	legend->SetTextFont(72);
	legend->SetTextSize(0.03);
	
	ofstream summary (OutSummary, ios::out);

	//TCanvas *canvas = new TCanvas("canvas","nearline analysis",200,10,600,400);
	//canvas->SetGrid();
	
   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntriesFast();

   //Fixed scale histograms
   //1D histograms of all branches
   TDirectory *cdFixedHistos = FileOut->mkdir("HistosFixedScale");
   cdFixedHistos->cd();
   TDirectory *cdFixedData = cdFixedHistos->mkdir("dataFixedHistos");
   cdFixedData->cd();
   TH1F *RunNumberHisto = new TH1F("RunNumber","RunNumber",100,0.,100.);
   TH1F *RunTypeHisto = new TH1F("RunType","RunType",4,0.,3.);
   TH1F *EventNumberHisto = new TH1F("EventNumber","EventNumber",1000,0.,100000.);
   TH1F *BufferSequenceNumberHisto = new TH1F("BufferSequenceNumber","BufferSequenceNumber",100,0.,1024.);
   TH1F *RealTimeLRHisto = new TH1F("RealTimeLR","RealTimeLR",100,1213e6,1215e6);
   TH1F *RealTimeHRHisto = new TH1F("RealTimeHR","RealTimeHR",100,0.,1e6);
   TH1F *LrRampTimeAdcHisto = new TH1F("LrRampTimeAdc","LrRampTimeAdc",100,0.,1023.);
   TH1F *HrRampTimeAdcHisto = new TH1F("HrRampTimeAdc","HrRampTimeAdc",100,0.,1023.);
   TH1F *Mcp1TimeAdcHisto = new TH1F("Mcp1TimeAdc","Mcp1TimeAdc",100,0.,16383.);
   TH1F *Mcp1PhAdcHisto = new TH1F("Mcp1PhAdc","Mcp1PhAdc",100,0.,1023.);
   TH1F *Mcp2TimeAdcHisto = new TH1F("Mcp2TimeAdc","Mcp2TimeAdc",100,0.,16383.);
   TH1F *Mcp2PhAdcHisto = new TH1F("Mcp2PhAdc","Mcp2PhAdc",100,0.,1023.);
   TH1F *Mcp3TimeAdcHisto = new TH1F("Mcp3TimeAdc","Mcp3TimeAdc",100,0.,16383.);
   TH1F *Mcp3PhAdcHisto = new TH1F("Mcp3PhAdc","Mcp3PhAdc",100,0.,1023.);
   TH1F *Mcp4TimeAdcHisto = new TH1F("Mcp4TimeAdc","Mcp4TimeAdc",100,0.,16383.);
   TH1F *Mcp4PhAdcHisto = new TH1F("Mcp4PhAdc","Mcp4PhAdc",100,0.,1023.);
   TH1F *nScaler1Histo = new TH1F("nScaler1","nScaler1",100,0.,1023.);
   TH1F *nScaler2Histo = new TH1F("nScaler2","nScaler2",100,0.,1023.);
   TH1F *nScaler3Histo = new TH1F("nScaler3","nScaler3",100,0.,1023.);
   TH1F *CherenkovLatchADCHisto = new TH1F("CherenkovLatchADC","CherenkovLatchADC",10,0.,10.);
   
;

   //1D histograms of computed variables
   cdFixedHistos->cd();
   TDirectory *cdFixedComputed = cdFixedHistos->mkdir("ComputedFixedHistos");
   cdFixedComputed->cd();
   TH1F *Mcp12TimeAdcHisto = new TH1F("Mcp12TimeAdc","Mcp12TimeAdc",100,0.,16384.);
   TH1F *Mcp34TimeAdcHisto = new TH1F("Mcp34TimeAdc","Mcp34TimeAdc",100,0.,16384.);
   TH1F *McpTofHisto = new TH1F("McpTof","McpTof",100,0.,16384.);
   TH1F *NMcpFiredHisto = new TH1F("NMcpFiredHisto","NMcpFiredHisto",4,0.,4.);
   TH1F *LrRampTimeAdcDiffHisto = new TH1F("LrRampTimeAdcDiff","LrRampTimeAdcDiff",100,0.,1024.);
   TH1F *HrRampTimeAdcDiffHisto = new TH1F("HrRampTimeAdcDiff","HrRampTimeAdcDiff",100,0.,1024.);

   //2D histograms of charge vs time of flight by adc
   cdFixedHistos->cd();
   TDirectory *cdFixedOthers = cdFixedHistos->mkdir("OtherFixedHistos");
   cdFixedOthers->cd();
   TH2F *PhVsTimeAdc1Histo = new TH2F("PhVsTimeAdc1","Ph Vs Time for ADC 1",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc2Histo = new TH2F("PhVsTimeAdc2","Ph Vs Time for ADC 2",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc3Histo = new TH2F("PhVsTimeAdc3","Ph Vs Time for ADC 3",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc4Histo = new TH2F("PhVsTimeAdc4","Ph Vs Time for ADC 4",100,0.,16384.,100,0.,16384.);
   /*//Other histos
   TH2F *BufferSeqHisto = new TH2F("BufferSequenceN","BufferSequenceNumber vs EventNumber",100,0.,nentries+10.,100,0.,300.);
   TH2F *BufferStartHRHisto = new TH2F("BufferStartHR","RealTimeHR vs RealTimeLR",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferEndHRHisto = new TH2F("BufferEndHR","HrRampTimeAdc vs LrRampTimeAdc",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferDelayHisto = new TH2F("BufferDelay","BufferDelay vs EventNumber",100,0.,nentries+10.,100,0.,250.);
   TH2F *EventNumberVsTimeHisto = new TH2F("EventNumberVsTime","EventNumber vs Start Time",100,1.2111586e9,1.2131589e9,100,0.,nentries+10.);
*/
   cout << "Now root is creating some histograms setting automatically the scales" << endl;

   //Automatic scale histograms
   //1D Histograms of all branches
   TDirectory *cdCustomHistos = FileOut->mkdir("HistosCustomScale");
   cdCustomHistos->cd();
   TDirectory *cdCustomData = cdCustomHistos->mkdir("dataCustomHistos");
   cdCustomData->cd();
   fChain->Draw("RunNumber>>RunNumber_C");
   TH1F *RunNumber_CHisto = (TH1F*)gDirectory->Get("RunNumber_C");
   fChain->Draw("RunType>>RunType_C");
   TH1F *RunType_CHisto = (TH1F*)gDirectory->Get("RunType_C");
   fChain->Draw("EventNumber>>EventNumber_C");
   TH1F *EventNumber_CHisto = (TH1F*)gDirectory->Get("EventNumber_C");
   fChain->Draw("BufferSequenceNumber>>BufferSequenceNumber_C");
   TH1F *BufferSequenceNumber_CHisto = (TH1F*)gDirectory->Get("BufferSequenceNumber_C");
   fChain->Draw("RealTimeLR>>RealTimeLR_C");
   TH1F *RealTimeLR_CHisto = (TH1F*)gDirectory->Get("RealTimeLR_C");
   fChain->Draw("RealTimeHR>>RealTimeHR_C");
   TH1F *RealTimeHR_CHisto = (TH1F*)gDirectory->Get("RealTimeHR_C");
   fChain->Draw("LrRampTimeAdc>>LrRampTimeAdc_C");
   TH1F *LrRampTimeAdc_CHisto = (TH1F*)gDirectory->Get("LrRampTimeAdc_C");
   fChain->Draw("HrRampTimeAdc>>HrRampTimeAdc_C");
   TH1F *HrRampTimeAdc_CHisto = (TH1F*)gDirectory->Get("HrRampTimeAdc_C");
   fChain->Draw("Mcp1TimeAdc>>Mcp1TimeAdc_C");
   TH1F *Mcp1TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp1TimeAdc_C");
   fChain->Draw("Mcp1PhAdc>>Mcp1PhAdc_C");
   TH1F *Mcp1PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp1PhAdc_C");
   fChain->Draw("Mcp2TimeAdc>>Mcp2TimeAdc_C");
   TH1F *Mcp2TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp2TimeAdc_C");
   fChain->Draw("Mcp2PhAdc>>Mcp2PhAdc_C");
   TH1F *Mcp2PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp2PhAdc_C");
   fChain->Draw("Mcp3TimeAdc>>Mcp3TimeAdc_C");
   TH1F *Mcp3TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp3TimeAdc_C");
   fChain->Draw("Mcp3PhAdc>>Mcp3PhAdc_C");
   TH1F *Mcp3PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp3PhAdc_C");
   fChain->Draw("Mcp4TimeAdc>>Mcp4TimeAdc_C");
   TH1F *Mcp4TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp4TimeAdc_C");
   fChain->Draw("Mcp4PhAdc>>Mcp4PhAdc_C");
   TH1F *Mcp4PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp4PhAdc_C");
   fChain->Draw("nScaler1>>nScaler1_C");
   TH1F *nScaler1_CHisto = (TH1F*)gDirectory->Get("nScaler1_C");
   fChain->Draw("nScaler2>>nScaler2_C");
   TH1F *nScaler2_CHisto = (TH1F*)gDirectory->Get("nScaler2_C");
   fChain->Draw("nScaler3>>nScaler3_C");
   TH1F *nScaler3_CHisto = (TH1F*)gDirectory->Get("nScaler3_C");
   fChain->Draw("CherenkovLatchADC>>CherenkovLatchADC_C");
   TH1F *CherenkovLatchADC_CHisto = (TH1F*)gDirectory->Get("CherenkovLatchADC_C");
   
   ;

   //1D histograms of computed variables
   cdCustomHistos->cd();
   TDirectory *cdCustomComputed = cdCustomHistos->mkdir("ComputedCustomHistos");
   cdCustomComputed->cd();
   fChain->Draw("Mcp12TimeAdc>>Mcp12TimeAdc_C");
   TH1F *Mcp12TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp12TimeAdc_C");
   fChain->Draw("Mcp34TimeAdc>>Mcp34TimeAdc_C");
   TH1F *Mcp34TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp34TimeAdc_C");
   fChain->Draw("McpTof>>McpTof_C");
   TH1F *McpTof_CHisto = (TH1F*)gDirectory->Get("McpTof_C");
   fChain->Draw("NMcpFired>>NMcpFired_C");
   TH1F *NMcpFired_CHisto = (TH1F*)gDirectory->Get("NMcpFired_C");
   fChain->Draw("LrRampTimeAdcDiff>>LrRampTimeAdcDiff_C");
   TH1F *LrRampTimeAdcDiff_CHisto = (TH1F*)gDirectory->Get("LrRampTimeAdcDiff_C");
   fChain->Draw("HrRampTimeAdcDiff>>HrRampTimeAdcDiff_C");
   TH1F *HrRampTimeAdcDiff_CHisto = (TH1F*)gDirectory->Get("HrRampTimeAdcDiff_C");

   //2D histograms of charge vs time of flight by adc
   cdCustomHistos->cd();
   TDirectory *cdCustomOthers = cdCustomHistos->mkdir("OtherCustomHistos");
   cdCustomOthers->cd();
   fChain->Draw("Mcp1PhAdc:Mcp1TimeAdc>>PhVsTimeAdc1_C");
   TH2F *PhVsTimeAdc1_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc1_C");
   fChain->Draw("Mcp2PhAdc:Mcp2TimeAdc>>PhVsTimeAdc2_C");
   TH2F *PhVsTimeAdc2_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc2_C");
   fChain->Draw("Mcp3PhAdc:Mcp3TimeAdc>>PhVsTimeAdc3_C");
   TH2F *PhVsTimeAdc3_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc3_C");
   fChain->Draw("Mcp4PhAdc:Mcp4TimeAdc>>PhVsTimeAdc4_C");
   TH2F *PhVsTimeAdc4_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc4_C");
   //2D histogram of Mcp1TimeAdc vs time
   fChain->Draw("LrRampTimeAdc:HrRampTimeAdc>>RampVsTime_C(1000)");
   TH2F *RampVsTime_CHisto = (TH2F*)gDirectory->Get("RampVsTime_C");

   //2D histograms of all variables vs unique event number
   TDirectory *cdEventsHistos = FileOut->mkdir("HistosVsEvents");
   cdEventsHistos->cd();
   TDirectory *cdEventsData = cdEventsHistos->mkdir("dataVsEventsHistos");
   cdEventsData->cd();
   fChain->Draw("RunType:UniqueEventNumber>>RunType_VE");
   TH2F *RunType_VEHisto = (TH2F*)gDirectory->Get("RunType_VE");
   fChain->Draw("RunNumber:UniqueEventNumber>>RunNumber_VE");
   TH2F *RunNumber_VEHisto = (TH2F*)gDirectory->Get("RunNumber_VE");
   fChain->Draw("EventNumber:UniqueEventNumber>>EventNumber_VE");
   TH2F *EventNumber_VEHisto = (TH2F*)gDirectory->Get("EventNumber_VE");
   fChain->Draw("BufferSequenceNumber:UniqueEventNumber>>BufferSequenceNumber_VE");
   TH2F *BufferSequenceNumber_VEHisto = (TH2F*)gDirectory->Get("BufferSequenceNumber_VE");
   fChain->Draw("RealTimeLR:UniqueEventNumber>>RealTimeLR_VE");
   TH2F *RealTimeLR_VEHisto = (TH2F*)gDirectory->Get("RealTimeLR_VE");
   fChain->Draw("RealTimeHR:UniqueEventNumber>>RealTimeHR_VE");
   TH2F *RealTimeHR_VEHisto = (TH2F*)gDirectory->Get("RealTimeHR_VE");
   fChain->Draw("LrRampTimeAdc:UniqueEventNumber>>LrRampTimeAdc_VE");
   TH2F *LrRampTimeAdc_VEHisto = (TH2F*)gDirectory->Get("LrRampTimeAdc_VE");
   fChain->Draw("HrRampTimeAdc:UniqueEventNumber>>HrRampTimeAdc_VE");
   TH2F *HrRampTimeAdc_VEHisto = (TH2F*)gDirectory->Get("HrRampTimeAdc_VE");
   fChain->Draw("Mcp1TimeAdc:UniqueEventNumber>>Mcp1TimeAdc_VE");
   TH2F *Mcp1TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp1TimeAdc_VE");
   fChain->Draw("Mcp1PhAdc:UniqueEventNumber>>Mcp1PhAdc_VE");
   TH2F *Mcp1PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp1PhAdc_VE");
   fChain->Draw("Mcp2TimeAdc:UniqueEventNumber>>Mcp2TimeAdc_VE");
   TH2F *Mcp2TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp2TimeAdc_VE");
   fChain->Draw("Mcp2PhAdc:UniqueEventNumber>>Mcp2PhAdc_VE");
   TH2F *Mcp2PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp2PhAdc_VE");
   fChain->Draw("Mcp3TimeAdc:UniqueEventNumber>>Mcp3TimeAdc_VE");
   TH2F *Mcp3TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp3TimeAdc_VE");
   fChain->Draw("Mcp3PhAdc:UniqueEventNumber>>Mcp3PhAdc_VE");
   TH2F *Mcp3PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp3PhAdc_VE");
   fChain->Draw("Mcp4TimeAdc:UniqueEventNumber>>Mcp4TimeAdc_VE");
   TH2F *Mcp4TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp4TimeAdc_VE");
   fChain->Draw("Mcp4PhAdc:UniqueEventNumber>>Mcp4PhAdc_VE");
   TH2F *Mcp4PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp4PhAdc_VE");
   fChain->Draw("nScaler1:UniqueEventNumber>>nScaler1_VE");
   TH2F *nScaler1_VEHisto = (TH2F*)gDirectory->Get("nScaler1_VE");
   fChain->Draw("nScaler2:UniqueEventNumber>>nScaler2_VE");
   TH2F *nScaler2_VEHisto = (TH2F*)gDirectory->Get("nScaler2_VE");
   fChain->Draw("nScaler3:UniqueEventNumber>>nScaler3_VE");
   TH2F *nScaler3_VEHisto = (TH2F*)gDirectory->Get("nScaler3_VE");
   fChain->Draw("CherenkovLatchADC:UniqueEventNumber>>CherenkovLatchADC_VE");
   TH2F *CherenkovLatchADC_VEHisto = (TH2F*)gDirectory->Get("CherenkovLatchADC_VE");
   
   cdEventsHistos->cd();
   TDirectory *cdEventsComputed = cdEventsHistos->mkdir("ComputedVsEventsHistos");
   cdEventsComputed->cd();
   fChain->Draw("Mcp12TimeAdc:UniqueEventNumber>>Mcp12TimeAdc_VE");
   TH2F *Mcp12TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp12TimeAdc_VE");
   fChain->Draw("Mcp34TimeAdc:UniqueEventNumber>>Mcp34TimeAdc_VE");
   TH2F *Mcp34TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp34TimeAdc_VE");
   fChain->Draw("McpTof:UniqueEventNumber>>McpTof_VE");
   TH2F *McpTof_VEHisto = (TH2F*)gDirectory->Get("McpTof_VE");
   fChain->Draw("NMcpFired:UniqueEventNumber>>NMcpFired_VE");
   TH2F *NMcpFired_VEHisto = (TH2F*)gDirectory->Get("NMcpFired_VE");
   fChain->Draw("LrRampTimeAdcDiff:UniqueEventNumber>>LrRampTimeAdcDiff_VE");
   TH2F *LrRampTimeAdcDiff_VEHisto = (TH2F*)gDirectory->Get("LrRampTimeAdcDiff_VE");
   fChain->Draw("HrRampTimeAdcDiff:UniqueEventNumber>>HrRampTimeAdcDiff_VE");
   TH2F *HrRampTimeAdcDiff_VEHisto = (TH2F*)gDirectory->Get("HrRampTimeAdcDiff_VE");
   
   cdFixedHistos->cd();

   Long64_t nbytes = 0, nb = 0;
   
   float RunNumber0 = -999.;

   //***************************************************************************//
   //					LOOP					//
   //***************************************************************************//
   cout << "Looping over events" << endl;
   for (Long64_t jentry=0; jentry<nentries;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
	fChain->Fill();
      
      if (RunNumber0 == -999.) RunNumber0 = data_RunNumber;
      



      //Calculate mean and rms of each leaf
      mean_RunNumber += data_RunNumber;
      mean_RunType += data_RunType;
      mean_EventNumber += data_EventNumber;
      mean_BufferSequenceNumber += data_BufferSequenceNumber;
      mean_RealTimeLR += data_RealTimeLR;
      mean_RealTimeHR += data_RealTimeHR;
      mean_LrRampTimeAdc += data_LrRampTimeAdc;
      mean_HrRampTimeAdc += data_HrRampTimeAdc;
      mean_Mcp1TimeAdc += data_Mcp1TimeAdc;
      mean_Mcp1PhAdc += data_Mcp1PhAdc;
      mean_Mcp2TimeAdc += data_Mcp2TimeAdc;
      mean_Mcp2PhAdc += data_Mcp2PhAdc;
      mean_Mcp3TimeAdc += data_Mcp3TimeAdc;
      mean_Mcp3PhAdc += data_Mcp3PhAdc;
      mean_Mcp4TimeAdc += data_Mcp4TimeAdc;
      mean_Mcp4PhAdc += data_Mcp4PhAdc;
      mean_nScaler1 += data_nScaler1;
      mean_nScaler2 += data_nScaler2;
      mean_nScaler3 += data_nScaler3;
      mean_CherenkovLatchADC += data_CherenkovLatchADC;
       rms_RunNumber += data_RunNumber*data_RunNumber;
       rms_RunType += data_RunType*data_RunType;
       rms_EventNumber += data_EventNumber*data_EventNumber;
       rms_BufferSequenceNumber += data_BufferSequenceNumber*data_BufferSequenceNumber;
       rms_RealTimeLR += data_RealTimeLR*data_RealTimeLR;
       rms_RealTimeHR += data_RealTimeHR*data_RealTimeHR;
       rms_LrRampTimeAdc += data_LrRampTimeAdc*data_LrRampTimeAdc;
       rms_HrRampTimeAdc += data_HrRampTimeAdc*data_HrRampTimeAdc;
       rms_Mcp1TimeAdc += data_Mcp1TimeAdc*data_Mcp1TimeAdc;
       rms_Mcp1PhAdc += data_Mcp1PhAdc*data_Mcp1PhAdc;
       rms_Mcp2TimeAdc += data_Mcp2TimeAdc*data_Mcp2TimeAdc;
       rms_Mcp2PhAdc += data_Mcp2PhAdc*data_Mcp2PhAdc;
       rms_Mcp3TimeAdc += data_Mcp3TimeAdc*data_Mcp3TimeAdc;
       rms_Mcp3PhAdc += data_Mcp3PhAdc*data_Mcp3PhAdc;
       rms_Mcp4TimeAdc += data_Mcp4TimeAdc*data_Mcp4TimeAdc;
       rms_Mcp4PhAdc += data_Mcp4PhAdc*data_Mcp4PhAdc;
       rms_nScaler1 += data_nScaler1*data_nScaler1;
       rms_nScaler2 += data_nScaler2*data_nScaler2;
       rms_nScaler3 += data_nScaler3*data_nScaler3;
       rms_CherenkovLatchADC += data_CherenkovLatchADC*data_CherenkovLatchADC;

	//Calculate mean and RMS for computed data
	mean_McpTof += computed_McpTof;
	mean_IsMcp1Fired += computed_IsMcp1Fired;
	mean_IsMcp2Fired += computed_IsMcp2Fired;
	mean_IsMcp3Fired += computed_IsMcp3Fired;
	mean_IsMcp4Fired += computed_IsMcp4Fired;
	mean_NMcpFired += computed_NMcpFired;
	mean_LrRampTimeAdcDiff += computed_LrRampTimeAdcDiff;
	mean_HrRampTimeAdcDiff += computed_HrRampTimeAdcDiff;
	rms_McpTof += computed_McpTof*computed_McpTof;
	rms_IsMcp1Fired += computed_IsMcp1Fired*computed_IsMcp1Fired;
	rms_IsMcp2Fired += computed_IsMcp2Fired*computed_IsMcp2Fired;
	rms_IsMcp3Fired += computed_IsMcp3Fired*computed_IsMcp3Fired;
	rms_IsMcp4Fired += computed_IsMcp4Fired*computed_IsMcp4Fired;
	rms_NMcpFired += computed_NMcpFired*computed_NMcpFired;
	rms_LrRampTimeAdcDiff += computed_LrRampTimeAdcDiff*computed_LrRampTimeAdcDiff;
	rms_HrRampTimeAdcDiff += computed_HrRampTimeAdcDiff*computed_HrRampTimeAdcDiff;
	if((int) computed_IsMcp1Fired && (int) computed_IsMcp2Fired) {
		n_Mcp12TimeAdc += 1;
		mean_Mcp12TimeAdc += computed_Mcp12TimeAdc;
		rms_Mcp12TimeAdc += computed_Mcp12TimeAdc*computed_Mcp12TimeAdc;
	}
	if((int) computed_IsMcp3Fired && (int) computed_IsMcp4Fired) {
		n_Mcp34TimeAdc += 1;
		mean_Mcp34TimeAdc += computed_Mcp34TimeAdc;
		rms_Mcp34TimeAdc += computed_Mcp34TimeAdc*computed_Mcp34TimeAdc;
	}
	if((int) computed_NMcpFired == 4) {
		n_McpTof_true += 1;
		mean_McpTof_true += computed_McpTof;
		rms_McpTof_true += computed_McpTof*computed_McpTof;
	}
      
      //Fill Histos
      RunNumberHisto->Fill(data_RunNumber);
      RunTypeHisto->Fill(data_RunType);
      EventNumberHisto->Fill(data_EventNumber);
      BufferSequenceNumberHisto->Fill(data_BufferSequenceNumber);
      RealTimeLRHisto->Fill(data_RealTimeLR);
      RealTimeHRHisto->Fill(data_RealTimeHR);
      LrRampTimeAdcHisto->Fill(data_LrRampTimeAdc);
      HrRampTimeAdcHisto->Fill(data_HrRampTimeAdc);
      Mcp1TimeAdcHisto->Fill(data_Mcp1TimeAdc);
      Mcp1PhAdcHisto->Fill(data_Mcp1PhAdc);
      Mcp2TimeAdcHisto->Fill(data_Mcp2TimeAdc);
      Mcp2PhAdcHisto->Fill(data_Mcp2PhAdc);
      Mcp3TimeAdcHisto->Fill(data_Mcp3TimeAdc);
      Mcp3PhAdcHisto->Fill(data_Mcp3PhAdc);
      Mcp4TimeAdcHisto->Fill(data_Mcp4TimeAdc);
      Mcp4PhAdcHisto->Fill(data_Mcp4PhAdc);
      nScaler1Histo->Fill(data_nScaler1);
      nScaler2Histo->Fill(data_nScaler2);
      nScaler3Histo->Fill(data_nScaler3);
      CherenkovLatchADCHisto->Fill(data_CherenkovLatchADC);
      /*
      BufferSeqHisto->Fill(data_EventNumber,data_BufferSequenceNumber);
      BufferStartHRHisto->Fill(data_RealTimeLR,data_RealTimeHR);
      BufferEndHRHisto->Fill(data_LrRampTimeAdc,data_HrRampTimeAdc);
      BufferDelayHisto->Fill(data_EventNumber,data_LrRampTimeAdc+data_HrRampTimeAdc-data_RealTimeLR-data_RealTimeHR);
      EventNumberVsTimeHisto->Fill(data_RealTimeLR+data_RealTimeHR,data_EventNumber);
*/
      PhVsTimeAdc1Histo->Fill(data_Mcp1TimeAdc,data_Mcp1PhAdc);
      PhVsTimeAdc2Histo->Fill(data_Mcp2TimeAdc,data_Mcp2PhAdc);
      PhVsTimeAdc3Histo->Fill(data_Mcp3TimeAdc,data_Mcp3PhAdc);
      PhVsTimeAdc4Histo->Fill(data_Mcp4TimeAdc,data_Mcp4PhAdc);

      if((int) computed_IsMcp1Fired && (int) computed_IsMcp2Fired) Mcp12TimeAdcHisto->Fill(computed_Mcp12TimeAdc);
      if((int) computed_IsMcp3Fired && (int) computed_IsMcp4Fired) Mcp34TimeAdcHisto->Fill(computed_Mcp34TimeAdc);
      if((int) computed_NMcpFired == 4)McpTofHisto->Fill(computed_McpTof);
      NMcpFiredHisto->Fill(computed_NMcpFired);
      LrRampTimeAdcDiffHisto->Fill(computed_LrRampTimeAdcDiff);
      HrRampTimeAdcDiffHisto->Fill(computed_HrRampTimeAdcDiff);

   }
   cout << "End of loop" << endl;
   //***************************************************************************//
   //				END OF LOOP					//
   //***************************************************************************//
   
   //Means and RMSs
   mean_RunNumber=mean_RunNumber/nentries;
   mean_RunType=   mean_RunType/nentries;
   mean_EventNumber=   mean_EventNumber/nentries;
   mean_BufferSequenceNumber=   mean_BufferSequenceNumber/nentries;
   mean_RealTimeLR=   mean_RealTimeLR/nentries;
   mean_RealTimeHR=   mean_RealTimeHR/nentries;
   mean_LrRampTimeAdc=   mean_LrRampTimeAdc/nentries;
   mean_HrRampTimeAdc=   mean_HrRampTimeAdc/nentries;
   mean_Mcp1TimeAdc=   mean_Mcp1TimeAdc/nentries;
   mean_Mcp1PhAdc=   mean_Mcp1PhAdc/nentries;
   mean_Mcp2TimeAdc=   mean_Mcp2TimeAdc/nentries;
   mean_Mcp2PhAdc=   mean_Mcp2PhAdc/nentries;
   mean_Mcp3TimeAdc=   mean_Mcp3TimeAdc/nentries;
   mean_Mcp3PhAdc=   mean_Mcp3PhAdc/nentries;
   mean_Mcp4TimeAdc=   mean_Mcp4TimeAdc/nentries;
   mean_Mcp4PhAdc=   mean_Mcp4PhAdc/nentries;
   mean_nScaler1=   mean_nScaler1/nentries;
   mean_nScaler2=   mean_nScaler2/nentries;
   mean_nScaler3=   mean_nScaler3/nentries;
   mean_CherenkovLatchADC=   mean_CherenkovLatchADC/nentries;

   rms_RunNumber=sqrt( (double)rms_RunNumber/nentries -mean_RunNumber*mean_RunNumber);
   rms_RunType=sqrt( (double)rms_RunType/nentries -mean_RunType*mean_RunType);
   rms_EventNumber=sqrt( (double)rms_EventNumber/nentries -mean_EventNumber*mean_EventNumber);
   rms_BufferSequenceNumber=sqrt( (double)rms_BufferSequenceNumber/nentries -mean_BufferSequenceNumber*mean_BufferSequenceNumber);
   rms_RealTimeLR=sqrt( (double)rms_RealTimeLR/nentries -mean_RealTimeLR*mean_RealTimeLR);
   rms_RealTimeHR=sqrt( (double)rms_RealTimeHR/nentries -mean_RealTimeHR*mean_RealTimeHR);
   rms_LrRampTimeAdc=sqrt( (double)rms_LrRampTimeAdc/nentries -mean_LrRampTimeAdc*mean_LrRampTimeAdc);
   rms_HrRampTimeAdc=sqrt( (double)rms_HrRampTimeAdc/nentries -mean_HrRampTimeAdc*mean_HrRampTimeAdc);
   rms_Mcp1TimeAdc=sqrt( (double)rms_Mcp1TimeAdc/nentries -mean_Mcp1TimeAdc*mean_Mcp1TimeAdc);
   rms_Mcp1PhAdc=sqrt( (double)rms_Mcp1PhAdc/nentries -mean_Mcp1PhAdc*mean_Mcp1PhAdc);
   rms_Mcp2TimeAdc=sqrt( (double)rms_Mcp2TimeAdc/nentries -mean_Mcp2TimeAdc*mean_Mcp2TimeAdc);
   rms_Mcp2PhAdc=sqrt( (double)rms_Mcp2PhAdc/nentries -mean_Mcp2PhAdc*mean_Mcp2PhAdc);
   rms_Mcp3TimeAdc=sqrt( (double)rms_Mcp3TimeAdc/nentries -mean_Mcp3TimeAdc*mean_Mcp3TimeAdc);
   rms_Mcp3PhAdc=sqrt( (double)rms_Mcp3PhAdc/nentries -mean_Mcp3PhAdc*mean_Mcp3PhAdc);
   rms_Mcp4TimeAdc=sqrt( (double)rms_Mcp4TimeAdc/nentries -mean_Mcp4TimeAdc*mean_Mcp4TimeAdc);
   rms_Mcp4PhAdc=sqrt( (double)rms_Mcp4PhAdc/nentries -mean_Mcp4PhAdc*mean_Mcp4PhAdc);
   rms_nScaler1=sqrt( (double)rms_nScaler1/nentries -mean_nScaler1*mean_nScaler1);
   rms_nScaler2=sqrt( (double)rms_nScaler2/nentries -mean_nScaler2*mean_nScaler2);
   rms_nScaler3=sqrt( (double)rms_nScaler3/nentries -mean_nScaler3*mean_nScaler3);
   rms_CherenkovLatchADC=sqrt( (double)rms_CherenkovLatchADC/nentries -mean_CherenkovLatchADC*mean_CherenkovLatchADC);
   


   //idem for computed data
   mean_Mcp12TimeAdc= mean_Mcp12TimeAdc/n_Mcp12TimeAdc;
   mean_Mcp34TimeAdc= mean_Mcp34TimeAdc/n_Mcp34TimeAdc;
   mean_McpTof= mean_McpTof/nentries;
   mean_IsMcp1Fired= mean_IsMcp1Fired/nentries;
   mean_IsMcp2Fired= mean_IsMcp2Fired/nentries;
   mean_IsMcp3Fired= mean_IsMcp3Fired/nentries;
   mean_IsMcp4Fired= mean_IsMcp4Fired/nentries;
   mean_NMcpFired= mean_NMcpFired/nentries;
   mean_LrRampTimeAdcDiff= mean_LrRampTimeAdcDiff/nentries;
   mean_HrRampTimeAdcDiff= mean_HrRampTimeAdcDiff/nentries;
   mean_McpTof_true= mean_McpTof_true/n_McpTof_true;

   rms_Mcp12TimeAdc=sqrt( (double)rms_Mcp12TimeAdc/n_Mcp12TimeAdc -mean_Mcp12TimeAdc*mean_Mcp12TimeAdc);
   rms_Mcp34TimeAdc=sqrt( (double)rms_Mcp34TimeAdc/n_Mcp34TimeAdc -mean_Mcp34TimeAdc*mean_Mcp34TimeAdc);
   rms_McpTof=sqrt( (double)rms_McpTof/nentries -mean_McpTof*mean_McpTof);
   rms_IsMcp1Fired=sqrt( (double)rms_IsMcp1Fired/nentries -mean_IsMcp1Fired*mean_IsMcp1Fired);
   rms_IsMcp2Fired=sqrt( (double)rms_IsMcp2Fired/nentries -mean_IsMcp2Fired*mean_IsMcp2Fired);
   rms_IsMcp3Fired=sqrt( (double)rms_IsMcp3Fired/nentries -mean_IsMcp3Fired*mean_IsMcp3Fired);
   rms_IsMcp4Fired=sqrt( (double)rms_IsMcp4Fired/nentries -mean_IsMcp4Fired*mean_IsMcp4Fired);
   rms_NMcpFired=sqrt( (double)rms_NMcpFired/nentries -mean_NMcpFired*mean_NMcpFired);
   rms_LrRampTimeAdcDiff=sqrt( (double)rms_LrRampTimeAdcDiff/nentries -mean_LrRampTimeAdcDiff*mean_LrRampTimeAdcDiff);
   rms_HrRampTimeAdcDiff=sqrt( (double)rms_HrRampTimeAdcDiff/nentries -mean_HrRampTimeAdcDiff*mean_HrRampTimeAdcDiff);
   rms_McpTof_true=sqrt( (double)rms_McpTof_true/n_McpTof_true -mean_McpTof_true*mean_McpTof_true);

   summary << "Summary of the run " << data_RunNumber << endl;
   if (data_RunNumber != RunNumber0) summary << "Warning, more than one run was analysed (first was " << RunNumber0 << "). Following statistics concern all data analyzed." << endl;
   summary << "Number of entries: " << nentries << endl;
   summary << endl << endl;
   summary << "Means and RMSs of data from input ntuple" << endl;
   summary << "mean_RunNumber = " << mean_RunNumber /*<< " +/- " << mean_RunNumber/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RunType = " << mean_RunType /*<< " +/- " << mean_RunType/sqrt( (double)nentries)*/ << endl;
   summary << "mean_EventNumber = " << mean_EventNumber /*<< " +/- " << mean_EventNumber/sqrt( (double)nentries)*/ << endl;
   summary << "mean_BufferSequenceNumber = " << mean_BufferSequenceNumber /*<< " +/- " << mean_BufferSequenceNumber/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RealTimeLR = " << mean_RealTimeLR /*<< " +/- " << mean_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "mean_RealTimeHR = " << mean_RealTimeHR /*<< " +/- " << mean_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "mean_LrRampTimeAdc = " << mean_LrRampTimeAdc /*<< " +/- " << mean_LrRampTimeAdc/sqrt( (double)nentries)*/ << endl;
   summary << "mean_HrRampTimeAdc = " << mean_HrRampTimeAdc /*<< " +/- " << mean_HrRampTimeAdc/sqrt( (double)nentries)*/ << endl;
   summary << "mean_Mcp1TimeAdc = " << mean_Mcp1TimeAdc << " +/- " << mean_Mcp1TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp1PhAdc = " << mean_Mcp1PhAdc << " +/- " << mean_Mcp1PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp2TimeAdc = " << mean_Mcp2TimeAdc << " +/- " << mean_Mcp2TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp2PhAdc = " << mean_Mcp2PhAdc << " +/- " << mean_Mcp2PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp3TimeAdc = " << mean_Mcp3TimeAdc << " +/- " << mean_Mcp3TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp3PhAdc = " << mean_Mcp3PhAdc << " +/- " << mean_Mcp3PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp4TimeAdc = " << mean_Mcp4TimeAdc << " +/- " << mean_Mcp4TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp4PhAdc = " << mean_Mcp4PhAdc << " +/- " << mean_Mcp4PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_nScaler1 = " << mean_nScaler1 << " +/- " << mean_nScaler1/sqrt( (double)nentries) << endl;
   summary << "mean_nScaler2 = " << mean_nScaler2 << " +/- " << mean_nScaler2/sqrt( (double)nentries) << endl;
   summary << "mean_nScaler3 = " << mean_nScaler3 << " +/- " << mean_nScaler3/sqrt( (double)nentries) << endl;
   summary << "mean_CherenkovLatchADC = " << mean_CherenkovLatchADC /*<< " +/- " << mean_CherenkovLatchADC/sqrt( (double)nentries)*/ << endl;
   summary << "**************************************" << endl;
   summary << "rms_RunNumber = " << rms_RunNumber/* << " +/- " << rms_RunNumber/sqrt( (double)nentries)*/ << endl;
   summary << "rms_RunType = " << rms_RunType/* << " +/- " << rms_RunType/sqrt( (double)nentries)*/ << endl;
   summary << "rms_EventNumber = " << rms_EventNumber/* << " +/- " << rms_EventNumber/sqrt( (double)nentries)*/ << endl;
   summary << "rms_BufferSequenceNumber = " << rms_BufferSequenceNumber/* << " +/- " << rms_BufferSequenceNumber/sqrt( (double)nentries)*/ << endl;
   summary << "rms_RealTimeLR = " << rms_RealTimeLR/* << " +/- " << rms_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "rms_RealTimeHR = " << rms_RealTimeHR/* << " +/- " << rms_RealTimeLR/sqrt( (double)nentries)*/ << endl;
   summary << "rms_LrRampTimeAdc = " << rms_LrRampTimeAdc/* << " +/- " << rms_LrRampTimeAdc/sqrt( (double)nentries)*/ << endl;
   summary << "rms_HrRampTimeAdc = " << rms_HrRampTimeAdc/* << " +/- " << rms_HrRampTimeAdc/sqrt( (double)nentries)*/ << endl;
   summary << "rms_Mcp1TimeAdc = " << rms_Mcp1TimeAdc << " +/- " << rms_Mcp1TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp1PhAdc = " << rms_Mcp1PhAdc << " +/- " << rms_Mcp1PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp2TimeAdc = " << rms_Mcp2TimeAdc << " +/- " << rms_Mcp2TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp2PhAdc = " << rms_Mcp2PhAdc << " +/- " << rms_Mcp2PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp3TimeAdc = " << rms_Mcp3TimeAdc << " +/- " << rms_Mcp3TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp3PhAdc = " << rms_Mcp3PhAdc << " +/- " << rms_Mcp3PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp4TimeAdc = " << rms_Mcp4TimeAdc << " +/- " << rms_Mcp4TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp4PhAdc = " << rms_Mcp4PhAdc << " +/- " << rms_Mcp4PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_nScaler1 = " << rms_nScaler1 << " +/- " << rms_nScaler1/sqrt( (double)nentries) << endl;
   summary << "rms_nScaler2 = " << rms_nScaler2 << " +/- " << rms_nScaler2/sqrt( (double)nentries) << endl;
   summary << "rms_nScaler3 = " << rms_nScaler3 << " +/- " << rms_nScaler3/sqrt( (double)nentries) << endl;
   summary << "rms_CherenkovLatchADC = " << rms_CherenkovLatchADC/* << " +/- " << rms_CherenkovLatchADC/sqrt( (double)nentries)*/ << endl;
   summary << endl << "**************************************" << endl << endl;
   summary << "Means and RMSs from computed data" << endl;
   summary << "mean_Mcp12TimeAdc = " << mean_Mcp12TimeAdc << " +/- " << mean_Mcp12TimeAdc/sqrt( (double)n_Mcp12TimeAdc) << " counting " << n_Mcp12TimeAdc << " entries with both MCPs 1 and 2 fired" << endl;
   summary << "mean_Mcp34TimeAdc = " << mean_Mcp34TimeAdc << " +/- " << mean_Mcp34TimeAdc/sqrt( (double)n_Mcp34TimeAdc) << " counting " << n_Mcp34TimeAdc << " entries with both MCPs 3 and 4 fired" << endl;
   summary << "mean_McpTof = " << mean_McpTof << " +/- " << mean_McpTof/sqrt( (double)nentries) << endl;
   summary << "mean_McpTof_true = " << mean_McpTof_true << " +/- " << mean_McpTof_true/sqrt( (double)n_McpTof_true) << " counting " << n_McpTof_true << " entries with all 4 MCPs fired" << endl;
   summary << "mean_IsMcp1Fired = " << mean_IsMcp1Fired << " +/- " << mean_IsMcp1Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp2Fired = " << mean_IsMcp2Fired << " +/- " << mean_IsMcp2Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp3Fired = " << mean_IsMcp3Fired << " +/- " << mean_IsMcp3Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp4Fired = " << mean_IsMcp4Fired << " +/- " << mean_IsMcp4Fired/sqrt( (double)nentries) << endl;
   summary << "mean_NMcpFired = " << mean_NMcpFired << " +/- " << mean_NMcpFired/sqrt( (double)nentries) << endl;
   summary << "mean_LrRampTimeAdcDiff = " << mean_LrRampTimeAdcDiff << " +/- " << mean_LrRampTimeAdcDiff/sqrt( (double)nentries) << endl;
   summary << "mean_HrRampTimeAdcDiff = " << mean_HrRampTimeAdcDiff << " +/- " << mean_HrRampTimeAdcDiff/sqrt( (double)nentries) << endl;
   summary << "**************************************" << endl;
   summary << "rms_Mcp12TimeAdc = " << rms_Mcp12TimeAdc << " +/- " << rms_Mcp12TimeAdc/sqrt( (double)n_Mcp12TimeAdc) << endl;
   summary << "rms_Mcp34TimeAdc = " << rms_Mcp34TimeAdc << " +/- " << rms_Mcp34TimeAdc/sqrt( (double)n_Mcp34TimeAdc) << endl;
   summary << "rms_McpTof = " << rms_McpTof << " +/- " << rms_McpTof/sqrt( (double)nentries) << endl;
   summary << "rms_McpTof_true = " << rms_McpTof_true << " +/- " << rms_McpTof_true/sqrt( (double)n_McpTof_true) << endl;
   summary << "rms_IsMcp1Fired = " << rms_IsMcp1Fired << " +/- " << rms_IsMcp1Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp2Fired = " << rms_IsMcp2Fired << " +/- " << rms_IsMcp2Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp3Fired = " << rms_IsMcp3Fired << " +/- " << rms_IsMcp3Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp4Fired = " << rms_IsMcp4Fired << " +/- " << rms_IsMcp4Fired/sqrt( (double)nentries) << endl;
   summary << "rms_NMcpFired = " << rms_NMcpFired << " +/- " << rms_NMcpFired/sqrt( (double)nentries) << endl;
   summary << "rms_LrRampTimeAdcDiff = " << rms_LrRampTimeAdcDiff << " +/- " << rms_LrRampTimeAdcDiff/sqrt( (double)nentries) << endl;
   summary << "rms_HrRampTimeAdcDiff = " << rms_HrRampTimeAdcDiff << " +/- " << rms_HrRampTimeAdcDiff/sqrt( (double)nentries) << endl;

/*   //Write histos
   cdFixedData->cd();
   RunNumberHisto->Write();
   RunTypeHisto->Write();
   EventNumberHisto->Write();
   BufferSequenceNumberHisto->Write();
   RealTimeLRHisto->Write();
   RealTimeHRHisto->Write();
   LrRampTimeAdcHisto->Write();
   HrRampTimeAdcHisto->Write();
   Mcp1TimeAdcHisto->Write();
   Mcp1PhAdcHisto->Write();
   Mcp2TimeAdcHisto->Write();
   Mcp2PhAdcHisto->Write();
   Mcp3TimeAdcHisto->Write();
   Mcp3PhAdcHisto->Write();
   Mcp4TimeAdcHisto->Write();
   Mcp4PhAdcHisto->Write();
   nScaler1Histo->Write();
   nScaler2Histo->Write();
   nScaler3Histo->Write();
   CherenkovLatchADCHisto->Write();

   cdFixedComputed->cd();
   Mcp12TimeAdcHisto->Write();
   Mcp34TimeAdcHisto->Write();
   McpTofHisto->Write();
   
   cdFixedOthers->cd();
   PhVsTimeAdc1Histo->Write();
   PhVsTimeAdc2Histo->Write();
   PhVsTimeAdc3Histo->Write();
   PhVsTimeAdc4Histo->Write();
   BufferSeqHisto->Write();
   BufferStartHRHisto->Write();
   BufferEndHRHisto->Write();
   BufferDelayHisto->Write();
   EventNumberVsTimeHisto->Write();
*/
   
   //Write histos into image files
   
   //create directories
   string OutImageStr = OutImage;
   string ImageName;
   string cmd = "mkdir -p " + OutImageStr + "FixedHistos/DataFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/ComputedFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/OtherFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/DataCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/ComputedCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/OtherCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/DataEventsHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/ComputedEventsHistos";
   system(cmd.c_str());
   
   TCanvas *c1 = new TCanvas("c1","c1",1);
   
/*   //Fixed scale histos
   BufferSeqHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferSeqHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferStartHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferEndHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferDelayHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferDelayHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberVsTimeHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/EventNumberVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
  */ 

   cout << "Now creating .gif image files for histograms" << endl;

   RunNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RunTypeHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RealTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRampTimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/LrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/HrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   Mcp1TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   nScaler1Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/nScaler1Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/nScaler2Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler3Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/nScaler3Histo.gif";
   c1->Print(ImageName.c_str());
   CherenkovLatchADCHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/CherenkovLatchADCHisto.gif";
   c1->Print(ImageName.c_str());
   
   PhVsTimeAdc1Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc1Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc2Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc3Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc3Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc4Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc4Histo.gif";
   c1->Print(ImageName.c_str());
   
   //PhVsTimeAdc1Histo->SetMarkerStyle(20);
   PhVsTimeAdc1Histo->SetMarkerColor(kGreen);
   PhVsTimeAdc1Histo->Draw();
   //PhVsTimeAdc2Histo->SetMarkerStyle(21);
   PhVsTimeAdc2Histo->SetMarkerColor(kBlue);
   PhVsTimeAdc2Histo->Draw("same");
   //PhVsTimeAdc3Histo->SetMarkerStyle(22);
   PhVsTimeAdc3Histo->SetMarkerColor(kCyan);
   PhVsTimeAdc3Histo->Draw("same");
   //PhVsTimeAdc4Histo->SetMarkerStyle(23);
   PhVsTimeAdc4Histo->SetMarkerColor(kRed);
   PhVsTimeAdc4Histo->Draw("same");
   PhVsTimeAdc1Histo->SetMarkerStyle(2);
   PhVsTimeAdc2Histo->SetMarkerStyle(2);
   PhVsTimeAdc3Histo->SetMarkerStyle(2);
   PhVsTimeAdc4Histo->SetMarkerStyle(2);
   legend->AddEntry(PhVsTimeAdc1Histo,"ADC1","p");
   legend->AddEntry(PhVsTimeAdc2Histo,"ADC2","p");
   legend->AddEntry(PhVsTimeAdc3Histo,"ADC3","p");
   legend->AddEntry(PhVsTimeAdc4Histo,"ADC4","p");
   legend->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdcAll.gif";
   c1->Print(ImageName.c_str());

   c1->SetLogy();
   Mcp12TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTofHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFiredHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   LrRampTimeAdcDiffHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/LrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdcDiffHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/HrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());
   
   
      //Custom scale histos
   
   RunNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RunType_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeHR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RealTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRampTimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/LrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/HrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy();
   Mcp1TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   nScaler1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/nScaler1Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/nScaler2Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler3_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/nScaler3Histo.gif";
   c1->Print(ImageName.c_str());
   CherenkovLatchADC_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/CherenkovLatchADCHisto.gif";
   c1->Print(ImageName.c_str());
   
   PhVsTimeAdc1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc1Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc2Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc3_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc3Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc4_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc4Histo.gif";
   c1->Print(ImageName.c_str());
   RampVsTime_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/RampVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
   

   c1->SetLogy();
   Mcp12TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTof_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFired_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   c1->SetLogy(0);
   LrRampTimeAdcDiff_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/LrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdcDiff_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/HrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());


      //Vs EventNumber histos
   
   RunType_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   RunNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeLR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RealTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   RealTimeHR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RealTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   LrRampTimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/LrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/HrRampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   nScaler1_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/nScaler1Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler2_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/nScaler2Histo.gif";
   c1->Print(ImageName.c_str());
   nScaler3_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/nScaler3Histo.gif";
   c1->Print(ImageName.c_str());
   CherenkovLatchADC_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/CherenkovLatchADCHisto.gif";
   c1->Print(ImageName.c_str());

   Mcp12TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTof_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFired_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   LrRampTimeAdcDiff_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/LrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());
   HrRampTimeAdcDiff_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/HrRampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());

   
   FileOut->Write();
   FileOut->Close();
}
