#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "TFile.h"
#include "TTree.h"
#include "TNtuple.h"
#include "Riostream.h"

//Pedestrals for Mcp ADCs
#define MCP1PHMIN 13
#define MCP1TIMEMIN 1
#define MCP2PHMIN 9
#define MCP2TIMEMIN 1
#define MCP3PHMIN 21
#define MCP3TIMEMIN 1
#define MCP4PHMIN 15
#define MCP4TIMEMIN 1

#define TAB '\t'

//Purpose : This program reads an ascii file formatted as indicated in tupleFormat.txt
//	    and outputs the data in a root file.

//Modified May 14 2008 Emilien Chapon

//Modified May 22 2008 Emilien Chapon: support for new type of data

using namespace std;



TTree* convert(const char* infile, const char* outfile)	{

	ifstream in;
	in.open(infile);

	int tupleCount=0;

	//Following structure contains data read from ascii ntuple
	typedef struct {float RunNumber, RunType, EventNumber, BufferSequenceNumber, RealTimeLR, RealTimeHR, LrRampTimeAdc,
		HrRampTimeAdc, Mcp1TimeAdc, Mcp1PhAdc, Mcp2TimeAdc, Mcp2PhAdc, Mcp3TimeAdc, Mcp3PhAdc, Mcp4TimeAdc, 
		Mcp4PhAdc, nScaler1, nScaler2, nScaler3, CherenkovLatchADC;} DATA;

	//Following structure contains data that we compute from the variables of the ascii ntuple
	typedef struct {
		float 	   UniqueEventNumber;	//In case of the analyse of several runs, gives a unique number to each event
		float	   Mcp12TimeAdc;	//AdcTime2 - AdcTime1 when both MCPs 1 and 2 were fired
		float	   Mcp34TimeAdc;	//AdcTime4 - AdcTime3 when both MCPs 3 and 4 were fired
		float	   McpTof;		//Time of Flight calculated from Mcp times
		float	   IsMcp1Fired;		//1 if Mcp1 was fired, else 0
		float	   IsMcp2Fired;		//1 if Mcp2 was fired, else 0
		float	   IsMcp3Fired;		//1 if Mcp3 was fired, else 0
		float	   IsMcp4Fired;		//1 if Mcp4 was fired, else 0
		float 	   NMcpFired; 		//Number of MCPs fired
		float	   LrRampTimeAdcDiff;	//Difference between low res ramp time adc counts for two successive events
		float	   HrRampTimeAdcDiff;	//Difference between hi res ramp time adc counts for two successive events
	} COMPUTED;

	static DATA data;
	static COMPUTED computed;
	
	TFile *f = new TFile(outfile,"RECREATE");

	TTree *ntuple = new TTree("ntuple","root tree from ascii file");
	ntuple->Branch("data",&data.RunNumber,"RunNumber:RunType:EventNumber:BufferSequenceNumber:RealTimeLR:RealTimeHR:LrRampTimeAdc:HrRampTimeAdc:Mcp1TimeAdc:Mcp1PhAdc:Mcp2TimeAdc:Mcp2PhAdc:Mcp3TimeAdc:Mcp3PhAdc:Mcp4TimeAdc:Mcp4PhAdc:nScaler1:nScaler2:nScaler3:CherenkovLatchADC");
	ntuple->Branch("computed",&computed.UniqueEventNumber,"UniqueEventNumber:Mcp12TimeAdc:Mcp34TimeAdc:McpTof:IsMcp1Fired:IsMcp2Fired:IsMcp3Fired:IsMcp4Fired:NMcpFired:LrRampTimeAdcDiff:HrRampTimeAdcDiff");

//We do not take into account the first line, which contains the names of the fields
string null;
in >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null;

//Ask if we want to analyse the full ntuple, or only one part (for online verification of data quality)
cout << "Do you want to analyse full input ntuple (y/n) ?" << endl;
string answer;
bool fullAnalyse;
int firstLine,lastLine;
cin >> answer;
if (answer == "n") {
	fullAnalyse=false;
	cout << "First line ?" << endl;
	cin >> firstLine;
	cout << "Last line (0=last) ? You can use \"wc -l ntuple.ascii\" in a prompt to count lines." << endl;
	cin >> lastLine;
}
else {
	fullAnalyse=true;
	firstLine=1;
	lastLine=0;
}

	float LrRampTimeAdcPrev=0.;	//will be useful to calculate LrRampTimeAdcDiff
	float HrRampTimeAdcPrev=0.;	//will be useful to calculate HrRampTimeAdcDiff

   //Beginning of forever loop
   cout << "Now reading the ascii ntuple" << endl;
   while(1) {
	//Check if we are within the range of lines entered by user, in case of partial analyse
	tupleCount++;
	if (lastLine !=0 && tupleCount > lastLine) break;

	//Read data from ascii file
	in >> data.RunNumber >> data.RunType >> data.EventNumber >> data.BufferSequenceNumber >> data.RealTimeLR >>
	   data.RealTimeHR >> data.LrRampTimeAdc >> data.HrRampTimeAdc >> data.Mcp1TimeAdc >> data.Mcp1PhAdc >> 
	   data.Mcp2TimeAdc >> data.Mcp2PhAdc >> data.Mcp3TimeAdc >> data.Mcp3PhAdc >> data.Mcp4TimeAdc >> data.Mcp4PhAdc >> 
	   data.nScaler1 >> data.nScaler2 >> data.nScaler3 >> data.CherenkovLatchADC;
	

	if (!in.good()) break;

	//Calculate computed variables
	computed.UniqueEventNumber = tupleCount;
	computed.IsMcp1Fired= (data.Mcp1TimeAdc > MCP1TIMEMIN || data.Mcp1PhAdc > MCP1PHMIN) ? 1. : 0.;
	computed.IsMcp2Fired= (data.Mcp2TimeAdc > MCP2TIMEMIN || data.Mcp2PhAdc > MCP2PHMIN) ? 1. : 0.;
	computed.IsMcp3Fired= (data.Mcp3TimeAdc > MCP3TIMEMIN || data.Mcp3PhAdc > MCP3PHMIN) ? 1. : 0.;
	computed.IsMcp4Fired= (data.Mcp4TimeAdc > MCP4TIMEMIN || data.Mcp4PhAdc > MCP4PHMIN) ? 1. : 0.;
	computed.NMcpFired= computed.IsMcp1Fired + computed.IsMcp2Fired + computed.IsMcp3Fired + computed.IsMcp4Fired;
	computed.Mcp12TimeAdc = ((int) computed.IsMcp1Fired && (int) computed.IsMcp2Fired) ? data.Mcp2TimeAdc - data.Mcp1TimeAdc : 0.;
	computed.Mcp34TimeAdc = ((int) computed.IsMcp3Fired && (int) computed.IsMcp4Fired) ? data.Mcp4TimeAdc - data.Mcp3TimeAdc : 0.;
	computed.McpTof = (computed.NMcpFired == 4.) ? (data.Mcp4TimeAdc + data.Mcp3TimeAdc)/2 - (data.Mcp2TimeAdc + data.Mcp1TimeAdc)/2 : 0.;
	computed.LrRampTimeAdcDiff=data.LrRampTimeAdc-LrRampTimeAdcPrev;
	computed.HrRampTimeAdcDiff=data.HrRampTimeAdc-HrRampTimeAdcPrev;
	LrRampTimeAdcPrev=data.LrRampTimeAdc;
	HrRampTimeAdcPrev=data.HrRampTimeAdc;


	if (tupleCount >= firstLine) ntuple->Fill();
/*	if (tupleCount < 5) {
	   cout << " " << data.RunNumber << " " << data.RunType << " " << data.EventNumber << " " << data.BufferSequenceNumber << " " << data.RealTimeLR << " " << data.RealTimeHR << " " << data.LrRampTimeAdc << " " << data.HrRampTimeAdc << " " << data.Mcp1TimeAdc << " " << data.Mcp1PhAdc << " " << data.Mcp2TimeAdc << " " << data.Mcp2PhAdc << " " << data.Mcp3TimeAdc << " " << data.Mcp3PhAdc << " " << data.Mcp4TimeAdc << " " << data.Mcp4PhAdc << " " << data.nScaler1 << " " << data.nScaler2 << " " << data.nScaler3 << " " << data.CherenkovLatchADC << endl; 
	}
*/

   } // end of forever loop

	if (fullAnalyse) cout << "Found " << tupleCount  << " lines."  << endl;
	else cout << "Read " << tupleCount - firstLine << " lines." << endl;

	in.close();
	f->Write();
//	f->Close();
	return ntuple;
}
