#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "Riostream.h"
#define PSECNtupleAna_cxx
#include "PSECNtupleAna.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <string>
#include <math.h>

void PSECNtupleAna::Loop(const char* OutHistos, const char* OutSummary, const char* OutImage)
{
//   In a ROOT session, you can do:
//      Root > .L PSECNtupleAna.C
//      Root > PSECNtupleAna t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
	
	
	//Structure of this (really) big function:
	//- declaration of means, rms
	//- declaration of fixed scale histos
	//- declaration of custom scale histos, and automatic creation by TTree::Draw
	//- Loop over events
	// * Fill histos
	// * modify means and rms
	//- final calculation of means and rms
	//- write the summary
	//- write images of histos

	//Declaration of means and RMSs
	double         mean_RunNumber=0.;
	double         mean_RunType=0.;
	double         mean_EventNumber=0.;
	double         mean_BufferSequenceNumber=0.;
	double         mean_BufferStartTimeLR=0.;
	double         mean_BufferStartTimeHR=0.;
	double         mean_BufferEndTimeLR=0.;
	double         mean_BufferEndTimeHR=0.;
	double         mean_RampTimeAdc=0.;
	double         mean_Mcp1TimeAdc=0.;
	double         mean_Mcp1PhAdc=0.;
	double         mean_Mcp2TimeAdc=0.;
	double         mean_Mcp2PhAdc=0.;
	double         mean_Mcp3TimeAdc=0.;
	double         mean_Mcp3PhAdc=0.;
	double         mean_Mcp4TimeAdc=0.;
	double         mean_Mcp4PhAdc=0.;
	double         mean_Trig1TimeTdc=0.;
	double         mean_Trig2TimeTdc=0.;
	double         mean_CherenkovTimeTdc=0.;
	double         rms_RunNumber=0.;
	double         rms_RunType=0.;
	double         rms_EventNumber=0.;
	double         rms_BufferSequenceNumber=0.;
	double         rms_BufferStartTimeLR=0.;
	double         rms_BufferStartTimeHR=0.;
	double         rms_BufferEndTimeLR=0.;
	double         rms_BufferEndTimeHR=0.;
	double         rms_RampTimeAdc=0.;
	double         rms_Mcp1TimeAdc=0.;
	double         rms_Mcp1PhAdc=0.;
	double         rms_Mcp2TimeAdc=0.;
	double         rms_Mcp2PhAdc=0.;
	double         rms_Mcp3TimeAdc=0.;
	double         rms_Mcp3PhAdc=0.;
	double         rms_Mcp4TimeAdc=0.;
	double         rms_Mcp4PhAdc=0.;
	double         rms_Trig1TimeTdc=0.;
	double         rms_Trig2TimeTdc=0.;
	double         rms_CherenkovTimeTdc=0.;

	//Idem for computed data
	double	       mean_Mcp12TimeAdc=0.;
	double	       mean_Mcp34TimeAdc=0.;
	double	       mean_McpTof=0.;		//This is the mean of time of flight above all entries
	double	       mean_McpTof_true=0.;	//This is the mean of time of flight considering only entries where 4 MCPs were fired
	double	       n_McpTof_true=0.;	//Number of entries to consider for the previous mean
	double	       mean_IsMcp1Fired=0.;
	double	       mean_IsMcp2Fired=0.;
	double	       mean_IsMcp3Fired=0.;
	double	       mean_IsMcp4Fired=0.;
	double	       mean_NMcpFired=0.;
	double	       mean_RampTimeAdcDiff;
	double	       rms_Mcp12TimeAdc=0.;
	double	       rms_Mcp34TimeAdc=0.;
	double	       rms_McpTof=0.;
	double	       rms_McpTof_true=0.;
	double	       rms_IsMcp1Fired=0.;
	double	       rms_IsMcp2Fired=0.;
	double	       rms_IsMcp3Fired=0.;
	double	       rms_IsMcp4Fired=0.;
	double	       rms_NMcpFired=0.;
	double	       rms_RampTimeAdcDiff;

	TFile *FileOut = new TFile(OutHistos,"RECREATE");
	
	TLegend *legend=new TLegend(0.6,0.65,0.88,0.85);
	legend->SetTextFont(72);
	legend->SetTextSize(0.03);
	
	ofstream summary (OutSummary, ios::out);

	//TCanvas *canvas = new TCanvas("canvas","nearline analysis",200,10,600,400);
	//canvas->SetGrid();
	
   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntriesFast();

   //Fixed scale histograms
   //1D histograms of all branches
   TDirectory *cdFixedHistos = FileOut->mkdir("HistosFixedScale");
   cdFixedHistos->cd();
   TDirectory *cdFixedData = cdFixedHistos->mkdir("dataFixedHistos");
   cdFixedData->cd();
   TH1F *RunNumberHisto = new TH1F("RunNumber","RunNumber",100,0.,100.);
   TH1F *RunTypeHisto = new TH1F("RunType","RunType",100,0.,100.);
   TH1F *EventNumberHisto = new TH1F("EventNumber","EventNumber",1000,0.,100000.);
   TH1F *BufferSequenceNumberHisto = new TH1F("BufferSequenceNumber","BufferSequenceNumber",100,0.,1024.);
   TH1F *BufferStartTimeLRHisto = new TH1F("BufferStartTimeLR","BufferStartTimeLR",100,0.,2e9);
   TH1F *BufferStartTimeHRHisto = new TH1F("BufferStartTimeHR","BufferStartTimeHR",100,0.,1e6);
   TH1F *BufferEndTimeLRHisto = new TH1F("BufferEndTimeLR","BufferEndTimeLR",100,0.,2e9);
   TH1F *BufferEndTimeHRHisto = new TH1F("BufferEndTimeHR","BufferEndTimeHR",100,0.,1e6);
   TH1F *RampTimeAdcHisto = new TH1F("RampTimeAdc","RampTimeAdc",100,0.,1024.);
   TH1F *Mcp1TimeAdcHisto = new TH1F("Mcp1TimeAdc","Mcp1TimeAdc",100,0.,16384.);
   TH1F *Mcp1PhAdcHisto = new TH1F("Mcp1PhAdc","Mcp1PhAdc",100,0.,16384.);
   TH1F *Mcp2TimeAdcHisto = new TH1F("Mcp2TimeAdc","Mcp2TimeAdc",100,0.,16384.);
   TH1F *Mcp2PhAdcHisto = new TH1F("Mcp2PhAdc","Mcp2PhAdc",100,0.,16384.);
   TH1F *Mcp3TimeAdcHisto = new TH1F("Mcp3TimeAdc","Mcp3TimeAdc",100,0.,16384.);
   TH1F *Mcp3PhAdcHisto = new TH1F("Mcp3PhAdc","Mcp3PhAdc",100,0.,16384.);
   TH1F *Mcp4TimeAdcHisto = new TH1F("Mcp4TimeAdc","Mcp4TimeAdc",100,0.,16384.);
   TH1F *Mcp4PhAdcHisto = new TH1F("Mcp4PhAdc","Mcp4PhAdc",100,0.,16384.);
   TH1F *Trig1TimeTdcHisto = new TH1F("Trig1TimeTdc","Trig1TimeTdc",100,0.,1024.);
   TH1F *Trig2TimeTdcHisto = new TH1F("Trig2TimeTdc","Trig2TimeTdc",100,0.,1024.);
   TH1F *CherenkovTimeTdcHisto = new TH1F("CherenkovTimeTdc","CherenkovTimeTdc",100,0.,1024.);
   
;

   //1D histograms of computed variables
   cdFixedHistos->cd();
   TDirectory *cdFixedComputed = cdFixedHistos->mkdir("ComputedFixedHistos");
   cdFixedComputed->cd();
   TH1F *Mcp12TimeAdcHisto = new TH1F("Mcp12TimeAdc","Mcp12TimeAdc",100,0.,16384.);
   TH1F *Mcp34TimeAdcHisto = new TH1F("Mcp34TimeAdc","Mcp34TimeAdc",100,0.,16384.);
   TH1F *McpTofHisto = new TH1F("McpTof","McpTof",100,0.,16384.);
   TH1F *NMcpFiredHisto = new TH1F("NMcpFiredHisto","NMcpFiredHisto",4,0.,4.);
   TH1F *RampTimeAdcDiffHisto = new TH1F("RampTimeAdcDiff","RampTimeAdcDiff",100,0.,1024.);

   //2D histograms of charge vs time of flight by adc
   cdFixedHistos->cd();
   TDirectory *cdFixedOthers = cdFixedHistos->mkdir("OtherFixedHistos");
   cdFixedOthers->cd();
   TH2F *PhVsTimeAdc1Histo = new TH2F("PhVsTimeAdc1","Ph Vs Time for ADC 1",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc2Histo = new TH2F("PhVsTimeAdc2","Ph Vs Time for ADC 2",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc3Histo = new TH2F("PhVsTimeAdc3","Ph Vs Time for ADC 3",100,0.,16384.,100,0.,16384.);
   TH2F *PhVsTimeAdc4Histo = new TH2F("PhVsTimeAdc4","Ph Vs Time for ADC 4",100,0.,16384.,100,0.,16384.);
   /*//Other histos
   TH2F *BufferSeqHisto = new TH2F("BufferSequenceN","BufferSequenceNumber vs EventNumber",100,0.,nentries+10.,100,0.,300.);
   TH2F *BufferStartHRHisto = new TH2F("BufferStartHR","BufferStartTimeHR vs BufferStartTimeLR",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferEndHRHisto = new TH2F("BufferEndHR","BufferEndTimeHR vs BufferEndTimeLR",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferDelayHisto = new TH2F("BufferDelay","BufferDelay vs EventNumber",100,0.,nentries+10.,100,0.,250.);
   TH2F *EventNumberVsTimeHisto = new TH2F("EventNumberVsTime","EventNumber vs Start Time",100,1.2111586e9,1.2131589e9,100,0.,nentries+10.);
*/
   cout << "Now root is creating some histograms setting automatically the scales" << endl;

   //Automatic scale histograms
   //1D Histograms of all branches
   TDirectory *cdCustomHistos = FileOut->mkdir("HistosCustomScale");
   cdCustomHistos->cd();
   TDirectory *cdCustomData = cdCustomHistos->mkdir("dataCustomHistos");
   cdCustomData->cd();
   fChain->Draw("RunNumber>>RunNumber_C");
   TH1F *RunNumber_CHisto = (TH1F*)gDirectory->Get("RunNumber_C");
   fChain->Draw("RunType>>RunType_C");
   TH1F *RunType_CHisto = (TH1F*)gDirectory->Get("RunType_C");
   fChain->Draw("EventNumber>>EventNumber_C");
   TH1F *EventNumber_CHisto = (TH1F*)gDirectory->Get("EventNumber_C");
   fChain->Draw("BufferSequenceNumber>>BufferSequenceNumber_C");
   TH1F *BufferSequenceNumber_CHisto = (TH1F*)gDirectory->Get("BufferSequenceNumber_C");
   fChain->Draw("BufferStartTimeLR>>BufferStartTimeLR_C");
   TH1F *BufferStartTimeLR_CHisto = (TH1F*)gDirectory->Get("BufferStartTimeLR_C");
   fChain->Draw("BufferStartTimeHR>>BufferStartTimeHR_C");
   TH1F *BufferStartTimeHR_CHisto = (TH1F*)gDirectory->Get("BufferStartTimeHR_C");
   fChain->Draw("BufferEndTimeLR>>BufferEndTimeLR_C");
   TH1F *BufferEndTimeLR_CHisto = (TH1F*)gDirectory->Get("BufferEndTimeLR_C");
   fChain->Draw("BufferEndTimeHR>>BufferEndTimeHR_C");
   TH1F *BufferEndTimeHR_CHisto = (TH1F*)gDirectory->Get("BufferEndTimeHR_C");
   fChain->Draw("RampTimeAdc>>RampTimeAdc_C");
   TH1F *RampTimeAdc_CHisto = (TH1F*)gDirectory->Get("RampTimeAdc_C");
   fChain->Draw("Mcp1TimeAdc>>Mcp1TimeAdc_C");
   TH1F *Mcp1TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp1TimeAdc_C");
   fChain->Draw("Mcp1PhAdc>>Mcp1PhAdc_C");
   TH1F *Mcp1PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp1PhAdc_C");
   fChain->Draw("Mcp2TimeAdc>>Mcp2TimeAdc_C");
   TH1F *Mcp2TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp2TimeAdc_C");
   fChain->Draw("Mcp2PhAdc>>Mcp2PhAdc_C");
   TH1F *Mcp2PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp2PhAdc_C");
   fChain->Draw("Mcp3TimeAdc>>Mcp3TimeAdc_C");
   TH1F *Mcp3TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp3TimeAdc_C");
   fChain->Draw("Mcp3PhAdc>>Mcp3PhAdc_C");
   TH1F *Mcp3PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp3PhAdc_C");
   fChain->Draw("Mcp4TimeAdc>>Mcp4TimeAdc_C");
   TH1F *Mcp4TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp4TimeAdc_C");
   fChain->Draw("Mcp4PhAdc>>Mcp4PhAdc_C");
   TH1F *Mcp4PhAdc_CHisto = (TH1F*)gDirectory->Get("Mcp4PhAdc_C");
   fChain->Draw("Trig1TimeTdc>>Trig1TimeTdc_C");
   TH1F *Trig1TimeTdc_CHisto = (TH1F*)gDirectory->Get("Trig1TimeTdc_C");
   fChain->Draw("Trig2TimeTdc>>Trig2TimeTdc_C");
   TH1F *Trig2TimeTdc_CHisto = (TH1F*)gDirectory->Get("Trig2TimeTdc_C");
   fChain->Draw("CherenkovTimeTdc>>CherenkovTimeTdc_C");
   TH1F *CherenkovTimeTdc_CHisto = (TH1F*)gDirectory->Get("CherenkovTimeTdc_C");
   
   ;

   //1D histograms of computed variables
   cdCustomHistos->cd();
   TDirectory *cdCustomComputed = cdCustomHistos->mkdir("ComputedCustomHistos");
   cdCustomComputed->cd();
   fChain->Draw("Mcp12TimeAdc>>Mcp12TimeAdc_C");
   TH1F *Mcp12TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp12TimeAdc_C");
   fChain->Draw("Mcp34TimeAdc>>Mcp34TimeAdc_C");
   TH1F *Mcp34TimeAdc_CHisto = (TH1F*)gDirectory->Get("Mcp34TimeAdc_C");
   fChain->Draw("McpTof>>McpTof_C");
   TH1F *McpTof_CHisto = (TH1F*)gDirectory->Get("McpTof_C");
   fChain->Draw("NMcpFired>>NMcpFired_C");
   TH1F *NMcpFired_CHisto = (TH1F*)gDirectory->Get("NMcpFired_C");
   fChain->Draw("RampTimeAdcDiff>>RampTimeAdcDiff_C");
   TH1F *RampTimeAdcDiff_CHisto = (TH1F*)gDirectory->Get("RampTimeAdcDiff_C");

   //2D histograms of charge vs time of flight by adc
   cdCustomHistos->cd();
   TDirectory *cdCustomOthers = cdCustomHistos->mkdir("OtherCustomHistos");
   cdCustomOthers->cd();
   fChain->Draw("Mcp1PhAdc:Mcp1TimeAdc>>PhVsTimeAdc1_C");
   TH2F *PhVsTimeAdc1_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc1_C");
   fChain->Draw("Mcp2PhAdc:Mcp2TimeAdc>>PhVsTimeAdc2_C");
   TH2F *PhVsTimeAdc2_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc2_C");
   fChain->Draw("Mcp3PhAdc:Mcp3TimeAdc>>PhVsTimeAdc3_C");
   TH2F *PhVsTimeAdc3_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc3_C");
   fChain->Draw("Mcp4PhAdc:Mcp4TimeAdc>>PhVsTimeAdc4_C");
   TH2F *PhVsTimeAdc4_CHisto = (TH2F*)gDirectory->Get("PhVsTimeAdc4_C");
   //2D histogram of RampTimeAdc vs time
   fChain->Draw("RampTimeAdc:BufferEndTimeHR>>RampVsTime_C(1000)");
   TH2F *RampVsTime_CHisto = (TH2F*)gDirectory->Get("RampVsTime_C");

   //2D histograms of all variables vs unique event number
   TDirectory *cdEventsHistos = FileOut->mkdir("HistosVsEvents");
   cdEventsHistos->cd();
   TDirectory *cdEventsData = cdEventsHistos->mkdir("dataVsEventsHistos");
   cdEventsData->cd();
   fChain->Draw("RunType:UniqueEventNumber>>RunType_VE");
   TH2F *RunType_VEHisto = (TH2F*)gDirectory->Get("RunType_VE");
   fChain->Draw("RunNumber:UniqueEventNumber>>RunNumber_VE");
   TH2F *RunNumber_VEHisto = (TH2F*)gDirectory->Get("RunNumber_VE");
   fChain->Draw("EventNumber:UniqueEventNumber>>EventNumber_VE");
   TH2F *EventNumber_VEHisto = (TH2F*)gDirectory->Get("EventNumber_VE");
   fChain->Draw("BufferSequenceNumber:UniqueEventNumber>>BufferSequenceNumber_VE");
   TH2F *BufferSequenceNumber_VEHisto = (TH2F*)gDirectory->Get("BufferSequenceNumber_VE");
   fChain->Draw("BufferStartTimeLR:UniqueEventNumber>>BufferStartTimeLR_VE");
   TH2F *BufferStartTimeLR_VEHisto = (TH2F*)gDirectory->Get("BufferStartTimeLR_VE");
   fChain->Draw("BufferStartTimeHR:UniqueEventNumber>>BufferStartTimeHR_VE");
   TH2F *BufferStartTimeHR_VEHisto = (TH2F*)gDirectory->Get("BufferStartTimeHR_VE");
   fChain->Draw("BufferEndTimeLR:UniqueEventNumber>>BufferEndTimeLR_VE");
   TH2F *BufferEndTimeLR_VEHisto = (TH2F*)gDirectory->Get("BufferEndTimeLR_VE");
   fChain->Draw("BufferEndTimeHR:UniqueEventNumber>>BufferEndTimeHR_VE");
   TH2F *BufferEndTimeHR_VEHisto = (TH2F*)gDirectory->Get("BufferEndTimeHR_VE");
   fChain->Draw("RampTimeAdc:UniqueEventNumber>>RampTimeAdc_VE");
   TH2F *RampTimeAdc_VEHisto = (TH2F*)gDirectory->Get("RampTimeAdc_VE");
   fChain->Draw("Mcp1TimeAdc:UniqueEventNumber>>Mcp1TimeAdc_VE");
   TH2F *Mcp1TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp1TimeAdc_VE");
   fChain->Draw("Mcp1PhAdc:UniqueEventNumber>>Mcp1PhAdc_VE");
   TH2F *Mcp1PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp1PhAdc_VE");
   fChain->Draw("Mcp2TimeAdc:UniqueEventNumber>>Mcp2TimeAdc_VE");
   TH2F *Mcp2TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp2TimeAdc_VE");
   fChain->Draw("Mcp2PhAdc:UniqueEventNumber>>Mcp2PhAdc_VE");
   TH2F *Mcp2PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp2PhAdc_VE");
   fChain->Draw("Mcp3TimeAdc:UniqueEventNumber>>Mcp3TimeAdc_VE");
   TH2F *Mcp3TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp3TimeAdc_VE");
   fChain->Draw("Mcp3PhAdc:UniqueEventNumber>>Mcp3PhAdc_VE");
   TH2F *Mcp3PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp3PhAdc_VE");
   fChain->Draw("Mcp4TimeAdc:UniqueEventNumber>>Mcp4TimeAdc_VE");
   TH2F *Mcp4TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp4TimeAdc_VE");
   fChain->Draw("Mcp4PhAdc:UniqueEventNumber>>Mcp4PhAdc_VE");
   TH2F *Mcp4PhAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp4PhAdc_VE");
   fChain->Draw("Trig1TimeTdc:UniqueEventNumber>>Trig1TimeTdc_VE");
   TH2F *Trig1TimeTdc_VEHisto = (TH2F*)gDirectory->Get("Trig1TimeTdc_VE");
   fChain->Draw("Trig2TimeTdc:UniqueEventNumber>>Trig2TimeTdc_VE");
   TH2F *Trig2TimeTdc_VEHisto = (TH2F*)gDirectory->Get("Trig2TimeTdc_VE");
   fChain->Draw("CherenkovTimeTdc:UniqueEventNumber>>CherenkovTimeTdc_VE");
   TH2F *CherenkovTimeTdc_VEHisto = (TH2F*)gDirectory->Get("CherenkovTimeTdc_VE");
   
   cdEventsHistos->cd();
   TDirectory *cdEventsComputed = cdEventsHistos->mkdir("ComputedVsEventsHistos");
   cdEventsComputed->cd();
   fChain->Draw("Mcp12TimeAdc:UniqueEventNumber>>Mcp12TimeAdc_VE");
   TH2F *Mcp12TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp12TimeAdc_VE");
   fChain->Draw("Mcp34TimeAdc:UniqueEventNumber>>Mcp34TimeAdc_VE");
   TH2F *Mcp34TimeAdc_VEHisto = (TH2F*)gDirectory->Get("Mcp34TimeAdc_VE");
   fChain->Draw("McpTof:UniqueEventNumber>>McpTof_VE");
   TH2F *McpTof_VEHisto = (TH2F*)gDirectory->Get("McpTof_VE");
   fChain->Draw("NMcpFired:UniqueEventNumber>>NMcpFired_VE");
   TH2F *NMcpFired_VEHisto = (TH2F*)gDirectory->Get("NMcpFired_VE");
   fChain->Draw("RampTimeAdcDiff:UniqueEventNumber>>RampTimeAdcDiff_VE");
   TH2F *RampTimeAdcDiff_VEHisto = (TH2F*)gDirectory->Get("RampTimeAdcDiff_VE");
   
   cdFixedHistos->cd();

   Long64_t nbytes = 0, nb = 0;
   
   float RunNumber0 = -999.;

   //***************************************************************************//
   //					LOOP					//
   //***************************************************************************//
   cout << "Looping over events" << endl;
   for (Long64_t jentry=0; jentry<nentries;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
	fChain->Fill();
      
      if (RunNumber0 == -999.) RunNumber0 = data_RunNumber;
      



      //Calculate mean and rms of each leaf
      mean_RunNumber += data_RunNumber;
      mean_RunType += data_RunType;
      mean_EventNumber += data_EventNumber;
      mean_BufferSequenceNumber += data_BufferSequenceNumber;
      mean_BufferStartTimeLR += data_BufferStartTimeLR;
      mean_BufferStartTimeHR += data_BufferStartTimeHR;
      mean_BufferEndTimeLR += data_BufferEndTimeLR;
      mean_BufferEndTimeHR += data_BufferEndTimeHR;
      mean_RampTimeAdc += data_RampTimeAdc;
      mean_Mcp1TimeAdc += data_Mcp1TimeAdc;
      mean_Mcp1PhAdc += data_Mcp1PhAdc;
      mean_Mcp2TimeAdc += data_Mcp2TimeAdc;
      mean_Mcp2PhAdc += data_Mcp2PhAdc;
      mean_Mcp3TimeAdc += data_Mcp3TimeAdc;
      mean_Mcp3PhAdc += data_Mcp3PhAdc;
      mean_Mcp4TimeAdc += data_Mcp4TimeAdc;
      mean_Mcp4PhAdc += data_Mcp4PhAdc;
      mean_Trig1TimeTdc += data_Trig1TimeTdc;
      mean_Trig2TimeTdc += data_Trig2TimeTdc;
      mean_CherenkovTimeTdc += data_CherenkovTimeTdc;
       rms_RunNumber += data_RunNumber*data_RunNumber;
       rms_RunType += data_RunType*data_RunType;
       rms_EventNumber += data_EventNumber*data_EventNumber;
       rms_BufferSequenceNumber += data_BufferSequenceNumber*data_BufferSequenceNumber;
       rms_BufferStartTimeLR += data_BufferStartTimeLR*data_BufferStartTimeLR;
       rms_BufferStartTimeHR += data_BufferStartTimeHR*data_BufferStartTimeHR;
       rms_BufferEndTimeLR += data_BufferEndTimeLR*data_BufferEndTimeLR;
       rms_BufferEndTimeHR += data_BufferEndTimeHR*data_BufferEndTimeHR;
       rms_RampTimeAdc += data_RampTimeAdc*data_RampTimeAdc;
       rms_Mcp1TimeAdc += data_Mcp1TimeAdc*data_Mcp1TimeAdc;
       rms_Mcp1PhAdc += data_Mcp1PhAdc*data_Mcp1PhAdc;
       rms_Mcp2TimeAdc += data_Mcp2TimeAdc*data_Mcp2TimeAdc;
       rms_Mcp2PhAdc += data_Mcp2PhAdc*data_Mcp2PhAdc;
       rms_Mcp3TimeAdc += data_Mcp3TimeAdc*data_Mcp3TimeAdc;
       rms_Mcp3PhAdc += data_Mcp3PhAdc*data_Mcp3PhAdc;
       rms_Mcp4TimeAdc += data_Mcp4TimeAdc*data_Mcp4TimeAdc;
       rms_Mcp4PhAdc += data_Mcp4PhAdc*data_Mcp4PhAdc;
       rms_Trig1TimeTdc += data_Trig1TimeTdc*data_Trig1TimeTdc;
       rms_Trig2TimeTdc += data_Trig2TimeTdc*data_Trig2TimeTdc;
       rms_CherenkovTimeTdc += data_CherenkovTimeTdc*data_CherenkovTimeTdc;

	//Calculate mean and RMS for computed data
	mean_Mcp12TimeAdc += computed_Mcp12TimeAdc;
	mean_Mcp34TimeAdc += computed_Mcp34TimeAdc;
	mean_McpTof += computed_McpTof;
	mean_IsMcp1Fired += computed_IsMcp1Fired;
	mean_IsMcp2Fired += computed_IsMcp2Fired;
	mean_IsMcp3Fired += computed_IsMcp3Fired;
	mean_IsMcp4Fired += computed_IsMcp4Fired;
	mean_NMcpFired += computed_NMcpFired;
	mean_RampTimeAdcDiff += computed_RampTimeAdcDiff;
	rms_Mcp12TimeAdc += computed_Mcp12TimeAdc*computed_Mcp12TimeAdc;
	rms_Mcp34TimeAdc += computed_Mcp34TimeAdc*computed_Mcp34TimeAdc;
	rms_McpTof += computed_McpTof*computed_McpTof;
	rms_IsMcp1Fired += computed_IsMcp1Fired*computed_IsMcp1Fired;
	rms_IsMcp2Fired += computed_IsMcp2Fired*computed_IsMcp2Fired;
	rms_IsMcp3Fired += computed_IsMcp3Fired*computed_IsMcp3Fired;
	rms_IsMcp4Fired += computed_IsMcp4Fired*computed_IsMcp4Fired;
	rms_NMcpFired += computed_NMcpFired*computed_NMcpFired;
	rms_RampTimeAdcDiff += computed_RampTimeAdcDiff*computed_RampTimeAdcDiff;
	if(computed_NMcpFired == 4) {
		n_McpTof_true += 1;
		mean_McpTof_true += computed_McpTof;
		rms_McpTof_true += computed_McpTof*computed_McpTof;
	}
      
      //Fill Histos
      RunNumberHisto->Fill(data_RunNumber);
      RunTypeHisto->Fill(data_RunType);
      EventNumberHisto->Fill(data_EventNumber);
      BufferSequenceNumberHisto->Fill(data_BufferSequenceNumber);
      BufferStartTimeLRHisto->Fill(data_BufferStartTimeLR);
      BufferStartTimeHRHisto->Fill(data_BufferStartTimeHR);
      BufferEndTimeLRHisto->Fill(data_BufferEndTimeLR);
      BufferEndTimeHRHisto->Fill(data_BufferEndTimeHR);
      RampTimeAdcHisto->Fill(data_RampTimeAdc);
      Mcp1TimeAdcHisto->Fill(data_Mcp1TimeAdc);
      Mcp1PhAdcHisto->Fill(data_Mcp1PhAdc);
      Mcp2TimeAdcHisto->Fill(data_Mcp2TimeAdc);
      Mcp2PhAdcHisto->Fill(data_Mcp2PhAdc);
      Mcp3TimeAdcHisto->Fill(data_Mcp3TimeAdc);
      Mcp3PhAdcHisto->Fill(data_Mcp3PhAdc);
      Mcp4TimeAdcHisto->Fill(data_Mcp4TimeAdc);
      Mcp4PhAdcHisto->Fill(data_Mcp4PhAdc);
      Trig1TimeTdcHisto->Fill(data_Trig1TimeTdc);
      Trig2TimeTdcHisto->Fill(data_Trig2TimeTdc);
      CherenkovTimeTdcHisto->Fill(data_CherenkovTimeTdc);
      /*
      BufferSeqHisto->Fill(data_EventNumber,data_BufferSequenceNumber);
      BufferStartHRHisto->Fill(data_BufferStartTimeLR,data_BufferStartTimeHR);
      BufferEndHRHisto->Fill(data_BufferEndTimeLR,data_BufferEndTimeHR);
      BufferDelayHisto->Fill(data_EventNumber,data_BufferEndTimeLR+data_BufferEndTimeHR-data_BufferStartTimeLR-data_BufferStartTimeHR);
      EventNumberVsTimeHisto->Fill(data_BufferStartTimeLR+data_BufferStartTimeHR,data_EventNumber);
*/
      PhVsTimeAdc1Histo->Fill(data_Mcp1TimeAdc,data_Mcp1PhAdc);
      PhVsTimeAdc2Histo->Fill(data_Mcp2TimeAdc,data_Mcp2PhAdc);
      PhVsTimeAdc3Histo->Fill(data_Mcp3TimeAdc,data_Mcp3PhAdc);
      PhVsTimeAdc4Histo->Fill(data_Mcp4TimeAdc,data_Mcp4PhAdc);

      Mcp12TimeAdcHisto->Fill(computed_Mcp12TimeAdc);
      Mcp34TimeAdcHisto->Fill(computed_Mcp34TimeAdc);
      McpTofHisto->Fill(computed_McpTof);
      NMcpFiredHisto->Fill(computed_NMcpFired);
      RampTimeAdcDiffHisto->Fill(computed_RampTimeAdcDiff);

   }
   cout << "End of loop" << endl;
   //***************************************************************************//
   //				END OF LOOP					//
   //***************************************************************************//
   
   //Means and RMSs
   mean_RunNumber=mean_RunNumber/nentries;
   mean_RunType=   mean_RunType/nentries;
   mean_EventNumber=   mean_EventNumber/nentries;
   mean_BufferSequenceNumber=   mean_BufferSequenceNumber/nentries;
   mean_BufferStartTimeLR=   mean_BufferStartTimeLR/nentries;
   mean_BufferStartTimeHR=   mean_BufferStartTimeHR/nentries;
   mean_BufferEndTimeLR=   mean_BufferEndTimeLR/nentries;
   mean_BufferEndTimeHR=   mean_BufferEndTimeHR/nentries;
   mean_RampTimeAdc=   mean_RampTimeAdc/nentries;
   mean_Mcp1TimeAdc=   mean_Mcp1TimeAdc/nentries;
   mean_Mcp1PhAdc=   mean_Mcp1PhAdc/nentries;
   mean_Mcp2TimeAdc=   mean_Mcp2TimeAdc/nentries;
   mean_Mcp2PhAdc=   mean_Mcp2PhAdc/nentries;
   mean_Mcp3TimeAdc=   mean_Mcp3TimeAdc/nentries;
   mean_Mcp3PhAdc=   mean_Mcp3PhAdc/nentries;
   mean_Mcp4TimeAdc=   mean_Mcp4TimeAdc/nentries;
   mean_Mcp4PhAdc=   mean_Mcp4PhAdc/nentries;
   mean_Trig1TimeTdc=   mean_Trig1TimeTdc/nentries;
   mean_Trig2TimeTdc=   mean_Trig2TimeTdc/nentries;
   mean_CherenkovTimeTdc=   mean_CherenkovTimeTdc/nentries;

   rms_RunNumber=sqrt( (double)rms_RunNumber/nentries -mean_RunNumber*mean_RunNumber);
   rms_RunType=sqrt( (double)rms_RunType/nentries -mean_RunType*mean_RunType);
   rms_EventNumber=sqrt( (double)rms_EventNumber/nentries -mean_EventNumber*mean_EventNumber);
   rms_BufferSequenceNumber=sqrt( (double)rms_BufferSequenceNumber/nentries -mean_BufferSequenceNumber*mean_BufferSequenceNumber);
   rms_BufferStartTimeLR=sqrt( (double)rms_BufferStartTimeLR/nentries -mean_BufferStartTimeLR*mean_BufferStartTimeLR);
   rms_BufferStartTimeHR=sqrt( (double)rms_BufferStartTimeHR/nentries -mean_BufferStartTimeHR*mean_BufferStartTimeHR);
   rms_BufferEndTimeLR=sqrt( (double)rms_BufferEndTimeLR/nentries -mean_BufferEndTimeLR*mean_BufferEndTimeLR);
   rms_BufferEndTimeHR=sqrt( (double)rms_BufferEndTimeHR/nentries -mean_BufferEndTimeHR*mean_BufferEndTimeHR);
   rms_RampTimeAdc=sqrt( (double)rms_RampTimeAdc/nentries -mean_RampTimeAdc*mean_RampTimeAdc);
   rms_Mcp1TimeAdc=sqrt( (double)rms_Mcp1TimeAdc/nentries -mean_Mcp1TimeAdc*mean_Mcp1TimeAdc);
   rms_Mcp1PhAdc=sqrt( (double)rms_Mcp1PhAdc/nentries -mean_Mcp1PhAdc*mean_Mcp1PhAdc);
   rms_Mcp2TimeAdc=sqrt( (double)rms_Mcp2TimeAdc/nentries -mean_Mcp2TimeAdc*mean_Mcp2TimeAdc);
   rms_Mcp2PhAdc=sqrt( (double)rms_Mcp2PhAdc/nentries -mean_Mcp2PhAdc*mean_Mcp2PhAdc);
   rms_Mcp3TimeAdc=sqrt( (double)rms_Mcp3TimeAdc/nentries -mean_Mcp3TimeAdc*mean_Mcp3TimeAdc);
   rms_Mcp3PhAdc=sqrt( (double)rms_Mcp3PhAdc/nentries -mean_Mcp3PhAdc*mean_Mcp3PhAdc);
   rms_Mcp4TimeAdc=sqrt( (double)rms_Mcp4TimeAdc/nentries -mean_Mcp4TimeAdc*mean_Mcp4TimeAdc);
   rms_Mcp4PhAdc=sqrt( (double)rms_Mcp4PhAdc/nentries -mean_Mcp4PhAdc*mean_Mcp4PhAdc);
   rms_Trig1TimeTdc=sqrt( (double)rms_Trig1TimeTdc/nentries -mean_Trig1TimeTdc*mean_Trig1TimeTdc);
   rms_Trig2TimeTdc=sqrt( (double)rms_Trig2TimeTdc/nentries -mean_Trig2TimeTdc*mean_Trig2TimeTdc);
   rms_CherenkovTimeTdc=sqrt( (double)rms_CherenkovTimeTdc/nentries -mean_CherenkovTimeTdc*mean_CherenkovTimeTdc);
   


   //idem for computed data
   mean_Mcp12TimeAdc= mean_Mcp12TimeAdc/nentries;
   mean_Mcp34TimeAdc= mean_Mcp34TimeAdc/nentries;
   mean_McpTof= mean_McpTof/nentries;
   mean_IsMcp1Fired= mean_IsMcp1Fired/nentries;
   mean_IsMcp2Fired= mean_IsMcp2Fired/nentries;
   mean_IsMcp3Fired= mean_IsMcp3Fired/nentries;
   mean_IsMcp4Fired= mean_IsMcp4Fired/nentries;
   mean_NMcpFired= mean_NMcpFired/nentries;
   mean_RampTimeAdcDiff= mean_RampTimeAdcDiff/nentries;
   mean_McpTof_true= mean_McpTof_true/n_McpTof_true;

   rms_Mcp12TimeAdc=sqrt( (double)rms_Mcp12TimeAdc/nentries -mean_Mcp12TimeAdc*mean_Mcp12TimeAdc);
   rms_Mcp34TimeAdc=sqrt( (double)rms_Mcp34TimeAdc/nentries -mean_Mcp34TimeAdc*mean_Mcp34TimeAdc);
   rms_McpTof=sqrt( (double)rms_McpTof/nentries -mean_McpTof*mean_McpTof);
   rms_IsMcp1Fired=sqrt( (double)rms_IsMcp1Fired/nentries -mean_IsMcp1Fired*mean_IsMcp1Fired);
   rms_IsMcp2Fired=sqrt( (double)rms_IsMcp2Fired/nentries -mean_IsMcp2Fired*mean_IsMcp2Fired);
   rms_IsMcp3Fired=sqrt( (double)rms_IsMcp3Fired/nentries -mean_IsMcp3Fired*mean_IsMcp3Fired);
   rms_IsMcp4Fired=sqrt( (double)rms_IsMcp4Fired/nentries -mean_IsMcp4Fired*mean_IsMcp4Fired);
   rms_NMcpFired=sqrt( (double)rms_NMcpFired/nentries -mean_NMcpFired*mean_NMcpFired);
   rms_RampTimeAdcDiff=sqrt( (double)rms_RampTimeAdcDiff/nentries -mean_RampTimeAdcDiff*mean_RampTimeAdcDiff);
   rms_McpTof_true=sqrt( (double)rms_McpTof_true/n_McpTof_true -mean_McpTof_true*mean_McpTof_true);

   summary << "Summary of the run " << data_RunNumber << endl;
   if (data_RunNumber != RunNumber0) summary << "Warning, more than one run was analysed (first was " << RunNumber0 << "). Following statistics concern all data analyzed." << endl;
   summary << "Number of entries: " << nentries << endl;
   summary << endl << endl;
   summary << "Means and RMSs of data from input ntuple" << endl;
   summary << "mean_RunNumber = " << mean_RunNumber << " +/- " << mean_RunNumber/sqrt( (double)nentries) << endl;
   summary << "mean_RunType = " << mean_RunType << " +/- " << mean_RunType/sqrt( (double)nentries) << endl;
   summary << "mean_EventNumber = " << mean_EventNumber << " +/- " << mean_EventNumber/sqrt( (double)nentries) << endl;
   summary << "mean_BufferSequenceNumber = " << mean_BufferSequenceNumber << " +/- " << mean_BufferSequenceNumber/sqrt( (double)nentries) << endl;
   summary << "mean_BufferStartTimeLR = " << mean_BufferStartTimeLR << " +/- " << mean_BufferStartTimeLR/sqrt( (double)nentries) << endl;
   summary << "mean_BufferStartTimeHR = " << mean_BufferStartTimeHR << " +/- " << mean_BufferStartTimeLR/sqrt( (double)nentries) << endl;
   summary << "mean_BufferEndTimeLR = " << mean_BufferEndTimeLR << " +/- " << mean_BufferEndTimeLR/sqrt( (double)nentries) << endl;
   summary << "mean_BufferEndTimeHR = " << mean_BufferEndTimeHR << " +/- " << mean_BufferEndTimeHR/sqrt( (double)nentries) << endl;
   summary << "mean_RampTimeAdc = " << mean_RampTimeAdc << " +/- " << mean_RampTimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp1TimeAdc = " << mean_Mcp1TimeAdc << " +/- " << mean_Mcp1TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp1PhAdc = " << mean_Mcp1PhAdc << " +/- " << mean_Mcp1PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp2TimeAdc = " << mean_Mcp2TimeAdc << " +/- " << mean_Mcp2TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp2PhAdc = " << mean_Mcp2PhAdc << " +/- " << mean_Mcp2PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp3TimeAdc = " << mean_Mcp3TimeAdc << " +/- " << mean_Mcp3TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp3PhAdc = " << mean_Mcp3PhAdc << " +/- " << mean_Mcp3PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp4TimeAdc = " << mean_Mcp4TimeAdc << " +/- " << mean_Mcp4TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp4PhAdc = " << mean_Mcp4PhAdc << " +/- " << mean_Mcp4PhAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Trig1TimeTdc = " << mean_Trig1TimeTdc << " +/- " << mean_Trig1TimeTdc/sqrt( (double)nentries) << endl;
   summary << "mean_Trig2TimeTdc = " << mean_Trig2TimeTdc << " +/- " << mean_Trig2TimeTdc/sqrt( (double)nentries) << endl;
   summary << "mean_CherenkovTimeTdc = " << mean_CherenkovTimeTdc << " +/- " << mean_CherenkovTimeTdc/sqrt( (double)nentries) << endl;
   summary << "**************************************" << endl;
   summary << "rms_RunNumber = " << rms_RunNumber << " +/- " << rms_RunNumber/sqrt( (double)nentries) << endl;
   summary << "rms_RunType = " << rms_RunType << " +/- " << rms_RunType/sqrt( (double)nentries) << endl;
   summary << "rms_EventNumber = " << rms_EventNumber << " +/- " << rms_EventNumber/sqrt( (double)nentries) << endl;
   summary << "rms_BufferSequenceNumber = " << rms_BufferSequenceNumber << " +/- " << rms_BufferSequenceNumber/sqrt( (double)nentries) << endl;
   summary << "rms_BufferStartTimeLR = " << rms_BufferStartTimeLR << " +/- " << rms_BufferStartTimeLR/sqrt( (double)nentries) << endl;
   summary << "rms_BufferStartTimeHR = " << rms_BufferStartTimeHR << " +/- " << rms_BufferStartTimeLR/sqrt( (double)nentries) << endl;
   summary << "rms_BufferEndTimeLR = " << rms_BufferEndTimeLR << " +/- " << rms_BufferEndTimeLR/sqrt( (double)nentries) << endl;
   summary << "rms_BufferEndTimeHR = " << rms_BufferEndTimeHR << " +/- " << rms_BufferEndTimeHR/sqrt( (double)nentries) << endl;
   summary << "rms_RampTimeAdc = " << rms_RampTimeAdc << " +/- " << rms_RampTimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp1TimeAdc = " << rms_Mcp1TimeAdc << " +/- " << rms_Mcp1TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp1PhAdc = " << rms_Mcp1PhAdc << " +/- " << rms_Mcp1PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp2TimeAdc = " << rms_Mcp2TimeAdc << " +/- " << rms_Mcp2TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp2PhAdc = " << rms_Mcp2PhAdc << " +/- " << rms_Mcp2PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp3TimeAdc = " << rms_Mcp3TimeAdc << " +/- " << rms_Mcp3TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp3PhAdc = " << rms_Mcp3PhAdc << " +/- " << rms_Mcp3PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp4TimeAdc = " << rms_Mcp4TimeAdc << " +/- " << rms_Mcp4TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp4PhAdc = " << rms_Mcp4PhAdc << " +/- " << rms_Mcp4PhAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Trig1TimeTdc = " << rms_Trig1TimeTdc << " +/- " << rms_Trig1TimeTdc/sqrt( (double)nentries) << endl;
   summary << "rms_Trig2TimeTdc = " << rms_Trig2TimeTdc << " +/- " << rms_Trig2TimeTdc/sqrt( (double)nentries) << endl;
   summary << "rms_CherenkovTimeTdc = " << rms_CherenkovTimeTdc << " +/- " << rms_CherenkovTimeTdc/sqrt( (double)nentries) << endl;
   summary << endl << "**************************************" << endl << endl;
   summary << "Means and RMSs from computed data" << endl;
   summary << "mean_Mcp12TimeAdc = " << mean_Mcp12TimeAdc << " +/- " << mean_Mcp12TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_Mcp34TimeAdc = " << mean_Mcp34TimeAdc << " +/- " << mean_Mcp34TimeAdc/sqrt( (double)nentries) << endl;
   summary << "mean_McpTof = " << mean_McpTof << " +/- " << mean_McpTof/sqrt( (double)nentries) << endl;
   summary << "mean_McpTof_true = " << mean_McpTof_true << " +/- " << mean_McpTof_true/sqrt( (double)n_McpTof_true) << " counting " << n_McpTof_true << " entries with all 4 MCPs fired" << endl;
   summary << "mean_IsMcp1Fired = " << mean_IsMcp1Fired << " +/- " << mean_IsMcp1Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp2Fired = " << mean_IsMcp2Fired << " +/- " << mean_IsMcp2Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp3Fired = " << mean_IsMcp3Fired << " +/- " << mean_IsMcp3Fired/sqrt( (double)nentries) << endl;
   summary << "mean_IsMcp4Fired = " << mean_IsMcp4Fired << " +/- " << mean_IsMcp4Fired/sqrt( (double)nentries) << endl;
   summary << "mean_NMcpFired = " << mean_NMcpFired << " +/- " << mean_NMcpFired/sqrt( (double)nentries) << endl;
   summary << "mean_RampTimeAdcDiff = " << mean_RampTimeAdcDiff << " +/- " << mean_RampTimeAdcDiff/sqrt( (double)nentries) << endl;
   summary << "**************************************" << endl;
   summary << "rms_Mcp12TimeAdc = " << rms_Mcp12TimeAdc << " +/- " << rms_Mcp12TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_Mcp34TimeAdc = " << rms_Mcp34TimeAdc << " +/- " << rms_Mcp34TimeAdc/sqrt( (double)nentries) << endl;
   summary << "rms_McpTof = " << rms_McpTof << " +/- " << rms_McpTof/sqrt( (double)nentries) << endl;
   summary << "rms_McpTof_true = " << rms_McpTof_true << " +/- " << rms_McpTof_true/sqrt( (double)n_McpTof_true) << endl;
   summary << "rms_IsMcp1Fired = " << rms_IsMcp1Fired << " +/- " << rms_IsMcp1Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp2Fired = " << rms_IsMcp2Fired << " +/- " << rms_IsMcp2Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp3Fired = " << rms_IsMcp3Fired << " +/- " << rms_IsMcp3Fired/sqrt( (double)nentries) << endl;
   summary << "rms_IsMcp4Fired = " << rms_IsMcp4Fired << " +/- " << rms_IsMcp4Fired/sqrt( (double)nentries) << endl;
   summary << "rms_NMcpFired = " << rms_NMcpFired << " +/- " << rms_NMcpFired/sqrt( (double)nentries) << endl;
   summary << "rms_RampTimeAdcDiff = " << rms_RampTimeAdcDiff << " +/- " << rms_RampTimeAdcDiff/sqrt( (double)nentries) << endl;

/*   //Write histos
   cdFixedData->cd();
   RunNumberHisto->Write();
   RunTypeHisto->Write();
   EventNumberHisto->Write();
   BufferSequenceNumberHisto->Write();
   BufferStartTimeLRHisto->Write();
   BufferStartTimeHRHisto->Write();
   BufferEndTimeLRHisto->Write();
   BufferEndTimeHRHisto->Write();
   RampTimeAdcHisto->Write();
   Mcp1TimeAdcHisto->Write();
   Mcp1PhAdcHisto->Write();
   Mcp2TimeAdcHisto->Write();
   Mcp2PhAdcHisto->Write();
   Mcp3TimeAdcHisto->Write();
   Mcp3PhAdcHisto->Write();
   Mcp4TimeAdcHisto->Write();
   Mcp4PhAdcHisto->Write();
   Trig1TimeTdcHisto->Write();
   Trig2TimeTdcHisto->Write();
   CherenkovTimeTdcHisto->Write();

   cdFixedComputed->cd();
   Mcp12TimeAdcHisto->Write();
   Mcp34TimeAdcHisto->Write();
   McpTofHisto->Write();
   
   cdFixedOthers->cd();
   PhVsTimeAdc1Histo->Write();
   PhVsTimeAdc2Histo->Write();
   PhVsTimeAdc3Histo->Write();
   PhVsTimeAdc4Histo->Write();
   BufferSeqHisto->Write();
   BufferStartHRHisto->Write();
   BufferEndHRHisto->Write();
   BufferDelayHisto->Write();
   EventNumberVsTimeHisto->Write();
*/
   
   //Write histos into image files
   
   //create directories
   string OutImageStr = OutImage;
   string ImageName;
   string cmd = "mkdir -p " + OutImageStr + "FixedHistos/DataFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/ComputedFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "FixedHistos/OtherFixedHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/DataCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/ComputedCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "CustomHistos/OtherCustomHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/DataEventsHistos";
   system(cmd.c_str());
   cmd = "mkdir -p " + OutImageStr + "EventsHistos/ComputedEventsHistos";
   system(cmd.c_str());
   
   TCanvas *c1 = new TCanvas("c1","c1",1);
   
/*   //Fixed scale histos
   BufferSeqHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferSeqHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferStartHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferEndHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferDelayHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/BufferDelayHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberVsTimeHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/EventNumberVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
  */ 

   cout << "Now creating .gif image files for histograms" << endl;

   RunNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RunTypeHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumberHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeLRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferStartTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferStartTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeLRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferEndTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeHRHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/BufferEndTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/RampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig1TimeTdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Trig1TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig2TimeTdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/Trig2TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   CherenkovTimeTdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/DataFixedHistos/CherenkovTimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   
   PhVsTimeAdc1Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc1Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc2Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc2Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc3Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc3Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc4Histo->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdc4Histo.gif";
   c1->Print(ImageName.c_str());
   
   //PhVsTimeAdc1Histo->SetMarkerStyle(20);
   PhVsTimeAdc1Histo->SetMarkerColor(kGreen);
   PhVsTimeAdc1Histo->Draw();
   //PhVsTimeAdc2Histo->SetMarkerStyle(21);
   PhVsTimeAdc2Histo->SetMarkerColor(kBlue);
   PhVsTimeAdc2Histo->Draw("same");
   //PhVsTimeAdc3Histo->SetMarkerStyle(22);
   PhVsTimeAdc3Histo->SetMarkerColor(kCyan);
   PhVsTimeAdc3Histo->Draw("same");
   //PhVsTimeAdc4Histo->SetMarkerStyle(23);
   PhVsTimeAdc4Histo->SetMarkerColor(kRed);
   PhVsTimeAdc4Histo->Draw("same");
   PhVsTimeAdc1Histo->SetMarkerStyle(2);
   PhVsTimeAdc2Histo->SetMarkerStyle(2);
   PhVsTimeAdc3Histo->SetMarkerStyle(2);
   PhVsTimeAdc4Histo->SetMarkerStyle(2);
   legend->AddEntry(PhVsTimeAdc1Histo,"ADC1","p");
   legend->AddEntry(PhVsTimeAdc2Histo,"ADC2","p");
   legend->AddEntry(PhVsTimeAdc3Histo,"ADC3","p");
   legend->AddEntry(PhVsTimeAdc4Histo,"ADC4","p");
   legend->Draw();
   ImageName = OutImageStr + "FixedHistos/OtherFixedHistos/PhVsTimeAdcAll.gif";
   c1->Print(ImageName.c_str());

   Mcp12TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdcHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTofHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFiredHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdcDiffHisto->Draw();
   ImageName = OutImageStr + "FixedHistos/ComputedFixedHistos/RampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());
   
   
      //Custom scale histos
   
   RunNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RunType_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumber_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeLR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferStartTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeHR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferStartTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeLR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferEndTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeHR_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/BufferEndTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/RampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig1TimeTdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Trig1TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig2TimeTdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/Trig2TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   CherenkovTimeTdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/DataCustomHistos/CherenkovTimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   
   PhVsTimeAdc1_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc1Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc2_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc2Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc3_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc3Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc4_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/PhVsTimeAdc4Histo.gif";
   c1->Print(ImageName.c_str());
   RampVsTime_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/OtherCustomHistos/RampVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
   


   Mcp12TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdc_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTof_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFired_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdcDiff_CHisto->Draw();
   ImageName = OutImageStr + "CustomHistos/ComputedCustomHistos/RampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());


      //Vs EventNumber histos
   
   RunType_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   RunNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumber_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeLR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferStartTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeHR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferStartTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeLR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferEndTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeHR_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/BufferEndTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/RampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig1TimeTdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Trig1TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig2TimeTdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/Trig2TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   CherenkovTimeTdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/DataEventsHistos/CherenkovTimeTdcHisto.gif";
   c1->Print(ImageName.c_str());

   Mcp12TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/Mcp12TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp34TimeAdc_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/Mcp34TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   McpTof_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/McpTofHisto.gif";
   c1->Print(ImageName.c_str());
   NMcpFired_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/NMcpFiredHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdcDiff_VEHisto->Draw();
   ImageName = OutImageStr + "EventsHistos/ComputedEventsHistos/RampTimeAdcDiffHisto.gif";
   c1->Print(ImageName.c_str());

   
   FileOut->Write();
   FileOut->Close();
}
