#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "Riostream.h"
#define PSECNtupleAna_cxx
#include "PSECNtupleAna.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <string>
#include <math.h>

void PSECNtupleAna::Loop(const char* OutHistos, const char* OutSummary, const char* OutImage)
{
//   In a ROOT session, you can do:
//      Root > .L PSECNtupleAna.C
//      Root > PSECNtupleAna t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch

	//Declaration of means and RMSs
	double         mean_RunNumber=0.;
	double         mean_RunType=0.;
	double         mean_EventNumber=0.;
	double         mean_BufferSequenceNumber=0.;
	double         mean_BufferStartTimeLR=0.;
	double         mean_BufferStartTimeHR=0.;
	double         mean_BufferEndTimeLR=0.;
	double         mean_BufferEndTimeHR=0.;
	double         mean_RampTimeAdc=0.;
	double         mean_Mcp1TimeAdc=0.;
	double         mean_Mcp1PhAdc=0.;
	double         mean_Mcp2TimeAdc=0.;
	double         mean_Mcp2PhAdc=0.;
	double         mean_Mcp3TimeAdc=0.;
	double         mean_Mcp3PhAdc=0.;
	double         mean_Mcp4TimeAdc=0.;
	double         mean_Mcp4PhAdc=0.;
	double         mean_Trig1TimeTdc=0.;
	double         mean_Trig2TimeTdc=0.;
	double         mean_CherenkovTimeTdc=0.;
	double         rms_RunNumber=0.;
	double         rms_RunType=0.;
	double         rms_EventNumber=0.;
	double         rms_BufferSequenceNumber=0.;
	double         rms_BufferStartTimeLR=0.;
	double         rms_BufferStartTimeHR=0.;
	double         rms_BufferEndTimeLR=0.;
	double         rms_BufferEndTimeHR=0.;
	double         rms_RampTimeAdc=0.;
	double         rms_Mcp1TimeAdc=0.;
	double         rms_Mcp1PhAdc=0.;
	double         rms_Mcp2TimeAdc=0.;
	double         rms_Mcp2PhAdc=0.;
	double         rms_Mcp3TimeAdc=0.;
	double         rms_Mcp3PhAdc=0.;
	double         rms_Mcp4TimeAdc=0.;
	double         rms_Mcp4PhAdc=0.;
	double         rms_Trig1TimeTdc=0.;
	double         rms_Trig2TimeTdc=0.;
	double         rms_CherenkovTimeTdc=0.;

	TFile *FileOut = new TFile(OutHistos,"RECREATE");
	
	TLegend *legend=new TLegend(0.6,0.65,0.88,0.85);
	legend->SetTextFont(72);
	legend->SetTextSize(0.03);
	
	ofstream summary (OutSummary, ios::out);

	//TCanvas *canvas = new TCanvas("canvas","nearline analysis",200,10,600,400);
	//canvas->SetGrid();
	
   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntriesFast();
   
   TH2F *BufferSeqHisto = new TH2F("BufferSequenceN","BufferSequenceNumber vs EventNumber",100,0.,nentries+10.,100,0.,300.);
   TH2F *BufferStartHRHisto = new TH2F("BufferStartHR","BufferStartTimeHR vs BufferStartTimeLR",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferEndHRHisto = new TH2F("BufferEndHR","BufferEndTimeHR vs BufferEndTimeLR",100,1.2111586e9,1.2111589e9,100,0.,1e6);
   TH2F *BufferDelayHisto = new TH2F("BufferDelay","BufferDelay vs EventNumber",100,0.,nentries+10.,100,0.,250.);
   TH2F *EventNumberVsTimeHisto = new TH2F("EventNumberVsTime","EventNumber vs Start Time",100,1.2111586e9,1.2131589e9,100,0.,nentries+10.);
   
   //1D histograms of all branches
   
   TH1F *RunNumberHisto = new TH1F("RunNumber","RunNumber",100,0.,100.);
   TH1F *RunTypeHisto = new TH1F("RunType","RunType",100,0.,100.);
   TH1F *EventNumberHisto = new TH1F("EventNumber","EventNumber",1000,0.,100000.);
   TH1F *BufferSequenceNumberHisto = new TH1F("BufferSequenceNumber","BufferSequenceNumber",100,0.,1024.);
   TH1F *BufferStartTimeLRHisto = new TH1F("BufferStartTimeLR","BufferStartTimeLR",100,0.,2e9);
   TH1F *BufferStartTimeHRHisto = new TH1F("BufferStartTimeHR","BufferStartTimeHR",100,0.,1e6);
   TH1F *BufferEndTimeLRHisto = new TH1F("BufferEndTimeLR","BufferEndTimeLR",100,0.,2e9);
   TH1F *BufferEndTimeHRHisto = new TH1F("BufferEndTimeHR","BufferEndTimeHR",100,0.,1e6);
   TH1F *RampTimeAdcHisto = new TH1F("RampTimeAdc","RampTimeAdc",100,0.,1024.);
   TH1F *Mcp1TimeAdcHisto = new TH1F("Mcp1TimeAdc","Mcp1TimeAdc",100,0.,16384.);
   TH1F *Mcp1PhAdcHisto = new TH1F("Mcp1PhAdc","Mcp1PhAdc",100,0.,16384.);
   TH1F *Mcp2TimeAdcHisto = new TH1F("Mcp2TimeAdc","Mcp2TimeAdc",100,0.,16384.);
   TH1F *Mcp2PhAdcHisto = new TH1F("Mcp2PhAdc","Mcp2PhAdc",100,0.,16384.);
   TH1F *Mcp3TimeAdcHisto = new TH1F("Mcp3TimeAdc","Mcp3TimeAdc",100,0.,16384.);
   TH1F *Mcp3PhAdcHisto = new TH1F("Mcp3PhAdc","Mcp3PhAdc",100,0.,16384.);
   TH1F *Mcp4TimeAdcHisto = new TH1F("Mcp4TimeAdc","Mcp4TimeAdc",100,0.,16384.);
   TH1F *Mcp4PhAdcHisto = new TH1F("Mcp4PhAdc","Mcp4PhAdc",100,0.,16384.);
   TH1F *Trig1TimeTdcHisto = new TH1F("Trig1TimeTdc","Trig1TimeTdc",100,0.,1024.);
   TH1F *Trig2TimeTdcHisto = new TH1F("Trig2TimeTdc","Trig2TimeTdc",100,0.,1024.);
   TH1F *CherenkovTimeTdcHisto = new TH1F("CherenkovTimeTdc","CherenkovTimeTdc",100,0.,1024.);
   
   //2D histograms of charge vs time of flight by adc
   TH2F *PhVsTimeAdc1Histo = new TH2F("PhVsTimeAdc1","Ph Vs Time for ADC 1",100,0.,1024.,100,0.,1024.);
   TH2F *PhVsTimeAdc2Histo = new TH2F("PhVsTimeAdc2","Ph Vs Time for ADC 2",100,0.,1024.,100,0.,1024.);
   TH2F *PhVsTimeAdc3Histo = new TH2F("PhVsTimeAdc3","Ph Vs Time for ADC 3",100,0.,1024.,100,0.,1024.);
   TH2F *PhVsTimeAdc4Histo = new TH2F("PhVsTimeAdc4","Ph Vs Time for ADC 4",100,0.,1024.,100,0.,1024.);

   Long64_t nbytes = 0, nb = 0;
   
   float RunNumber0 = -999.;
   
   //LOOP
   for (Long64_t jentry=0; jentry<nentries;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
      
      if (RunNumber0 == -999.) RunNumber0 = data_RunNumber;
      
      //Calculate mean and rms of each leaf
      mean_RunNumber += data_RunNumber;
      mean_RunType += data_RunType;
      mean_EventNumber += data_EventNumber;
      mean_BufferSequenceNumber += data_BufferSequenceNumber;
      mean_BufferStartTimeLR += data_BufferStartTimeLR;
      mean_BufferStartTimeHR += data_BufferStartTimeHR;
      mean_BufferEndTimeLR += data_BufferEndTimeLR;
      mean_BufferEndTimeHR += data_BufferEndTimeHR;
      mean_RampTimeAdc += data_RampTimeAdc;
      mean_Mcp1TimeAdc += data_Mcp1TimeAdc;
      mean_Mcp1PhAdc += data_Mcp1PhAdc;
      mean_Mcp2TimeAdc += data_Mcp2TimeAdc;
      mean_Mcp2PhAdc += data_Mcp2PhAdc;
      mean_Mcp3TimeAdc += data_Mcp3TimeAdc;
      mean_Mcp3PhAdc += data_Mcp3PhAdc;
      mean_Mcp4TimeAdc += data_Mcp4TimeAdc;
      mean_Mcp4PhAdc += data_Mcp4PhAdc;
      mean_Trig1TimeTdc += data_Trig1TimeTdc;
      mean_Trig2TimeTdc += data_Trig2TimeTdc;
      mean_CherenkovTimeTdc += data_CherenkovTimeTdc;
       rms_RunNumber += data_RunNumber*data_RunNumber;
       rms_RunType += data_RunType*data_RunType;
       rms_EventNumber += data_EventNumber*data_EventNumber;
       rms_BufferSequenceNumber += data_BufferSequenceNumber*data_BufferSequenceNumber;
       rms_BufferStartTimeLR += data_BufferStartTimeLR*data_BufferStartTimeLR;
       rms_BufferStartTimeHR += data_BufferStartTimeHR*data_BufferStartTimeHR;
       rms_BufferEndTimeLR += data_BufferEndTimeLR*data_BufferEndTimeLR;
       rms_BufferEndTimeHR += data_BufferEndTimeHR*data_BufferEndTimeHR;
       rms_RampTimeAdc += data_RampTimeAdc*data_RampTimeAdc;
       rms_Mcp1TimeAdc += data_Mcp1TimeAdc*data_Mcp1TimeAdc;
       rms_Mcp1PhAdc += data_Mcp1PhAdc*data_Mcp1PhAdc;
       rms_Mcp2TimeAdc += data_Mcp2TimeAdc*data_Mcp2TimeAdc;
       rms_Mcp2PhAdc += data_Mcp2PhAdc*data_Mcp2PhAdc;
       rms_Mcp3TimeAdc += data_Mcp3TimeAdc*data_Mcp3TimeAdc;
       rms_Mcp3PhAdc += data_Mcp3PhAdc*data_Mcp3PhAdc;
       rms_Mcp4TimeAdc += data_Mcp4TimeAdc*data_Mcp4TimeAdc;
       rms_Mcp4PhAdc += data_Mcp4PhAdc*data_Mcp4PhAdc;
       rms_Trig1TimeTdc += data_Trig1TimeTdc*data_Trig1TimeTdc;
       rms_Trig2TimeTdc += data_Trig2TimeTdc*data_Trig2TimeTdc;
       rms_CherenkovTimeTdc += data_CherenkovTimeTdc*data_CherenkovTimeTdc;
      
      //Fill Histos
      BufferSeqHisto->Fill(data_EventNumber,data_BufferSequenceNumber);
      BufferStartHRHisto->Fill(data_BufferStartTimeLR,data_BufferStartTimeHR);
      BufferEndHRHisto->Fill(data_BufferEndTimeLR,data_BufferEndTimeHR);
      BufferDelayHisto->Fill(data_EventNumber,data_BufferEndTimeLR+data_BufferEndTimeHR-data_BufferStartTimeLR-data_BufferStartTimeHR);
      EventNumberVsTimeHisto->Fill(data_BufferStartTimeLR+data_BufferStartTimeHR,data_EventNumber);
      
      RunNumberHisto->Fill(data_RunNumber);
      RunTypeHisto->Fill(data_RunType);
      EventNumberHisto->Fill(data_EventNumber);
      BufferSequenceNumberHisto->Fill(data_BufferSequenceNumber);
      BufferStartTimeLRHisto->Fill(data_BufferStartTimeLR);
      BufferStartTimeHRHisto->Fill(data_BufferStartTimeHR);
      BufferEndTimeLRHisto->Fill(data_BufferEndTimeLR);
      BufferEndTimeHRHisto->Fill(data_BufferEndTimeHR);
      RampTimeAdcHisto->Fill(data_RampTimeAdc);
      Mcp1TimeAdcHisto->Fill(data_Mcp1TimeAdc);
      Mcp1PhAdcHisto->Fill(data_Mcp1PhAdc);
      Mcp2TimeAdcHisto->Fill(data_Mcp2TimeAdc);
      Mcp2PhAdcHisto->Fill(data_Mcp2PhAdc);
      Mcp3TimeAdcHisto->Fill(data_Mcp3TimeAdc);
      Mcp3PhAdcHisto->Fill(data_Mcp3PhAdc);
      Mcp4TimeAdcHisto->Fill(data_Mcp4TimeAdc);
      Mcp4PhAdcHisto->Fill(data_Mcp4PhAdc);
      Trig1TimeTdcHisto->Fill(data_Trig1TimeTdc);
      Trig2TimeTdcHisto->Fill(data_Trig2TimeTdc);
      CherenkovTimeTdcHisto->Fill(data_CherenkovTimeTdc);
      
      PhVsTimeAdc1Histo->Fill(data_Mcp1TimeAdc,data_Mcp1PhAdc);
      PhVsTimeAdc2Histo->Fill(data_Mcp2TimeAdc,data_Mcp2PhAdc);
      PhVsTimeAdc3Histo->Fill(data_Mcp3TimeAdc,data_Mcp3PhAdc);
      PhVsTimeAdc4Histo->Fill(data_Mcp4TimeAdc,data_Mcp4PhAdc);

   }
   
   //Means and RMSs
   mean_RunNumber=mean_RunNumber/nentries;
   mean_RunType=   mean_RunType/nentries;
   mean_EventNumber=   mean_EventNumber/nentries;
   mean_BufferSequenceNumber=   mean_BufferSequenceNumber/nentries;
   mean_BufferStartTimeLR=   mean_BufferStartTimeLR/nentries;
   mean_BufferStartTimeHR=   mean_BufferStartTimeHR/nentries;
   mean_BufferEndTimeLR=   mean_BufferEndTimeLR/nentries;
   mean_BufferEndTimeHR=   mean_BufferEndTimeHR/nentries;
   mean_RampTimeAdc=   mean_RampTimeAdc/nentries;
   mean_Mcp1TimeAdc=   mean_Mcp1TimeAdc/nentries;
   mean_Mcp1PhAdc=   mean_Mcp1PhAdc/nentries;
   mean_Mcp2TimeAdc=   mean_Mcp2TimeAdc/nentries;
   mean_Mcp2PhAdc=   mean_Mcp2PhAdc/nentries;
   mean_Mcp3TimeAdc=   mean_Mcp3TimeAdc/nentries;
   mean_Mcp3PhAdc=   mean_Mcp3PhAdc/nentries;
   mean_Mcp4TimeAdc=   mean_Mcp4TimeAdc/nentries;
   mean_Mcp4PhAdc=   mean_Mcp4PhAdc/nentries;
   mean_Trig1TimeTdc=   mean_Trig1TimeTdc/nentries;
   mean_Trig2TimeTdc=   mean_Trig2TimeTdc/nentries;
   mean_CherenkovTimeTdc=   mean_CherenkovTimeTdc/nentries;
   rms_RunNumber=sqrt(rms_RunNumber/nentries -mean_RunNumber*mean_RunNumber);
   rms_RunType=sqrt(rms_RunType/nentries -mean_RunType*mean_RunType);
   rms_EventNumber=sqrt(rms_EventNumber/nentries -mean_EventNumber*mean_EventNumber);
   rms_BufferSequenceNumber=sqrt(rms_BufferSequenceNumber/nentries -mean_BufferSequenceNumber*mean_BufferSequenceNumber);
   rms_BufferStartTimeLR=sqrt(rms_BufferStartTimeLR/nentries -mean_BufferStartTimeLR*mean_BufferStartTimeLR);
   rms_BufferStartTimeHR=sqrt(rms_BufferStartTimeHR/nentries -mean_BufferStartTimeHR*mean_BufferStartTimeHR);
   rms_BufferEndTimeLR=sqrt(rms_BufferEndTimeLR/nentries -mean_BufferEndTimeLR*mean_BufferEndTimeLR);
   rms_BufferEndTimeHR=sqrt(rms_BufferEndTimeHR/nentries -mean_BufferEndTimeHR*mean_BufferEndTimeHR);
   rms_RampTimeAdc=sqrt(rms_RampTimeAdc/nentries -mean_RampTimeAdc*mean_RampTimeAdc);
   rms_Mcp1TimeAdc=sqrt(rms_Mcp1TimeAdc/nentries -mean_Mcp1TimeAdc*mean_Mcp1TimeAdc);
   rms_Mcp1PhAdc=sqrt(rms_Mcp1PhAdc/nentries -mean_Mcp1PhAdc*mean_Mcp1PhAdc);
   rms_Mcp2TimeAdc=sqrt(rms_Mcp2TimeAdc/nentries -mean_Mcp2TimeAdc*mean_Mcp2TimeAdc);
   rms_Mcp2PhAdc=sqrt(rms_Mcp2PhAdc/nentries -mean_Mcp2PhAdc*mean_Mcp2PhAdc);
   rms_Mcp3TimeAdc=sqrt(rms_Mcp3TimeAdc/nentries -mean_Mcp3TimeAdc*mean_Mcp3TimeAdc);
   rms_Mcp3PhAdc=sqrt(rms_Mcp3PhAdc/nentries -mean_Mcp3PhAdc*mean_Mcp3PhAdc);
   rms_Mcp4TimeAdc=sqrt(rms_Mcp4TimeAdc/nentries -mean_Mcp4TimeAdc*mean_Mcp4TimeAdc);
   rms_Mcp4PhAdc=sqrt(rms_Mcp4PhAdc/nentries -mean_Mcp4PhAdc*mean_Mcp4PhAdc);
   rms_Trig1TimeTdc=sqrt(rms_Trig1TimeTdc/nentries -mean_Trig1TimeTdc*mean_Trig1TimeTdc);
   rms_Trig2TimeTdc=sqrt(rms_Trig2TimeTdc/nentries -mean_Trig2TimeTdc*mean_Trig2TimeTdc);
   rms_CherenkovTimeTdc=sqrt(rms_CherenkovTimeTdc/nentries -mean_CherenkovTimeTdc*mean_CherenkovTimeTdc);
   
   summary << "Summary of the run " << data_RunNumber << endl;
   if (data_RunNumber != RunNumber0) summary << "Warning, more than one run was analysed (first was " << RunNumber0 << "). Following statistics concern all data analyzed." << endl;
   summary << endl << endl;
   summary << "mean_RunNumber = " << mean_RunNumber << endl;
   summary << "mean_RunType = " << mean_RunType << endl;
   summary << "mean_EventNumber = " << mean_EventNumber << endl;
   summary << "mean_BufferSequenceNumber = " << mean_BufferSequenceNumber << endl;
   summary << "mean_BufferStartTimeLR = " << mean_BufferStartTimeLR << endl;
   summary << "mean_BufferStartTimeHR = " << mean_BufferStartTimeHR << endl;
   summary << "mean_BufferEndTimeLR = " << mean_BufferEndTimeLR << endl;
   summary << "mean_BufferEndTimeHR = " << mean_BufferEndTimeHR << endl;
   summary << "mean_RampTimeAdc = " << mean_RampTimeAdc << endl;
   summary << "mean_Mcp1TimeAdc = " << mean_Mcp1TimeAdc << endl;
   summary << "mean_Mcp1PhAdc = " << mean_Mcp1PhAdc << endl;
   summary << "mean_Mcp2TimeAdc = " << mean_Mcp2TimeAdc << endl;
   summary << "mean_Mcp2PhAdc = " << mean_Mcp2PhAdc << endl;
   summary << "mean_Mcp3TimeAdc = " << mean_Mcp3TimeAdc << endl;
   summary << "mean_Mcp3PhAdc = " << mean_Mcp3PhAdc << endl;
   summary << "mean_Mcp4TimeAdc = " << mean_Mcp4TimeAdc << endl;
   summary << "mean_Mcp4PhAdc = " << mean_Mcp4PhAdc << endl;
   summary << "mean_Trig1TimeTdc = " << mean_Trig1TimeTdc << endl;
   summary << "mean_Trig2TimeTdc = " << mean_Trig2TimeTdc << endl;
   summary << "mean_CherenkovTimeTdc = " << mean_CherenkovTimeTdc << endl;
   summary << "**************************************" << endl;
   summary << "rms_RunNumber = " << rms_RunNumber << endl;
   summary << "rms_RunType = " << rms_RunType << endl;
   summary << "rms_EventNumber = " << rms_EventNumber << endl;
   summary << "rms_BufferSequenceNumber = " << rms_BufferSequenceNumber << endl;
   summary << "rms_BufferStartTimeLR = " << rms_BufferStartTimeLR << endl;
   summary << "rms_BufferStartTimeHR = " << rms_BufferStartTimeHR << endl;
   summary << "rms_BufferEndTimeLR = " << rms_BufferEndTimeLR << endl;
   summary << "rms_BufferEndTimeHR = " << rms_BufferEndTimeHR << endl;
   summary << "rms_RampTimeAdc = " << rms_RampTimeAdc << endl;
   summary << "rms_Mcp1TimeAdc = " << rms_Mcp1TimeAdc << endl;
   summary << "rms_Mcp1PhAdc = " << rms_Mcp1PhAdc << endl;
   summary << "rms_Mcp2TimeAdc = " << rms_Mcp2TimeAdc << endl;
   summary << "rms_Mcp2PhAdc = " << rms_Mcp2PhAdc << endl;
   summary << "rms_Mcp3TimeAdc = " << rms_Mcp3TimeAdc << endl;
   summary << "rms_Mcp3PhAdc = " << rms_Mcp3PhAdc << endl;
   summary << "rms_Mcp4TimeAdc = " << rms_Mcp4TimeAdc << endl;
   summary << "rms_Mcp4PhAdc = " << rms_Mcp4PhAdc << endl;
   summary << "rms_Trig1TimeTdc = " << rms_Trig1TimeTdc << endl;
   summary << "rms_Trig2TimeTdc = " << rms_Trig2TimeTdc << endl;
   summary << "rms_CherenkovTimeTdc = " << rms_CherenkovTimeTdc << endl;

   BufferSeqHisto->Write();
   BufferStartHRHisto->Write();
   BufferEndHRHisto->Write();
   BufferDelayHisto->Write();
   EventNumberVsTimeHisto->Write();
   
   RunNumberHisto->Write();
   RunTypeHisto->Write();
   EventNumberHisto->Write();
   BufferSequenceNumberHisto->Write();
   BufferStartTimeLRHisto->Write();
   BufferStartTimeHRHisto->Write();
   BufferEndTimeLRHisto->Write();
   BufferEndTimeHRHisto->Write();
   RampTimeAdcHisto->Write();
   Mcp1TimeAdcHisto->Write();
   Mcp1PhAdcHisto->Write();
   Mcp2TimeAdcHisto->Write();
   Mcp2PhAdcHisto->Write();
   Mcp3TimeAdcHisto->Write();
   Mcp3PhAdcHisto->Write();
   Mcp4TimeAdcHisto->Write();
   Mcp4PhAdcHisto->Write();
   Trig1TimeTdcHisto->Write();
   Trig2TimeTdcHisto->Write();
   CherenkovTimeTdcHisto->Write();
   
   PhVsTimeAdc1Histo->Write();
   PhVsTimeAdc2Histo->Write();
   PhVsTimeAdc3Histo->Write();
   PhVsTimeAdc4Histo->Write();
   
   
   
   //Write histos into image files
   TCanvas *c1 = new TCanvas("c1","c1",1);
   string OutImageStr = OutImage;
   string ImageName;
   
   BufferSeqHisto->Draw();
   ImageName = OutImageStr + "BufferSeqHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartHRHisto->Draw();
   ImageName = OutImageStr + "BufferStartHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndHRHisto->Draw();
   ImageName = OutImageStr + "BufferEndHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferDelayHisto->Draw();
   ImageName = OutImageStr + "BufferDelayHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberVsTimeHisto->Draw();
   ImageName = OutImageStr + "EventNumberVsTimeHisto.gif";
   c1->Print(ImageName.c_str());
   
   RunNumberHisto->Draw();
   ImageName = OutImageStr + "RunNumberHisto.gif";
   c1->Print(ImageName.c_str());
   RunTypeHisto->Draw();
   ImageName = OutImageStr + "RunTypeHisto.gif";
   c1->Print(ImageName.c_str());
   EventNumberHisto->Draw();
   ImageName = OutImageStr + "EventNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferSequenceNumberHisto->Draw();
   ImageName = OutImageStr + "BufferSequenceNumberHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeLRHisto->Draw();
   ImageName = OutImageStr + "BufferStartTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferStartTimeHRHisto->Draw();
   ImageName = OutImageStr + "BufferStartTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeLRHisto->Draw();
   ImageName = OutImageStr + "BufferEndTimeLRHisto.gif";
   c1->Print(ImageName.c_str());
   BufferEndTimeHRHisto->Draw();
   ImageName = OutImageStr + "BufferEndTimeHRHisto.gif";
   c1->Print(ImageName.c_str());
   RampTimeAdcHisto->Draw();
   ImageName = OutImageStr + "RampTimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1TimeAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp1TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp1PhAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp1PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2TimeAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp2TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp2PhAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp2PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3TimeAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp3TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp3PhAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp3PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4TimeAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp4TimeAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Mcp4PhAdcHisto->Draw();
   ImageName = OutImageStr + "Mcp4PhAdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig1TimeTdcHisto->Draw();
   ImageName = OutImageStr + "Trig1TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   Trig2TimeTdcHisto->Draw();
   ImageName = OutImageStr + "Trig2TimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   CherenkovTimeTdcHisto->Draw();
   ImageName = OutImageStr + "CherenkovTimeTdcHisto.gif";
   c1->Print(ImageName.c_str());
   
   PhVsTimeAdc1Histo->Draw();
   ImageName = OutImageStr + "PhVsTimeAdc1Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc2Histo->Draw();
   ImageName = OutImageStr + "PhVsTimeAdc2Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc3Histo->Draw();
   ImageName = OutImageStr + "PhVsTimeAdc3Histo.gif";
   c1->Print(ImageName.c_str());
   PhVsTimeAdc4Histo->Draw();
   ImageName = OutImageStr + "PhVsTimeAdc4Histo.gif";
   c1->Print(ImageName.c_str());
   
   //PhVsTimeAdc1Histo->SetMarkerStyle(20);
   PhVsTimeAdc1Histo->SetMarkerColor(kGreen);
   PhVsTimeAdc1Histo->Draw();
   //PhVsTimeAdc2Histo->SetMarkerStyle(21);
   PhVsTimeAdc2Histo->SetMarkerColor(kBlue);
   PhVsTimeAdc2Histo->Draw("same");
   //PhVsTimeAdc3Histo->SetMarkerStyle(22);
   PhVsTimeAdc3Histo->SetMarkerColor(kCyan);
   PhVsTimeAdc3Histo->Draw("same");
   //PhVsTimeAdc4Histo->SetMarkerStyle(23);
   PhVsTimeAdc4Histo->SetMarkerColor(kRed);
   PhVsTimeAdc4Histo->Draw("same");
   PhVsTimeAdc1Histo->SetMarkerStyle(2);
   PhVsTimeAdc2Histo->SetMarkerStyle(2);
   PhVsTimeAdc3Histo->SetMarkerStyle(2);
   PhVsTimeAdc4Histo->SetMarkerStyle(2);
   legend->AddEntry(PhVsTimeAdc1Histo,"ADC1","p");
   legend->AddEntry(PhVsTimeAdc2Histo,"ADC2","p");
   legend->AddEntry(PhVsTimeAdc3Histo,"ADC3","p");
   legend->AddEntry(PhVsTimeAdc4Histo,"ADC4","p");
   legend->Draw();
   ImageName = OutImageStr + "PhVsTimeAdcAll.gif";
   c1->Print(ImageName.c_str());
   
   FileOut->Write();
   FileOut->Close();
}
