#include <iostream>
#include <iomanip>
//#include "histogram.h"
#include <string>
#include "TFile.h"
#include "TTree.h"
#include "TNtuple.h"
#include "Riostream.h"

//Pedestrals for Mcp ADCs
#define DEV1PHMIN 40
#define TAC1TIMMIN 1
#define DEV2PHMIN 40
#define TAC2TIMMIN 1
#define TAC3TIMMIN 1

#define TAB '\t'

//Purpose : This program reads an ascii file formatted as indicated in tupleFormat.txt
//	    and outputs the data in a root file.

//Modified May 14 2008 Emilien Chapon

//Modified May 22 2008 Emilien Chapon: support for new type of data

using namespace std;



TTree* convert(const char* infile, const char* outfile)	{

	ifstream in;
	in.open(infile);

	int tupleCount=0;

	//Following structure contains data read from ascii ntuple
	typedef struct {float RunNum, RunTyp, EventNum, BufferSequenceN, RealTimeLR, RTimeHR, LrRamp,
		HrRamp, TAC1Tim, TAC2Tim, TAC3Tim, Dev1Ph, Dev2Ph, Scaler1, Scaler2, 
		Scaler3;} DATA;

	//Following structure contains data that we compute from the variables of the ascii ntuple
	typedef struct {
		float 	   UniqueEventNum;	//In case of the analyse of several runs, gives a unique number to each event
		float	   LrRampDiff;		//Difference between low res ramp time adc counts for two successive events
		float	   HrRampDiff;		//Difference between hi res ramp time adc counts for two successive events
		float	   TAC23Tim;	//Difference between TAC ADC counts for TAC 2 and 3
		float	   IsTAC1Fired;		//1 if TAC1 was fired, 0 else
		float	   IsTAC2Fired;		//1 if TAC2 was fired, 0 else
		float	   IsTAC3Fired;		//1 if TAC3 was fired, 0 else
		float	   NTACFired;		//Number of TAC fired
	} COMPUTED;

	static DATA data;
	static COMPUTED computed;
	
	TFile *f = new TFile(outfile,"RECREATE");

	TTree *ntuple = new TTree("ntuple","root tree from ascii file");
	ntuple->Branch("data",&data.RunNum,"RunNum:RunTyp:EventNum:BufferSequenceN:RealTimeLR:RTimeHR:LrRamp:HrRamp:TAC1Tim:TAC2Tim:TAC3Tim:Dev1Ph:Dev2Ph:Scaler1:Scaler2:Scaler3");
	ntuple->Branch("computed",&computed.UniqueEventNum,"UniqueEventNum:LrRampDiff:HrRampDiff:TAC23Tim:IsTAC1Fired:IsTAC2Fired:IsTAC3Fired:NTACFired");

//We do not take into account the first line, which contains the names of the fields
string null;
in >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null >> null;

//Ask if we want to analyse the full ntuple, or only one part (for online verification of data quality)
cout << "Do you want to analyse full input ntuple (y/n) ?" << endl;
string answer;
bool fullAnalyse;
int firstLine,lastLine;
cin >> answer;
if (answer == "n") {
	fullAnalyse=false;
	cout << "First line ?" << endl;
	cin >> firstLine;
	cout << "Last line (0=last) ? You can use \"wc -l ntuple.ascii\" in a prompt to count lines." << endl;
	cin >> lastLine;
}
else {
	fullAnalyse=true;
	firstLine=1;
	lastLine=0;
}

	float LrRampPrev=0.;	//will be useful to calculate LrRampDiff
	float HrRampPrev=0.;	//will be useful to calculate HrRampDiff

   //Beginning of forever loop
   cout << "Now reading the ascii ntuple" << endl;
   while(1) {
	//Check if we are within the range of lines entered by user, in case of partial analyse
	tupleCount++;
	if (lastLine !=0 && tupleCount > lastLine) break;

	//Read data from ascii file
	in >> data.RunNum >> data.RunTyp >> data.EventNum >> data.BufferSequenceN >> data.RealTimeLR >>
	   data.RTimeHR >> data.LrRamp >> data.HrRamp >> data.TAC1Tim >> data.TAC2Tim >> 
	   data.TAC3Tim >> data.Dev1Ph >> data.Dev2Ph >> data.Scaler1 >> data.Scaler2 >> data.Scaler3;
	

	if (!in.good()) break;

	//Calculate computed variables
	computed.UniqueEventNum = tupleCount;
	computed.LrRampDiff=data.LrRamp-LrRampPrev;
	computed.HrRampDiff=data.HrRamp-HrRampPrev;
	LrRampPrev=data.LrRamp;
	HrRampPrev=data.HrRamp;
	computed.IsTAC1Fired = data.TAC1Tim >= TAC1TIMMIN ? 1. : 0. ;
	computed.IsTAC2Fired = data.TAC2Tim >= TAC2TIMMIN ? 1. : 0. ;
	computed.IsTAC3Fired = data.TAC3Tim >= TAC3TIMMIN ? 1. : 0. ;
	computed.NTACFired = computed.IsTAC1Fired + computed.IsTAC2Fired + computed.IsTAC3Fired;
	computed.TAC23Tim= (computed.IsTAC2Fired == 1. && computed.IsTAC3Fired == 1.) ? data.TAC3Tim-data.TAC2Tim : 0.;


	if (tupleCount >= firstLine) ntuple->Fill();
/*	if (tupleCount < 5) {
	   cout << " " << data.RunNum << " " << data.RunTyp << " " << data.EventNum << " " << data.BufferSequenceN << " " << data.RealTimeLR << " " << data.RTimeHR << " " << data.LrRamp << " " << data.HrRamp << " " << data.TAC1Tim << " " << data.TAC2Tim << " " << data.TAC3Tim << " " << data.Dev1Ph << " " << data.Dev2Ph << " " << data.Scaler1 << " " << data.Scaler2 << " " << data.Scaler3 << " " << data.nScaler1 << " " << data.nScaler2 << " " << data.nScaler3 << " " << data.CherenkovLatchADC << endl; 
	}
*/

   } // end of forever loop

	if (fullAnalyse) cout << "Found " << tupleCount  << " lines."  << endl;
	else cout << "Read " << tupleCount - firstLine << " lines." << endl;

	in.close();
	f->Write();
//	f->Close();
	return ntuple;
}
