#include "Riostream.h"
#include "math.h"

/* This program consists of two parts. Part one (treeFill) creates the histograms and part two (treeHist) draws them. 
   In part one the xlow, xhigh, bins, and nuevents parameters will need to be set. 
   In part two the fitmin and fitmax parameters will need to be set.   
   
   To run the program
   root [0] .x tree.C 

   To run part one by itself 
   root [0] .L tree.C 
   root [1] treeFill()

   To run part two by itself
   root [0] .L tree.C
   root [1] treeHist()

*/


void treeFill()
{
  gROOT->Reset();                       // Reset ROOT

  ifstream in;

  Float_t EVENT,ADC0,ADC1,ADC2,TAC;
  char line[256];

  // Open data file
  in.open("run405.txt"); 

  int numevents = 10000;                                // Number of events
  int tacmean = 3640;                           // Mean value of timing resolution
  int xprojmean = 1385;                         // Mean value of x projection
  float p0 = 3673.0;                            // Fit constant
  float p1 = -1.152;                            // Fit constant
  float p2 = 0.01202;                           // Fit constant
  float p3 = -1.677e-5;                         // Fit constant
  float taccorrected;                           // Corrected timing resolution

  int h1xlow  = 3500;                           // Min histogram value to be graphed
  int h1xhigh = 3800;                           // Max histogram value to be graphed
  int h1xbins = (h1xhigh - h1xlow) / 1; // Number of histogram bins 

  int h2xlow  = 0;                              // Min histogram value to be graphed
  int h2xhigh = 400;                            // Max histogram value to be graphed
  int h2xbins = (h2xhigh - h2xlow) / 1; // Number of histogram bins 

  int h3xlow  = 0;                              // Min histogram value to be graphed
  int h3xhigh = 400;                    // Max histogram value to be graphed
  int h3xbins  = (h3xhigh - h3xlow);                            // Min histogram value to be graphed

  int h4xlow  = 0;                              // Min histogram value to be graphed
  int h4xhigh = 10000;                  // Max histogram value to be graphed
  int h4ylow  = h1xlow;                         // Min histogram value to be graphed
  int h4yhigh = h1xhigh;                        // Max histogram value to be graphed
  int h4xbins = (h4xhigh - h4xlow);                     // Number of histogram bins 
  int h4ybins = h1xbins;                        // Number of histogram bins 

  int h5xlow  = h2xlow;                         // Min histogram value to be graphed
  int h5xhigh = h2xhigh;                        // Max histogram value to be graphed
  int h5ylow  = h1xlow;                         // Min histogram value to be graphed
  int h5yhigh = h1xhigh;                        // Max histogram value to be graphed
  int h5xbins = h2xbins;                        // Number of histogram bins 
  int h5ybins = h1xbins;                        // Number of histogram bins 

  int h6xlow  = h3xlow;                         // Min histogram value to be graphed
  int h6xhigh = h3xhigh;                        // Max histogram value to be graphed
  int h6ylow  = h1xlow;                         // Min histogram value to be graphed
  int h6yhigh = h1xhigh;                        // Max histogram value to be graphed
  int h6xbins = h3xbins;                        // Number of histogram bins 
  int h6ybins = h1xbins;                        // Number of histogram bins 

  int h7xlow  = h2xlow;                         // Min histogram value to be graphed
  int h7xhigh = h2xhigh;                        // Max histogram value to be graphed
  int h7ylow  = h1xlow;                         // Min histogram value to be graphed
  int h7yhigh = h1xhigh;                        // Max histogram value to be graphed
  int h7xbins = h2xbins;                        // Number of histogram bins 
  int h7ybins = h1xbins;                        // Number of histogram bins 

  int h8xlow  = h3xlow;                         // Min histogram value to be graphed
  int h8xhigh = h3xhigh;                        // Max histogram value to be graphed
  int h8ylow  = h1xlow;                         // Min histogram value to be graphed
  int h8yhigh = h1xhigh;                        // Max histogram value to be graphed
  int h8xbins = h3xbins;                        // Number of histogram bins 
  int h8ybins = h1xbins;                        // Number of histogram bins 

  int h9xlow  = h1xlow;                         // Min histogram value to be graphed
  int h9xhigh = h1xhigh;                        // Max histogram value to be graphed
  int h9xbins = h1xbins;                        // Number of histogram bins 


  // Create a root file
  TFile *f = new TFile("test.root","RECREATE"); 

  // Create a 1D Histogram for the TAC data
  TH1F *h1 = new TH1F("h1","Timing Resolution RAW [TAC]",h1xbins,h1xlow,h1xhigh);

  // Create a 1D Histogram for the TAC data
  TH1F *h2 = new TH1F("h2","ADC0 [TOF1]",h2xbins,h2xlow,h2xhigh);

  // Create a 2D Histogram for the Drift plot
  TH1F *h3 = new TH1F("h3","ADC1 [TOF2]",h3xbins,h3xlow,h3xhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h4 = new TH2F("h4","TAC vs. EVENTS",h4xbins,h4xlow,h4xhigh,h4ybins,h4ylow,h4yhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h5 = new TH2F("h5","TAC vs. ADC0",h5xbins,h5xlow,h5xhigh,h5ybins,h5ylow,h5yhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h6 = new TH2F("h6","TAC vs. ADC1",h6xbins,h6xlow,h6xhigh,h6ybins,h6ylow,h6yhigh);

  // Create a TProfile plot of TAC vs ADC0
  TProfile *h1prof = new TProfile("h1prof","TAC vs. ADC0 Profile",h2xbins,h2xlow,h2xhigh);

  // Create a TProfile plot of TAC vs ADC1
  TProfile *h2prof = new TProfile("h2prof","TAC vs. ADC1 Profile",h3xbins,h3xlow,h3xhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h7 = new TH2F("h7","Corrected TAC vs. ADC0",h7xbins,h7xlow,h7xhigh,h7ybins,h7ylow,h7yhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH2F *h8 = new TH2F("h8","Corrected TAC vs. ADC1",h8xbins,h8xlow,h8xhigh,h8ybins,h8ylow,h8yhigh);

  // Create a TProfile plot of TAC vs ADC0
  TProfile *h3prof = new TProfile("h3prof","Corrected TAC vs. ADC0 Profile",h2xbins,h2xlow,h2xhigh);

  // Create a TProfile plot of TAC vs ADC1
  TProfile *h4prof = new TProfile("h4prof","Corrected TAC vs. ADC1 Profile",h3xbins,h3xlow,h3xhigh);

  // Create a 2D Histogram for the Correlation Plot
  TH1F *h9 = new TH1F("h9","Corrected TAC",h9xbins,h9xlow,h9xhigh);



  // Create an Ntuple
  // r1ch1: row 1 channel 1 in your data file.
  TNtuple *ntuple = new TNtuple("ntuple","MCP Data","EVENT:ADC0:ADC1:ADC2:TAC");

  // Get the first line w/ run information
  in.getline (line,256);

  // Loop over events
  int i = 1;   
  while (1)
    {
      // Row 1
      in >> EVENT >> ADC0 >> ADC1 >> ADC2 >> TAC;
      if (!in.good()) break;
      h1->Fill(TAC);
      h2->Fill(ADC0);
          h3->Fill(ADC1);
          h4->Fill(EVENT,TAC);
          h5->Fill(ADC0,TAC);
          h6->Fill(ADC1,TAC);
          h1prof->Fill(ADC0,TAC);
          h2prof->Fill(ADC1,TAC);
      taccorrected = tacmean + (TAC - ((p3 * ADC0 * ADC0 * ADC0) + (p2 * ADC0 * ADC0) + (p1 * ADC0) + p0));
      //taccorrected = tacmean + (taccorrected - ((p2 * ADC1 * ADC1) + (p1 * ADC1) + p0));
          h7->Fill(ADC0,taccorrected);
          h8->Fill(ADC1,taccorrected);
          h3prof->Fill(ADC0,taccorrected);
          h4prof->Fill(ADC1,taccorrected);
          h9->Fill(taccorrected);

      ntuple->Fill(EVENT,TAC,ADC0,ADC1,ADC2);

     
      i++;
    }

  printf( "Found %i points\n",i);

  in.close(); // Close the data file

  f->Write(); // Write the root file
  //f->Close(); // Close the root file

}

void treeCorrected()
{
  extern TStyle* gStyle;
  
  double par[10];
  
  TFile *f = new TFile("test.root");                    // Open the root file
  TAC = (TH1F*)f->Get("h1;1");                          // Get the 1D histogram
  ADC0 = (TH1F*)f->Get("h2;1");                          // Get the 1D histogram
  ADC1 = (TH1F*)f->Get("h3;1");                          // Get the 1D histogram
  TACvsEVENT = (TH2F*)f->Get("h4;1");                       // Get the 2D histogram
  TACvsADC0 = (TH2F*)f->Get("h5;1");                       // Get the 2D histogram
  TACvsADC1 = (TH2F*)f->Get("h6;1");                       // Get the 2D histogram
  PROFILE0 = (TProfile*)f->Get("h1prof;1");
  PROFILE1 = (TProfile*)f->Get("h2prof;1");
  corTACvsADC0 = (TH2F*)f->Get("h7;1");                       // Get the 2D histogram
  corTACvsADC1 = (TH2F*)f->Get("h8;1");                       // Get the 2D histogram
  corPROFILE0 = (TProfile*)f->Get("h3prof;1");
  corPROFILE1 = (TProfile*)f->Get("h4prof;1");
  corTAC = (TH1F*)f->Get("h9;1");                          // Get the 1D histogram

  //**************************
  //  ROOT Plotting Options
  //**************************
  // Set histogram plotting options
  gStyle->SetOptStat(2211);
  gStyle->SetOptFit(11);
  gStyle->SetFuncColor( 1 );
  gStyle->SetFrameFillColor( 10 );
  //gStyle->SetHistFillColor( 3 );
  gStyle->SetLabelSize( 0.03, "XYZ" );
  gStyle->SetStatColor( 0 );
  gStyle->SetStatFontSize( 0.03 );
  gStyle->SetTitleColor( 2 );
  gStyle->SetTitleW(0.5);
  //gStyle->SetMarkerColor( 6 );
  //gStyle->SetMarkerSize( 0.25 );
  //gStyle->SetMarkerStyle( 8 );
  gStyle->SetPaperSize(20,20);
  

  TCanvas *MyC1 = new TCanvas("MyC1","Canvas 1",1200,800);   // Create a canvas for our graphs
  MyC1->Divide(2,2);                                     // Divide the canvas into two pads

  MyC1->cd(1);               // pad 1
  TAC->SetXTitle( "TOF1 - TOF2");
  TAC->GetXaxis()->SetTitleColor( 2 );
  TAC->SetYTitle( "Events");
  TAC->GetYaxis()->SetTitleColor( 2 );
  TAC->GetYaxis()->SetTitleOffset( 1.2 );

  int fitmin1 = 3610;
  int fitmax1 = 3660;
  TF1* myfit = new TF1("myfit","gaus",fitmin1,fitmax1);
  myfit->SetParName(0,"Norm 1");
  myfit->SetParName(1,"Mean 1");
  myfit->SetParName(2,"Sigma 1");
  TAC->Fit("myfit","R");  // use this to fit over a range
  par[0] = myfit->GetParameter(0);
  par[1] = myfit->GetParameter(1);
  par[2] = myfit->GetParameter(2);

  int fitmin2 = 3500;
  int fitmax2 = 3800;
  TF1* myfit2 = new TF1("myfit2","gaus",fitmin2,fitmax2);
  myfit2->SetParName(0,"Norm 2");
  myfit2->SetParName(1,"Mean 2");
  myfit2->SetParName(2,"Sigma 2");
  //  myfit2->SetParName(3,"p3");
  TAC->Fit("myfit2","R");  // use this to fit over a range
  par[3] = myfit2->GetParameter(0);
  par[4] = myfit2->GetParameter(1);
  par[5] = myfit2->GetParameter(2);
  //  par[6] = myfit2->GetParameter(3);

  int fitmin3 = 3500;
  int fitmax3 = 3800;
  TF1* myfit3 = new TF1("myfit3","gaus(0)+gaus(3)",fitmin3,fitmax3);
  myfit3->SetParName(0,"Norm 1");
  myfit3->SetParName(1,"Mean 1");
  myfit3->SetParName(2,"Sigma 1");
  myfit3->SetParName(3,"Norm 2");
  myfit3->SetParName(4,"Mean 2");
  myfit3->SetParName(5,"Sigma 2");
  //  myfit3->SetParName(6,"p3");
  myfit3->SetParameter(0,par[0]);
  myfit3->SetParameter(1,par[1]);
  myfit3->SetParameter(2,par[2]);
  myfit3->SetParameter(3,par[3]);
  myfit3->SetParameter(4,par[4]);
  myfit3->SetParameter(5,par[5]);
  //  myfit3->SetParameter(6,par[6]);
  TAC->Draw();              // Draw the 1D histogram
  TAC->Fit("myfit3","R");  // use this to fit over a range

  MyC1->cd(2);               // pad 2
  ADC0->SetXTitle( "TOF1 ADC (ch0)");
  ADC0->GetXaxis()->SetTitleColor( 2 );
  ADC0->SetYTitle( "Events");
  ADC0->GetYaxis()->SetTitleColor( 2 );
  ADC0->GetYaxis()->SetTitleOffset( 1.2 );
  ADC0->Draw();              // Draw the 1D histogram

  MyC1->cd(3);               // pad 3
  ADC1->SetXTitle( "TOF2 ADC (ch1)");
  ADC1->GetXaxis()->SetTitleColor( 2 );
  ADC1->SetYTitle( "Events");
  ADC1->GetYaxis()->SetTitleColor( 2 );
  ADC1->GetYaxis()->SetTitleOffset( 1.2 );
  ADC1->Draw();              // Draw the 1D histogram

  MyC1->cd(4);               // pad 4
  TACvsEVENT->SetStats(kFALSE);
  TACvsEVENT->SetXTitle( "Events");
  TACvsEVENT->GetXaxis()->SetTitleColor( 2 );
  TACvsEVENT->SetYTitle( "TOF1 - TOF2");
  TACvsEVENT->GetYaxis()->SetTitleColor( 2 );
  TACvsEVENT->GetYaxis()->SetTitleOffset( 1.2 );
  TACvsEVENT->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC2 = new TCanvas("MyC2","Canvas 2",300,10,1200,800);   // Create a canvas for our graphs
  MyC2->Divide(2,2);                                     // Divide the canvas into two pads

  MyC2->cd(1);               // pad 2
  TACvsADC0->SetXTitle( "ADC (ch.0)");
  TACvsADC0->GetXaxis()->SetTitleColor( 2 );
  TACvsADC0->SetYTitle( "TOF1 - TOF2");
  TACvsADC0->GetYaxis()->SetTitleColor( 2 );
  TACvsADC0->GetYaxis()->SetTitleOffset( 1.2 );
  TACvsADC0->Draw();

  MyC2->cd(2);               // pad 2
  TACvsADC1->SetXTitle( "ADC (ch.1)");
  TACvsADC1->GetXaxis()->SetTitleColor( 2 );
  TACvsADC1->SetYTitle( "TOF1 - TOF2");
  TACvsADC1->GetYaxis()->SetTitleColor( 2 );
  TACvsADC1->GetYaxis()->SetTitleOffset( 1.2 );
  TACvsADC1->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  MyC2->cd(3);               // pad 2
  PROFILE0->SetXTitle( "ADC (ch.0)");
  PROFILE0->GetXaxis()->SetTitleColor( 2 );
  PROFILE0->SetYTitle( "TOF1 - TOF2");
  PROFILE0->GetYaxis()->SetTitleColor( 2 );
  PROFILE0->GetYaxis()->SetTitleOffset( 1.2 );
  PROFILE0->Draw();

  MyC2->cd(4);               // pad 2
  PROFILE1->SetXTitle( "ADC (ch.1)");
  PROFILE1->GetXaxis()->SetTitleColor( 2 );
  PROFILE1->SetYTitle( "TOF1 - TOF2");
  PROFILE1->GetYaxis()->SetTitleColor( 2 );
  PROFILE1->GetYaxis()->SetTitleOffset( 1.2 );
  PROFILE1->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC3 = new TCanvas("MyC3","Canvas 3",300,10,1200,800);   // Create a canvas for our graphs
  MyC3->Divide(2,2);                                     // Divide the canvas into two pads

  MyC3->cd(1);               // pad 2
  corTACvsADC0->SetXTitle( "ADC (ch.0)");
  corTACvsADC0->GetXaxis()->SetTitleColor( 2 );
  corTACvsADC0->SetYTitle( "TOF1 - TOF2");
  corTACvsADC0->GetYaxis()->SetTitleColor( 2 );
  corTACvsADC0->GetYaxis()->SetTitleOffset( 1.2 );
  corTACvsADC0->Draw();

  MyC3->cd(2);               // pad 2
  corTACvsADC1->SetXTitle( "ADC (ch.1)");
  corTACvsADC1->GetXaxis()->SetTitleColor( 2 );
  corTACvsADC1->SetYTitle( "TOF1 - TOF2");
  corTACvsADC1->GetYaxis()->SetTitleColor( 2 );
  corTACvsADC1->GetYaxis()->SetTitleOffset( 1.2 );
  corTACvsADC1->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  MyC3->cd(3);               // pad 2
  corPROFILE0->SetXTitle( "ADC (ch.0)");
  corPROFILE0->GetXaxis()->SetTitleColor( 2 );
  corPROFILE0->SetYTitle( "TOF1 - TOF2");
  corPROFILE0->GetYaxis()->SetTitleColor( 2 );
  corPROFILE0->GetYaxis()->SetTitleOffset( 1.2 );
  corPROFILE0->Draw();

  MyC3->cd(4);               // pad 2
  corPROFILE1->SetXTitle( "ADC (ch.1)");
  corPROFILE1->GetXaxis()->SetTitleColor( 2 );
  corPROFILE1->SetYTitle( "TOF1 - TOF2");
  corPROFILE1->GetYaxis()->SetTitleColor( 2 );
  corPROFILE1->GetYaxis()->SetTitleOffset( 1.2 );
  corPROFILE1->Draw("SCAT");     // Draw the 2D histogram as a scatter plot

  TCanvas *MyC4 = new TCanvas("MyC4","Canvas 4",1200,800);   // Create a canvas for our graphs
//  MyC1->Divide(2,2);                                     // Divide the canvas into two pads

//  MyC1->cd(1);               // pad 1
  corTAC->SetXTitle( "TOF1 - TOF2");
  corTAC->GetXaxis()->SetTitleColor( 2 );
  corTAC->SetYTitle( "Events");
  corTAC->GetYaxis()->SetTitleColor( 2 );
  corTAC->GetYaxis()->SetTitleOffset( 1.2 );

  int fitmin4 = 3610;
  int fitmax4 = 3660;
  TF1* myfit4 = new TF1("myfit4","gaus",fitmin4,fitmax4);
  myfit4->SetParName(0,"Norm");
  myfit4->SetParName(1,"Mean");
  myfit4->SetParName(2,"Sigma");
  corTAC->Draw();              // Draw the 1D histogram
  corTAC->Fit("myfit4","R");  // use this to fit over a range and comment out previous
}


void tree()
{
  treeFill();
  treeCorrected();
}

